"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var campaign_model_1 = require("../../../models/campaign.model");
var log4js = require("log4js");
var Knex = require("knex");
var knex_config_1 = require("../knex.config");
var disposition_code_model_1 = require("../../../models/disposition-code.model");
var logger = log4js.getLogger('ServerScripts');
var inconcertDB = Knex(knex_config_1.connections.inconcert);
var campaignTable = inconcertDB.raw('Campaign WITH (NOLOCK)');
var agentTable = inconcertDB.raw('Agent AS a WITH (NOLOCK)');
var userCampaignTable = inconcertDB.raw('UserCampaign AS u WITH (NOLOCK)');
var campaignPopupTable = inconcertDB.raw('CampaignPopup WITH (NOLOCK)');
var campaignDispositionCodes = inconcertDB.raw('CampaignGoalResultManagement WITH (NOLOCK)');
function getCampaigns(vcc, campaigns) {
    if (campaigns === void 0) { campaigns = null; }
    return new Promise(function (resolve, reject) {
        var builder = inconcertDB.select(['Id', 'Description']).from(campaignTable).where({
            VCC: vcc
        });
        if (campaigns && campaigns.length)
            builder = builder.whereIn('Campaign', campaigns);
        builder.then(function (result) {
            var c = [];
            if (result) {
                result.forEach(function (r) {
                    c.push(new campaign_model_1.Campaign(r.Id, r.Id));
                });
            }
            resolve(c);
        }, function (error) {
            reject(error);
        });
    });
}
exports.getCampaigns = getCampaigns;
function getCampaignWebdesignerURL(vcc, campaignId, interactionType) {
    return new Promise(function (resolve, reject) {
        var builder = inconcertDB.select('PopupData').from(campaignPopupTable).where({
            VCC: vcc,
            PopupType: 'WEBDESIGNER',
            CampaignId: campaignId
        }).whereIn('InteractionType', [interactionType, 'ALL']);
        builder.then(function (result) {
            var url = '';
            if (result && result.length > 0) {
                url = result[0].PopupData;
            }
            resolve(url);
        }, function (error) {
            reject(error);
        });
    });
}
exports.getCampaignWebdesignerURL = getCampaignWebdesignerURL;
function getDispositionCodes(vcc, campaigns) {
    return new Promise(function (resolve, reject) {
        logger.debug('[MMProdat::getDispositionCodes] Vcc received: ' + vcc);
        logger.debug('[MMProdat::getDispositionCodes] Campaigns received: ' + JSON.stringify(campaigns));
        var builder = inconcertDB.select(['CampaignId', 'Code', 'Description', 'Category1', 'Category2', 'Category3']).from(campaignDispositionCodes).where({
            VCC: vcc
        });
        if (campaigns && campaigns.length > 0)
            builder.whereIn('CampaignId', campaigns);
        builder.then(function (result) {
            var dispositionCodes = [];
            logger.debug('[MMProdat::getDispositionCodes] Received result: ' + JSON.stringify(result));
            result.forEach(function (row) {
                var treepath = (row.Category1 ? row.Category1 + '/' : '') + (row.Category2 ? row.Category2 + '/' : '') + (row.Category3 ? row.Category3 + '/' : '') + row.Description;
                var dp = new disposition_code_model_1.DispositionCode(vcc, row.Description, treepath);
                dp.code = row.Code;
                dispositionCodes.push(dp);
            });
            resolve(dispositionCodes);
        }, function (error) {
            logger.error('[MMProdat::getDispositionCodes] requesting error: ' + error);
            reject(error);
        });
    });
}
exports.getDispositionCodes = getDispositionCodes;
exports.default = {
    getCampaigns: getCampaigns,
    getCampaignWebdesignerURL: getCampaignWebdesignerURL,
    getDispositionCodes: getDispositionCodes
};
