"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function AgentMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.AgentMSSql = AgentMSSql;
function GetAgents(vcc, agentIds) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AgentIds', sql.VarChar(50), agentIds)
        .execute('GetAgents')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAgents = GetAgents;
function GetAgentsFromEvaluation(vcc, userId, campaigns, searchType) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('UserId', sql.VarChar(50), userId)
        .input('Campaigns', sql.VarChar(sql.MAX), campaigns)
        .input('SearchType', sql.VarChar(50), searchType)
        .execute('GetAgentsFromEvaluation')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAgentsFromEvaluation = GetAgentsFromEvaluation;
function GetAgentQualificationAverage(instance, idAgent) {
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('IdAgent', sql.VarChar(50), idAgent)
        .execute('GetAgentQualificationAverage')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAgentQualificationAverage = GetAgentQualificationAverage;
function BulkAgent(vcc, agents) {
    var table = SetAgentsTemporaryTable(vcc, agents);
    return BulkAgentTransaction(vcc, table);
}
exports.BulkAgent = BulkAgent;
function SetAgentsTemporaryTable(vcc, data) {
    var table = new sql.Table('#AgentTemp');
    table.create = true;
    table.columns.add('VCC', sql.VarChar(50), { nullable: true });
    table.columns.add('AgentId', sql.VarChar(50), { nullable: true });
    table.columns.add('Name', sql.VarChar(250), { nullable: true });
    data.map(function (d) {
        table.rows.add(vcc, d.userId, d.userName);
    });
    return table;
}
function BulkAgentTransaction(vcc, temporaryTable) {
    var transaction = new sql.Transaction(sqlConn);
    var request;
    return transaction.begin()
        .then(function (result) {
        logger.info('[AgentMSSql::BulkAgentTransaction] Creating transaction');
        return new sql.Request(transaction);
    })
        .then(function (result) {
        request = result;
        return request.bulk(temporaryTable)
            .then(function (rowCount) {
            logger.info('[AgentMSSql::BulkAgentTransaction] Inserted ' + rowCount + ' records to temporary Table');
            return true;
        })
            .catch(function (err) {
            logger.error('[AgentMSSql::BulkAgentTransaction] Error on bulk: ' + err.message);
            throw err;
        });
    })
        .then(function (result) {
        return request
            .execute('BulkAgent')
            .then(function (recordsets) {
            logger.info('[AgentMSSql::BulkAgentTransaction] Succed Execution');
            var results = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
            var promiseArray = [];
            promiseArray.push(Promise.resolve(results && results.length > 0 ? results[0] : null));
            promiseArray.push(transaction.commit());
            return Promise.all(promiseArray);
        })
            .catch(function (err) {
            logger.error('[AgentMSSql::BulkAgentTransaction] Error on: ' + err.message);
            return Promise.reject(err);
        });
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[AgentMSSql::BulkAgentTransaction] Parsing error: ' + err.message);
        transaction.rollback();
        return Promise.reject(err);
    });
}
