"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var sql = require('mssql');
var interaction_model_1 = require("../../models/interaction.model");
var logger = log4js.getLogger('Sql');
var sqlConn;
function AllegroMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.AllegroMSSql = AllegroMSSql;
function GetAllegroAgents(ip, vcc, type, campaign) {
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.Int, type)
        .input('Campaign', sql.VarChar(50), campaign)
        .execute('GetAllegroAgents')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroAgents = GetAllegroAgents;
function GetAllegroCampaigns(ip, vcc, type) {
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.Int, type)
        .execute('GetAllegroCampaigns')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroCampaigns = GetAllegroCampaigns;
function GetAllegroManagementResults(ip, vcc, type) {
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.Int, type)
        .execute('GetAllegroManagementResults')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroManagementResults = GetAllegroManagementResults;
function GetAllegroSummaryInteractions(ip, vcc, startDate, endDate, startHour, endHour, campaign, agent, resultCode, type, typeIteraction, origin, minDuration, maxDuration, evaluationFormat, format, InteractionIds, evaluated) {
    if (InteractionIds === void 0) { InteractionIds = ''; }
    if (evaluated === void 0) { evaluated = ''; }
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('StartDate', sql.DateTime, startDate)
        .input('EndDate', sql.DateTime, endDate)
        .input('StartHour', sql.VarChar(50), startHour)
        .input('EndHour', sql.VarChar(50), endHour)
        .input('Campaign', sql.VarChar(sql.MAX), campaign)
        .input('Agent', sql.VarChar(sql.MAX), agent)
        .input('ResultCode', sql.VarChar(sql.MAX), resultCode)
        .input('Type', sql.Int, type)
        .input('TypeInteraction', sql.VarChar(sql.MAX), typeIteraction)
        .input('Origin', sql.VarChar(sql.MAX), origin)
        .input('MinDuration', sql.Int, minDuration)
        .input('MaxDuration', sql.Int, maxDuration)
        .input('EvaluationFormat', sql.VarChar(50), evaluationFormat)
        .input('Format', sql.VarChar(50), format)
        .input("InteractionIds", sql.VarChar(sql.MAX), InteractionIds)
        .input("Evaluated", sql.VarChar(sql.MAX), evaluated)
        .execute('GetAllegroSummaryInteractions')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var allegroSummaryInteractions = (recordset && recordset.length > 0 ? recordset[0] : {});
        allegroSummaryInteractions.interactionType = 'CALL';
        allegroSummaryInteractions.time = allegroSummaryInteractions.time || '0';
        allegroSummaryInteractions.quantity = allegroSummaryInteractions.quantity || '0';
        logger.info("[AllegroMSSql::GetAllegroSummaryInteractions] Total Records : " + allegroSummaryInteractions.quantity);
        return Promise.resolve(allegroSummaryInteractions);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[AllegroMSSql::GetAllegroSummaryInteractions] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroSummaryInteractions = GetAllegroSummaryInteractions;
function GetAllegroFilterInteractions(ip, vcc, startDate, endDate, startHour, endHour, campaign, agent, resultCode, type, origin, minDuration, maxDuration, interactionIds, evaluated) {
    if (interactionIds === void 0) { interactionIds = ''; }
    if (evaluated === void 0) { evaluated = ''; }
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('StartDate', sql.DateTime, startDate)
        .input('EndDate', sql.DateTime, endDate)
        .input('StartHour', sql.VarChar(50), startHour)
        .input('EndHour', sql.VarChar(50), endHour)
        .input('Campaign', sql.NVarChar(sql.MAX), campaign)
        .input('Agent', sql.NVarChar(sql.MAX), agent)
        .input('ResultCode', sql.NVarChar(sql.MAX), resultCode)
        .input('Type', sql.Int, type)
        .input('Origin', sql.NVarChar(sql.MAX), origin)
        .input('MinDuration', sql.Int, minDuration)
        .input('MaxDuration', sql.Int, maxDuration)
        .input('InteractionIds', sql.VarChar(sql.MAX), interactionIds)
        .input('Evaluated', sql.VarChar(sql.MAX), evaluated)
        .execute('GetAllegroFilterInteractions')
        .then(function (recordsets) {
        var recorddata = (recordsets && recordsets.length > 0 ? recordsets[0] : "NO_RESULT");
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[1] : null);
        var result = new interaction_model_1.InteractionView;
        result.interactionType = 'CALL';
        result.totalRecords = recorddata[0].totalInteractions;
        result.totalTime = recorddata[0].totalAttentionTime;
        result.audios = recordset;
        return Promise.resolve(result);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroFilterInteractions = GetAllegroFilterInteractions;
function GetAllegroInteractionVideos(ip, vcc, type, interactionIds) {
    if (interactionIds === void 0) { interactionIds = ''; }
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.Int, type)
        .input('InteractionIds', sql.VarChar(sql.MAX), interactionIds)
        .execute('GetAllegroInteractionVideos')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroInteractionVideos = GetAllegroInteractionVideos;
function GetAllegroBillingInteractions(vcc, ip, type, secondsPerRequest, startDate, endDate, campaigns, agents, origins, startHour, endHour, minDuration, maxDuration, results, formats) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Ip', sql.VarChar(50), ip)
        .input('Type', sql.Int, type)
        .input('SecondsPerRequest', sql.Decimal(18, 5), secondsPerRequest)
        .input('StartDate', sql.DateTime, startDate)
        .input('EndDate', sql.DateTime, endDate)
        .input('Campaigns', sql.NVarChar(sql.MAX), campaigns)
        .input('Agents', sql.NVarChar(sql.MAX), agents)
        .input('Origins', sql.NVarChar(sql.MAX), origins)
        .input('StartHour', sql.VarChar(50), startHour)
        .input('EndHour', sql.VarChar(50), endHour)
        .input('MinDuration', sql.Int, minDuration)
        .input('MaxDuration', sql.Int, maxDuration)
        .input('Results', sql.NVarChar(sql.MAX), results)
        .input('Formats', sql.NVarChar(50), formats)
        .execute('GetAllegroBillingInteractions')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var allegroBillingInteractions = (recordset && recordset.length > 0 ? recordset[0] : null);
        allegroBillingInteractions.interactionType = 'CALL';
        logger.info("[AllegroMSSql::GetAllegroBillingInteractions] Total Records: " + allegroBillingInteractions.quantity);
        return Promise.resolve(allegroBillingInteractions);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroBillingInteractions = GetAllegroBillingInteractions;
function GetAllegroBillingInteractionsForEvaluationPlan(vcc, processId, secondsPerRequest) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('SecondsPerRequest', sql.Decimal(18, 5), secondsPerRequest)
        .execute('GetAllegroBillingInteractionsForEvaluationPlan')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var result = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info("[AllegroMSSql::GetAllegroBillingInteractionsForEvaluationPlan] Total Records :" + result.length);
        return Promise.resolve(result);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroBillingInteractionsForEvaluationPlan = GetAllegroBillingInteractionsForEvaluationPlan;
