"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var moment = require("moment");
var sql = require('mssql');
var promises_shared_1 = require("../../shared/promises.shared");
var logger = log4js.getLogger('Sql');
var sqlConn;
function AudiosMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.AudiosMSSql = AudiosMSSql;
exports.headerTemplateCSV = ['ID', 'Number', 'Gender', 'SilenceTime', 'DurationTime', 'Category1', 'Category2', 'Category3', 'Description', 'Campaign', 'Agent', 'Direction', 'StartDate', 'EndDate', 'DispositionCode', 'RelativePath'];
function GetAudiosProcess(pageSize, pageNumber, vcc, processId, id, startDate, endDate, startHour, startMinute, endHour, endMinute, campaign, agent, resultCode, type, idathaResults, totalIdathaResults, interactionType, interactionIds, folder) {
    return new sql.Request(sqlConn)
        .input('PageSize', sql.Int, pageSize)
        .input('PageNumber', sql.Int, pageNumber)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('ID', sql.VarChar(250), id)
        .input('StartDate', sql.VarChar(50), startDate)
        .input('EndDate', sql.VarChar(50), endDate)
        .input('StartHour', sql.Int, startHour)
        .input('StartMinute', sql.Int, startMinute)
        .input('EndHour', sql.Int, endHour)
        .input('EndMinute', sql.Int, endMinute)
        .input('Campaign', sql.VarChar(sql.MAX), campaign)
        .input('Agent', sql.VarChar(sql.MAX), agent)
        .input('ResultCode', sql.VarChar(sql.MAX), resultCode)
        .input('Type', sql.VarChar(50), type)
        .input('IdathaResults', sql.NVarChar(sql.MAX), idathaResults)
        .input('TotalIdathaResults', sql.Int, totalIdathaResults)
        .input('InteractionType', sql.VarChar(50), interactionType)
        .input('InteractionIds', sql.VarChar(sql.MAX), interactionIds)
        .input('Folder', sql.VarChar(50), folder)
        .execute('GetAudiosProcess')
        .then(function (recordsets) {
        var totalRows = (recordsets && recordsets.length > 0 ? recordsets[0][0] : null);
        var channels = (recordsets && recordsets.length > 1 ? recordsets[1] : null);
        var campaigns = (recordsets && recordsets.length > 2 ? recordsets[2] : null);
        var recordset = (recordsets && recordsets.length > 3 ? recordsets[3] : null);
        var data = {
            asyncDetails: totalRows.asyncDetails,
            audios: totalRows.audios,
            campaigns: campaigns,
            channels: channels,
            totalRows: totalRows.total,
            maxDurationTime: totalRows.maxDurationTime,
            recordset: recordset
        };
        return Promise.resolve(data);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAudiosProcess = GetAudiosProcess;
function GetAudiosTranscriptions(pageSize, pageNumber, vcc, processId, id, startDate, endDate, startHour, startMinute, endHour, endMinute, campaign, agent, resultCode, interactionIds) {
    if (interactionIds === void 0) { interactionIds = ''; }
    return new sql.Request(sqlConn)
        .input('PageSize', sql.Int, pageSize)
        .input('PageNumber', sql.Int, pageNumber)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('ID', sql.VarChar(250), id)
        .input('StartDate', sql.VarChar(50), startDate)
        .input('EndDate', sql.VarChar(50), endDate)
        .input('StartHour', sql.Int, startHour)
        .input('StartMinute', sql.Int, startMinute)
        .input('EndHour', sql.Int, endHour)
        .input('EndMinute', sql.Int, endMinute)
        .input('Campaign', sql.VarChar(sql.MAX), campaign)
        .input('Agent', sql.VarChar(sql.MAX), agent)
        .input('ResultCode', sql.VarChar(sql.MAX), resultCode)
        .input('InteractionIds', sql.VarChar(sql.MAX), interactionIds)
        .execute('GetAudiosTranscriptions')
        .then(function (recordsets) {
        var totalRows = (recordsets && recordsets.length > 0 ? recordsets[0][0] : null);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[1] : null);
        var data = { totalRows: totalRows.total, maxDurationTime: totalRows.maxDurationTime, recordset: recordset };
        return Promise.resolve(data);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAudiosTranscriptions = GetAudiosTranscriptions;
function GetAudios(vcc, processId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .execute('GetAudios')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAudios = GetAudios;
function GetPendingAudios(vcc, processId, environmentType) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('EnvironmentType', sql.VarChar(50), environmentType)
        .execute('GetPendingAudios')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetPendingAudios = GetPendingAudios;
function GetAllegroAudiosStatistic(vcc, id) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(250), id)
        .execute('GetAllegroAudiosStatistic')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0][0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroAudiosStatistic = GetAllegroAudiosStatistic;
function GetProcessSummary(vcc, processId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .execute('GetProcessSummary')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0][0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetProcessSummary = GetProcessSummary;
function GetProcessAudioSummary(vcc, processId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .execute('GetProcessAudioSummary')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetProcessAudioSummary = GetProcessAudioSummary;
function GetAudio(vcc, processId, id, section, part) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('ID', sql.VarChar(250), id)
        .input('Section', sql.Numeric(5, 0), section)
        .input('Part', sql.Numeric(5, 0), part)
        .execute('GetAudio')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var audio = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(audio);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAudio = GetAudio;
function SaveAudio(vcc, id, section, part, processId, number, gender, silenceTime, durationTime, category1, category2, category3, description, campaign, agent, type, init) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(250), id)
        .input('Section', sql.Numeric(5, 0), section)
        .input('Part', sql.Numeric(5, 0), part)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('Number', sql.VarChar(50), number)
        .input('Gender', sql.VarChar(50), gender)
        .input('silenceTime', sql.Numeric(12, 3), silenceTime)
        .input('DurationTime', sql.Numeric(12, 3), durationTime)
        .input('Category1', sql.VarChar(100), category1)
        .input('Category2', sql.VarChar(100), category2)
        .input('Category3', sql.VarChar(100), category3)
        .input('Description', sql.VarChar(100), description)
        .input('Campaign', sql.VarChar(100), campaign)
        .input('Agent', sql.VarChar(50), agent)
        .input('Type', sql.VarChar(50), type)
        .input('Init', sql.VarChar(50), init)
        .execute('SaveAudio')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveAudio = SaveAudio;
function GetAllegroInteractions(ip, vcc, interactions, type) {
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Interactions', sql.NVarChar(sql.MAX), interactions)
        .input('Type', sql.Int, type)
        .execute('GetAllegroInteractions')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroInteractions = GetAllegroInteractions;
function GetAllegroInteractionsSearch(instance, limit, agents, campaigns, dateIni, dateFin, minDuration, maxDuration, origins, ip, vcc, type, interactionsAll, interactionsNothing, selectedInteractions, unselectedInteractions) {
    return new sql.Request(sqlConn)
        .input('IdAgent', sql.VarChar(sql.MAX), agents)
        .input('IdCampaign', sql.VarChar(sql.MAX), campaigns)
        .input('EvaluationDateFrom', sql.VarChar(50), dateIni)
        .input('EvaluationDateTo', sql.VarChar(50), dateFin)
        .input('InteractionMinDuration', sql.Numeric, minDuration)
        .input('InteractionMaxDuration', sql.Numeric, maxDuration)
        .input('InteractionOrigins', sql.VarChar(sql.MAX), origins)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.Int, type)
        .input('interactionsAll', sql.Bit, interactionsAll)
        .input('interactionsNothing', sql.Bit, interactionsNothing)
        .input('SelectedInteractions', sql.VarChar(sql.MAX), selectedInteractions)
        .input('UnselectedInteractions', sql.VarChar(sql.MAX), unselectedInteractions)
        .execute('GetAllegroInteractionsSearch')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAllegroInteractionsSearch = GetAllegroInteractionsSearch;
function SaveAudioLinguisticProcess(vcc, id, section, part, processId, durationLinguisticTime, isTranscribed, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ID', sql.VarChar(250), id)
        .input('Section', sql.Numeric(5, 0), section)
        .input('Part', sql.Numeric(5, 0), part)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('durationLinguisticTime', sql.Numeric(12, 3), durationLinguisticTime)
        .input('IsTranscribed', sql.Int, isTranscribed)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveAudioLinguisticProcess')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveAudioLinguisticProcess = SaveAudioLinguisticProcess;
function GetTranscription(transcriptionDTO) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), transcriptionDTO.vcc)
        .input('ProcessId', sql.VarChar(50), transcriptionDTO.processId)
        .input('InteractionType', sql.VarChar(50), transcriptionDTO.interactionType)
        .input('Section', sql.Numeric(5, 0), transcriptionDTO.section)
        .input('Part', sql.Numeric(5, 0), transcriptionDTO.part)
        .input('Folder', sql.VarChar(50), transcriptionDTO.folder)
        .input('Campaign', sql.VarChar(50), transcriptionDTO.campaign)
        .input('ID', sql.VarChar(250), transcriptionDTO.id)
        .execute('GetTranscription')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var transcription = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(transcription);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetTranscription = GetTranscription;
function SaveTranscription(transcriptionDTO) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), transcriptionDTO.vcc)
        .input('ProcessId', sql.VarChar(50), transcriptionDTO.processId)
        .input('InteractionType', sql.VarChar(50), transcriptionDTO.interactionType)
        .input('Section', sql.Numeric(5, 0), transcriptionDTO.section ? transcriptionDTO.section : 0)
        .input('Part', sql.Numeric(5, 0), transcriptionDTO.part ? transcriptionDTO.part : 0)
        .input('Campaign', sql.VarChar(50), transcriptionDTO.campaign ? transcriptionDTO.campaign : null)
        .input('Folder', sql.VarChar(50), transcriptionDTO.folder ? transcriptionDTO.folder : null)
        .input('ID', sql.VarChar(250), transcriptionDTO.id)
        .input('Value', sql.Int, transcriptionDTO.value)
        .input('UserId', sql.VarChar(50), transcriptionDTO.userId)
        .execute('SaveTranscription')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveTranscription = SaveTranscription;
function TransactionBulkAudioSummary(tableAudioSummary, BulkProcessAudioSummary, ProcessAudioSummaryTemp) {
    var transaction = new sql.Transaction(sqlConn);
    var request;
    return transaction.begin()
        .then(function (result) {
        logger.info('TransactionBulkAudioSummary: Creando solicitud');
        return new sql.Request(transaction);
    })
        .then(function (result) {
        request = result;
        return request.bulk(tableAudioSummary)
            .then(function (rowCount) {
            logger.info('TransactionBulkProcessAudioSummary: Bulk se insertaron : ' + rowCount + ' en la tabla temporal ' + ProcessAudioSummaryTemp);
            return true;
        })
            .catch(function (err) {
            logger.error('TransactionBulkProcessAudioSummary: Al realizar el bulk: ' + err.message + ' en la tabla temporal ' + ProcessAudioSummaryTemp);
            throw err;
        });
    })
        .then(function (rowCount) {
        logger.info('TransactionBulkProcessAudioSummary: Paso de la data de la tabla temporal a la tabla final');
        return BulkProcessAudioSummary(request);
    })
        .then(function (result) {
        logger.info('TransactionBulkAudioSummary: Guardando la transaccion');
        return transaction.commit();
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        transaction.rollback();
        logger.error('TransactionBulkAudioSummary: Error al realizar el parseo: ' + err.message);
        return Promise.reject(err);
    });
}
function TransactionBulkAudio(vcc, table, tableProcessAudio, methodValidator, nameTable, SaveProcessSummary, process, hold, averageHold, tmo, averageTmo, durationTimeRate, audioQuantity, userId) {
    var transaction = new sql.Transaction(sqlConn);
    var request;
    return transaction.begin()
        .then(function (result) {
        logger.info('TransactionBulkAudio: Creando solicitud');
        return new sql.Request(transaction);
    })
        .then(function (result) {
        request = result;
        return request.bulk(table)
            .then(function (rowCount) {
            logger.info('TransactionBulkAudio: Bulk se insertaron : ' + rowCount + ' en la tabla temporal #' + nameTable);
            return request.bulk(tableProcessAudio)
                .then(function (rowCount) {
                logger.info('TransactionBulkAudio: Bulk se insertaron : ' + rowCount + ' en la tabla temporal ' + '#Process' + nameTable);
                return true;
            });
        })
            .catch(function (err) {
            logger.error('TransactionBulkAudio: Al realizar el bulk: ' + err.message + ' en la tabla temporal ' + nameTable);
            throw err;
        });
    })
        .then(function (rowCount) {
        logger.info('TransactionBulkAudio: Paso de la data de la tabla temporal a la tabla final (BulkAudio)');
        return methodValidator(request);
    })
        .then(function (rowCount) {
        logger.info('TransactionSaveProcessSummary: Actualizamos la tabla ProcessSummary con los valores procesados');
        return SaveProcessSummary(request, vcc, process.name, hold, averageHold, tmo, averageTmo, durationTimeRate, userId, moment().utc().toDate());
    })
        .then(function (result) {
        logger.info('TransactionBulkAudio: Guardando la transaccion');
        return transaction.commit();
    })
        .then(function (result) {
        return Promise.resolve(audioQuantity);
    })
        .catch(function (err) {
        transaction.rollback();
        logger.error('TransactionBulkAudio: Error al realizar el parseo: ' + err.message);
        return Promise.reject(err);
    });
}
function BulkAudioSummaryTemp(vcc, process, summary, userId) {
    var tableAudioSummary = new sql.Table('#ProcessAudioSummaryTemp');
    tableAudioSummary.create = true;
    tableAudioSummary.columns.add('VCC', sql.VarChar(50), { nullable: true });
    tableAudioSummary.columns.add('ProcessId', sql.VarChar(50), { nullable: true });
    tableAudioSummary.columns.add('Category1', sql.VarChar(100), { nullable: true });
    tableAudioSummary.columns.add('Category2', sql.VarChar(100), { nullable: true });
    tableAudioSummary.columns.add('Category3', sql.VarChar(100), { nullable: true });
    tableAudioSummary.columns.add('Description', sql.VarChar(100), { nullable: true });
    tableAudioSummary.columns.add('Total', sql.Int, { nullable: true });
    tableAudioSummary.columns.add('CreatedDate', sql.DateTime, { nullable: true });
    tableAudioSummary.columns.add('CreatedByUserId', sql.VarChar(50), { nullable: true });
    tableAudioSummary.columns.add('LastModifiedDate', sql.DateTime, { nullable: true });
    tableAudioSummary.columns.add('LastModifiedByUserId', sql.VarChar(50), { nullable: true });
    for (var _i = 0, summary_1 = summary; _i < summary_1.length; _i++) {
        var row = summary_1[_i];
        tableAudioSummary.rows.add(vcc, process.name, row.Category1 ? row.Category1 : null, row.Category2 ? row.Category2 : null, row.Category3 ? row.Category3 : null, row.Description ? row.Description : null, row.Total, moment().utc().toDate(), userId, moment().utc().toDate(), userId);
    }
    logger.info('BulkAudioSummaryTemp: Parseo Correcto');
    return TransactionBulkAudioSummary(tableAudioSummary, BulkProcessAudioSummary, '#ProcessAudioSummaryTemp');
}
exports.BulkAudioSummaryTemp = BulkAudioSummaryTemp;
function BulkAudioTemp(vcc, process, audio, userId) {
    var table = new sql.Table('#AudioTemp');
    table.create = true;
    var tableProcessAudio = new sql.Table('#ProcessAudioTemp');
    tableProcessAudio.create = true;
    var hold = 0;
    var tmo = 0;
    var averageHold = 0;
    var averageTmo = 0;
    var durationTimeRate = 0;
    var SilenceTime;
    var DurationTime;
    var cantAudios = 0;
    var id;
    table.columns.add('VCC', sql.VarChar(50), { nullable: true });
    table.columns.add('ID', sql.VarChar(250), { nullable: true });
    table.columns.add('Section', sql.Numeric(5, 0), { nullable: true });
    table.columns.add('Part', sql.Numeric(5, 0), { nullable: true });
    table.columns.add('Number', sql.VarChar(50), { nullable: true });
    table.columns.add('Gender', sql.VarChar(50), { nullable: true });
    table.columns.add('SilenceTime', sql.Numeric(12, 3), { nullable: true });
    table.columns.add('DurationTime', sql.Numeric(12, 3), { nullable: true });
    table.columns.add('Category1', sql.VarChar(100), { nullable: true });
    table.columns.add('Category2', sql.VarChar(100), { nullable: true });
    table.columns.add('Category3', sql.VarChar(100), { nullable: true });
    table.columns.add('Description', sql.VarChar(100), { nullable: true });
    table.columns.add('Campaign', sql.VarChar(100), { nullable: true });
    table.columns.add('Agent', sql.VarChar(50), { nullable: true });
    table.columns.add('AgentName', sql.VarChar(100), { nullable: true });
    table.columns.add('Type', sql.VarChar(50), { nullable: true });
    table.columns.add('Init', sql.VarChar(50), { nullable: true });
    table.columns.add('StartDate', sql.VarChar(50), { nullable: true });
    table.columns.add('EndDate', sql.VarChar(50), { nullable: true });
    table.columns.add('DispositionCode', sql.VarChar(50), { nullable: true });
    table.columns.add('RelativePath', sql.VarChar(500), { nullable: true });
    table.columns.add('FileName', sql.VarChar(200), { nullable: true });
    table.columns.add('CreatedDate', sql.DateTime, { nullable: true });
    table.columns.add('CreatedByUserId', sql.VarChar(50), { nullable: true });
    table.columns.add('LastModifiedDate', sql.DateTime, { nullable: true });
    table.columns.add('LastModifiedByUserId', sql.VarChar(50), { nullable: true });
    tableProcessAudio.columns.add('VCC', sql.VarChar(50), { nullable: true });
    tableProcessAudio.columns.add('ProcessId', sql.VarChar(50), { nullable: true });
    tableProcessAudio.columns.add('AudioId', sql.VarChar(250), { nullable: true });
    tableProcessAudio.columns.add('Section', sql.Numeric(5, 0), { nullable: true });
    tableProcessAudio.columns.add('Part', sql.Numeric(5, 0), { nullable: true });
    tableProcessAudio.columns.add('CreatedDate', sql.DateTime, { nullable: true });
    tableProcessAudio.columns.add('CreatedByUserId', sql.VarChar(50), { nullable: true });
    tableProcessAudio.columns.add('LastModifiedDate', sql.DateTime, { nullable: true });
    tableProcessAudio.columns.add('LastModifiedByUserId', sql.VarChar(50), { nullable: true });
    for (var _i = 0, audio_1 = audio; _i < audio_1.length; _i++) {
        var row = audio_1[_i];
        SilenceTime = row.SilenceTime * 1 || 0;
        DurationTime = row.DurationTime * 1 || 0;
        hold = hold + SilenceTime;
        tmo = tmo + DurationTime;
        id = process.environmentType === 'allegro' ? row.ID.split('_') : row.ID;
        if (process.maxDurationTime > process.minDurationTime) {
            durationTimeRate = durationTimeRate + ((DurationTime >= process.minDurationTime && DurationTime <= process.maxDurationTime) ? 1 : 0);
        }
        table.rows.add(vcc, process.environmentType === 'allegro' ? id[0] : id, process.environmentType === 'allegro' ? id.length > 1 ? id[1] : 0 : 0, process.environmentType === 'allegro' ? id.length > 2 ? id[2] : 0 : 0, row.Number ? row.Number : null, row.Gender ? row.Gender : null, row.SilenceTime ? row.SilenceTime : null, row.DurationTime ? row.DurationTime : null, row.Category1 ? row.Category1 : null, row.Category2 ? row.Category2 : null, row.Category3 ? row.Category3 : null, row.Description ? row.Description : null, row.Campaign ? row.Campaign : null, row.Agent ? row.Agent : null, row.AgentName ? row.AgentName : null, row.Direction ? row.Direction : row.Type || null, row.Init ? row.Init : null, row.StartDate ? row.StartDate : null, row.EndDate ? row.EndDate : null, row.DispositionCode ? row.DispositionCode : null, row.RelativePath ? row.RelativePath : null, row.FileName ? row.FileName : null, moment().utc().toDate(), userId, moment().utc().toDate(), userId);
        tableProcessAudio.rows.add(vcc, process.name, process.environmentType === 'allegro' ? id[0] : id, process.environmentType === 'allegro' ? id.length > 1 ? id[1] : 0 : 0, process.environmentType === 'allegro' ? id.length > 2 ? id[2] : 0 : 0, moment().utc().toDate(), userId, moment().utc().toDate(), userId);
    }
    averageHold = hold / audio.length;
    averageTmo = tmo / audio.length;
    averageHold = moment.utc(Number(averageHold) * 1000).format('HH:mm:ss');
    averageTmo = moment.utc(Number(averageTmo) * 1000).format('HH:mm:ss');
    hold = moment.utc(Number(hold) * 1000).format('HH:mm:ss');
    tmo = moment.utc(Number(tmo) * 1000).format('HH:mm:ss');
    durationTimeRate = (durationTimeRate / audio.length) * 100;
    durationTimeRate = durationTimeRate.toFixed(2) + '%';
    logger.info('BulkAudioTemp: Parseo Correcto');
    return TransactionBulkAudio(vcc, table, tableProcessAudio, BulkAudio, 'AudioTemp', SaveProcessSummary, process, hold, averageHold, tmo, averageTmo, durationTimeRate, audio.length, userId);
}
exports.BulkAudioTemp = BulkAudioTemp;
function BulkAudio(request) {
    return request.execute('BulkAudio')
        .then(function (recordset) {
        logger.info('BulkAudio: Ejecucion Correcta BulkAudio');
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('BulkAudio: Al ejecutar el procedure BulkAudio : ' + err.message);
        return Promise.reject(err);
    });
}
exports.BulkAudio = BulkAudio;
function BulkProcessAudioSummary(request) {
    return request.execute('BulkProcessAudioSummary')
        .then(function (recordset) {
        logger.info('BulkProcessAudioSummary: Ejecucion Correcta BulkProcessAudioSummary');
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('BulkProcessAudioSummary: Al ejecutar el procedure BulkProcessAudioSummary : ' + err.message);
        return Promise.reject(err);
    });
}
exports.BulkProcessAudioSummary = BulkProcessAudioSummary;
function SaveProcessSummary(request, vcc, processId, silenceTime, averageSilenceTime, tmo, averageDurationTime, durationTimeRate, userId, createdDate) {
    return request
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('SilenceTime', sql.VarChar(50), silenceTime)
        .input('AverageSilenceTime', sql.VarChar(50), averageSilenceTime)
        .input('DurationTime', sql.VarChar(50), tmo)
        .input('AverageDurationTime', sql.VarChar(50), averageDurationTime)
        .input('DurationTimeRate', sql.VarChar(50), durationTimeRate)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), userId)
        .input('LastModifiedDate', sql.DateTime, createdDate)
        .input('LastModifiedByUserId', sql.VarChar(50), userId)
        .execute('SaveProcessSummary')
        .then(function (recordset) {
        logger.info('SaveProcessSummary: Ejecucion Correcta SaveProcessSummary');
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('SaveProcessSummary: Al ejecutar el procedure SaveProcessSummary : ' + err.message);
        return Promise.reject(err);
    });
}
exports.SaveProcessSummary = SaveProcessSummary;
function SaveProcessSummaryUpdate(vcc, processId, silenceTime, averageSilenceTime, durationTime, averageDurationTime, durationTimeRate, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('SilenceTime', sql.VarChar(50), silenceTime)
        .input('AverageSilenceTime', sql.VarChar(50), averageSilenceTime)
        .input('DurationTime', sql.VarChar(50), durationTime)
        .input('AverageDurationTime', sql.VarChar(50), averageDurationTime)
        .input('DurationTimeRate', sql.VarChar(50), durationTimeRate)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveProcessSummary')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveProcessSummaryUpdate = SaveProcessSummaryUpdate;
function DeleteProcessAudioByProcessIdInteractionId(vcc, processId, id, isUpdate) {
    logger.info('DeleteProcessAudioByProcessIdInteractionId - params: ', vcc, id, isUpdate);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('ID', sql.VarChar(250), id)
        .input('IsUpdate', sql.VarChar(1), isUpdate)
        .execute('DeleteProcessAudioByProcessIdInteractionId')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteProcessAudioByProcessIdInteractionId = DeleteProcessAudioByProcessIdInteractionId;
function TransactionBulk(tableTemp, procedure, reference) {
    var transaction;
    var request;
    var isRollback = false;
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        transaction = new sql.Transaction(sqlConn);
        return transaction.begin();
    }).then(function (result) {
        return new sql.Request(transaction);
    })
        .then(function (result) {
        request = result;
        return request.bulk(tableTemp);
    }).then(function (rowCount) {
        logger.info('[AudiosMSSql::TransactionBulk] getting', rowCount, 'affected rows after performing bulk insert operation into temporary table:', reference);
        return procedure(request);
    })
        .then(function (result) {
        logger.info('[AudiosMSSql::TransactionBulk] getting', result, 'BulkInteractionProcess with temporary table:', reference);
        return transaction.commit();
    }).catch(function (err) {
        isRollback = true;
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[AudiosMSSql::TransactionBulk] getting', errorMessage, 'after commit transaction operation into temporary table:', reference);
        return transaction.rollback();
    }).then(function (result) {
        if (!isRollback)
            logger.info('[AudiosMSSql::TransactionBulk] getting result:', result, 'after commit transaction operation with temporary table:', reference);
        return Promise.resolve(isRollback ? false : true);
    }).catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[AudiosMSSql::TransactionBulk] getting', errorMessage, reference);
        return Promise.resolve(false);
    });
}
function BulkProgressTemp(progress) {
    var tempTableName = '#ProgressTemp';
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        var table = new sql.Table('#ProgressTemp');
        table.create = true;
        table.columns.add('Key', sql.VarChar(300), { nullable: true });
        table.columns.add('VCC', sql.VarChar(50), { nullable: true });
        table.columns.add('ProcessId', sql.VarChar(50), { nullable: true });
        table.columns.add('Type', sql.VarChar(50), { nullable: true });
        table.columns.add('TypeId', sql.VarChar(50), { nullable: true });
        table.columns.add('Total', sql.Int, { nullable: true });
        table.columns.add('Progress', sql.Int, { nullable: true });
        table.columns.add('Error', sql.Int, { nullable: true });
        table.columns.add('Percentage', sql.Float, { nullable: true });
        table.columns.add('StatusDescription', sql.VarChar(sql.MAX), { nullable: true });
        for (var _i = 0, progress_1 = progress; _i < progress_1.length; _i++) {
            var item = progress_1[_i];
            table.rows.add(item.key, item.vcc, item.processId, item.type, item.typeId, item.value, item.progress, item.error, item.percentage, item.statusDescription);
        }
        return table;
    }).then(function (table) {
        logger.info('[AudiosMSSql::BulkProgressTemp] CREATED', tempTableName, tempTableName.length);
        return TransactionBulk(table, BulkProgress, tempTableName);
    }).catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[AudiosMSSql::BulkProgressTemp] getting error:', errorMessage, 'after bulk insert operation into temporary table:', tempTableName);
        return Promise.resolve(false);
    });
}
exports.BulkProgressTemp = BulkProgressTemp;
function BulkProgress(request) {
    return request.execute('BulkProgress')
        .then(function (recordset) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[AudiosMSSql::BulkProgress] getting error:', errorMessage);
        return Promise.reject(errorMessage);
    });
}
exports.BulkProgress = BulkProgress;
function BulkProcessInteractionTemp(interactions) {
    var tempTableName = '#ProcessInteractionTemp';
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        var tableInteractionProcessTemp = new sql.Table('#ProcessInteractionTemp');
        tableInteractionProcessTemp.create = true;
        tableInteractionProcessTemp.columns.add('VCC', sql.VarChar(50), { nullable: true });
        tableInteractionProcessTemp.columns.add('ProcessId', sql.VarChar(50), { nullable: true });
        tableInteractionProcessTemp.columns.add('Campaign', sql.VarChar(50), { nullable: true });
        tableInteractionProcessTemp.columns.add('Channel', sql.VarChar(50), { nullable: true });
        tableInteractionProcessTemp.columns.add('Format', sql.VarChar(50), { nullable: true });
        tableInteractionProcessTemp.columns.add('ID', sql.VarChar(250), { nullable: true });
        tableInteractionProcessTemp.columns.add('Section', sql.VarChar(50), { nullable: true });
        tableInteractionProcessTemp.columns.add('Part', sql.VarChar(50), { nullable: true });
        tableInteractionProcessTemp.columns.add('Folder', sql.VarChar(50), { nullable: true });
        tableInteractionProcessTemp.columns.add('MediaInfoTime', sql.Numeric(12, 3), { nullable: true });
        tableInteractionProcessTemp.columns.add('DurationLinguisticTime', sql.Numeric(12, 3), { nullable: true });
        tableInteractionProcessTemp.columns.add('IsDiariced', sql.Bit, { nullable: true });
        tableInteractionProcessTemp.columns.add('ChannelsChanged', sql.Bit, { nullable: true });
        tableInteractionProcessTemp.columns.add('ValidChar', sql.Int, { nullable: true });
        tableInteractionProcessTemp.columns.add('ValidWords', sql.Int, { nullable: true });
        tableInteractionProcessTemp.columns.add('TotalPhrases', sql.Int, { nullable: true });
        tableInteractionProcessTemp.columns.add('CharsPerPhrase', sql.Int, { nullable: true });
        tableInteractionProcessTemp.columns.add('IsTranscribed', sql.Int, { nullable: true });
        tableInteractionProcessTemp.columns.add('IsCancel', sql.Bit, { nullable: true });
        tableInteractionProcessTemp.columns.add('FileName', sql.VarChar(500), { nullable: true });
        tableInteractionProcessTemp.columns.add('ViewFileName', sql.VarChar(500), { nullable: true });
        tableInteractionProcessTemp.columns.add('ErrorMessage', sql.VarChar(sql.MAX), { nullable: true });
        for (var _i = 0, interactions_1 = interactions; _i < interactions_1.length; _i++) {
            var interaction = interactions_1[_i];
            tableInteractionProcessTemp.rows.add(interaction.vcc, interaction.processId, interaction.campaign, interaction.interactionType, interaction.format, interaction.id, interaction.section, interaction.part, interaction.folder, interaction.mediaInfoTime, interaction.durationLinguisticTime, interaction.isDiariced, interaction.channelsChanged, interaction.validChar, interaction.validWords, interaction.totalPhrases, interaction.charsPerPhrase, interaction.isTranscribed, interaction.isCancel, interaction.fileName, interaction.viewFileName, interaction.errorMessage);
        }
        return tableInteractionProcessTemp;
    }).then(function (tableInteractionProcessTemp) {
        logger.info('[AudiosMSSql::BulkProcessInteractionTemp] CREATED', tempTableName);
        return TransactionBulk(tableInteractionProcessTemp, BulkProcessInteraction, tempTableName);
    }).catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[AudiosMSSql::BulkProcessInteractionTemp] getting error:', errorMessage, 'after bulk insert operation into temporary table:', tempTableName);
        return Promise.resolve(false);
    });
}
exports.BulkProcessInteractionTemp = BulkProcessInteractionTemp;
function BulkProcessInteraction(request) {
    return request.execute('BulkProcessInteraction')
        .then(function (recordset) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[BulkProcessInteraction] getting error:', errorMessage);
        return Promise.reject(errorMessage);
    });
}
exports.BulkProcessInteraction = BulkProcessInteraction;
function BulkInteractionQueryTemp(vcc, processId, queryId, interactions) {
    var tempTableName = '#InteractionQueryTemp';
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        var table = new sql.Table('#InteractionQueryTemp');
        table.create = true;
        table.columns.add('VCC', sql.VarChar(50), { nullable: true });
        table.columns.add('ProcessId', sql.VarChar(200), { nullable: true });
        table.columns.add('AudioId', sql.VarChar(250), { nullable: true });
        table.columns.add('Section', sql.Numeric(5, 0), { nullable: true });
        table.columns.add('Part', sql.Numeric(5, 0), { nullable: true });
        table.columns.add('QueryId', sql.VarChar(50), { nullable: true });
        var interactionContent = [];
        logger.info('[AudiosMSSql::BulkInteractionQueryTemp] Interactions', interactions);
        for (var _i = 0, interactions_2 = interactions; _i < interactions_2.length; _i++) {
            var interaction = interactions_2[_i];
            interactionContent = interaction.id.split('_');
            var part = 0;
            var section = 0;
            if (interactionContent && interactionContent.length >= 3) {
                part = parseInt(interactionContent[interactionContent.length - 1]) ? parseInt(interactionContent[interactionContent.length - 1]) > 100 ? 0 : parseInt(interactionContent[interactionContent.length - 1]) : 0;
                section = parseInt(interactionContent[interactionContent.length - 2]) ? parseInt(interactionContent[interactionContent.length - 2]) > 100 ? 0 : parseInt(interactionContent[interactionContent.length - 2]) : 0;
            }
            var index = interactionContent.length && interactionContent.length > 2 ? 2 : 0;
            var interactionId = interactionContent.slice(0, interactionContent.length - index).join('_');
            table.rows.add(vcc.substr(0, 50), processId.substr(0, 200), interactionId.substr(0, 50), section, part, queryId.substr(0, 50));
        }
        return table;
    }).then(function (table) {
        logger.info('[AudiosMSSql::BulkInteractionQueryTemp] CREATED', tempTableName);
        return TransactionBulk(table, BulkInteractionQuery, tempTableName);
    }).catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[AudiosMSSql::BulkInteractionQueryTemp] getting error:', errorMessage, 'after bulk insert operation into temporary table:', tempTableName);
        return Promise.resolve(false);
    });
}
exports.BulkInteractionQueryTemp = BulkInteractionQueryTemp;
function BulkInteractionQuery(request) {
    return request.execute('BulkInteractionQuery')
        .then(function (recordset) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[BulkInteractionQuery] getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.BulkInteractionQuery = BulkInteractionQuery;
function BulkInteractionQueryStudyTemp(vcc, processId, studyId, templateId, queryId, interactions) {
    logger.info('[AudioMssql::BulkInteractionQueryStudyTemp] getting data:', interactions);
    logger.info('[AudioMssql::BulkInteractionQueryStudyTemp] getting interactions data:', interactions.length);
    var tempTableName = '#InteractionQueryStudyTemp';
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        var table = new sql.Table('#InteractionQueryStudyTemp');
        table.create = true;
        table.columns.add('VCC', sql.VarChar(50), { nullable: true });
        table.columns.add('ProcessId', sql.VarChar(50), { nullable: true });
        table.columns.add('StudyId', sql.VarChar(200), { nullable: true });
        table.columns.add('TemplateId', sql.VarChar(200), { nullable: true });
        table.columns.add('TemplateThemeItemId', sql.VarChar(200), { nullable: true });
        table.columns.add('TranscriptionId', sql.VarChar(250), { nullable: true });
        table.columns.add('Total', sql.Float, { nullable: true });
        table.columns.add('Duration', sql.Float, { nullable: true });
        table.columns.add('Silence', sql.Float, { nullable: true });
        table.columns.add('OtherDuration', sql.Float, { nullable: true });
        table.columns.add('OtherSilence', sql.Float, { nullable: true });
        table.columns.add('Confidence', sql.Float, { nullable: true });
        table.columns.add('EndDate', sql.VarChar(50), { nullable: true });
        table.columns.add('Channel', sql.VarChar(50), { nullable: true });
        table.columns.add('Campaign', sql.VarChar(100), { nullable: true });
        table.columns.add('ValidChar', sql.Int, { nullable: true });
        table.columns.add('ValidWords', sql.Int, { nullable: true });
        var lenWordsIn = 0;
        var lenWordsOut = 0;
        var totalConfidenceIn = 0;
        var totalConfidenceOut = 0;
        var callDurationIn = 0;
        var callDurationOut = 0;
        var callSilenceIn = 0;
        var callSilenceOut = 0;
        var callConfidenceIn = 0;
        var callConfidenceOut = 0;
        var otherDurationIn = 0;
        var otherDurationOut = 0;
        var otherSilenceIn = 0;
        var otherSilenceOut = 0;
        var otherConfidenceIn = 0;
        var otherConfidenceOut = 0;
        var _loop_1 = function (interaction) {
            interaction.char = 0;
            lenWordsIn = 0;
            lenWordsOut = 0;
            callDurationIn = 0;
            callDurationOut = 0;
            callSilenceIn = 0;
            callSilenceOut = 0;
            callConfidenceIn = 0;
            callConfidenceOut = 0;
            otherDurationIn = 0;
            otherDurationOut = 0;
            otherSilenceIn = 0;
            otherSilenceOut = 0;
            otherConfidenceIn = 0;
            otherConfidenceOut = 0;
            if (interaction && interaction.words && interaction.words.length) {
                interaction.words.forEach(function (word) {
                    interaction.char += word.name.length + 1;
                    if (interaction.filter_data.channel === 'CALL') {
                        if (word.speaker === 'in') {
                            callDurationIn += word.duration;
                            callSilenceIn += word.silence < 0 ? 0 : word.silence;
                            callConfidenceIn += word.confidence;
                            lenWordsIn++;
                        }
                        else {
                            callDurationOut += word.duration;
                            callSilenceOut += word.silence < 0 ? 0 : word.silence;
                            callConfidenceOut += word.confidence;
                            lenWordsOut++;
                        }
                    }
                    else {
                        if (word.speaker === 'in') {
                            otherDurationIn += word.duration;
                            otherSilenceIn += word.silence < 0 ? 0 : word.silence;
                            otherConfidenceIn += word.confidence;
                            lenWordsIn++;
                        }
                        else {
                            otherDurationOut += word.duration;
                            otherSilenceOut += word.silence < 0 ? 0 : word.silence;
                            otherConfidenceOut += word.confidence;
                            lenWordsOut++;
                        }
                    }
                });
                if (lenWordsIn > 0) {
                    totalConfidenceIn = ((callConfidenceIn + otherConfidenceIn) / lenWordsIn);
                }
                if (lenWordsOut > 0) {
                    totalConfidenceOut = ((callConfidenceOut + otherConfidenceOut) / lenWordsOut);
                }
            }
            table.rows.add(vcc, processId, studyId, templateId, queryId, interaction.id, 1, (callDurationIn + callDurationOut), (callSilenceIn + callSilenceOut), (otherDurationIn + otherDurationOut), (otherSilenceIn + otherSilenceOut), ((totalConfidenceIn + totalConfidenceOut) / 2), moment(interaction.filter_data.end_date).format("YYYY-MM-DD"), interaction.filter_data.channel, interaction.filter_data.campaign, interaction.char, interaction.words ? interaction.words : 0);
        };
        for (var _i = 0, interactions_3 = interactions; _i < interactions_3.length; _i++) {
            var interaction = interactions_3[_i];
            _loop_1(interaction);
        }
        return table;
    }).then(function (table) {
        logger.info('[AudioMssql::BulkInteractionQueryStudyTemp] CREATED', tempTableName);
        return TransactionBulk(table, BulkInteractionQueryStudy, tempTableName);
    }).catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[AudioMssql::BulkInteractionQueryStudyTemp] getting error:', errorMessage, 'after bulk insert operation into temporary table:', tempTableName);
        return Promise.resolve(false);
    });
}
exports.BulkInteractionQueryStudyTemp = BulkInteractionQueryStudyTemp;
function BulkInteractionQueryStudy(request) {
    return request.execute('BulkInteractionQueryStudy')
        .then(function (recordset) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[AudioMssql::BulkInteractionQueryStudy] getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.BulkInteractionQueryStudy = BulkInteractionQueryStudy;
function BulkEvaluationPlanQueryTemp(vcc, processId, evaluationPlanId, queryId, interactions) {
    var tempTableName = '#EvaluationPlanQueryTemp';
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        var table = new sql.Table('#EvaluationPlanQueryTemp');
        table.create = true;
        table.columns.add('VCC', sql.VarChar(50), { nullable: true });
        table.columns.add('ProcessId', sql.VarChar(200), { nullable: true });
        table.columns.add('EvaluationPlanId', sql.VarChar(50), { nullable: true });
        table.columns.add('AudioId', sql.VarChar(250), { nullable: true });
        table.columns.add('Section', sql.Numeric(5, 0), { nullable: true });
        table.columns.add('Part', sql.Numeric(5, 0), { nullable: true });
        table.columns.add('QueryId', sql.VarChar(50), { nullable: true });
        table.columns.add('EndDate', sql.VarChar(50), { nullable: true });
        table.columns.add('Channel', sql.VarChar(50), { nullable: true });
        table.columns.add('Campaign', sql.VarChar(50), { nullable: true });
        var interactionContent = [];
        for (var _i = 0, interactions_4 = interactions; _i < interactions_4.length; _i++) {
            var interaction = interactions_4[_i];
            var interactionId = '';
            var part = 0;
            var section = 0;
            if (interaction.filter_data.channel === 'CALL') {
                interactionContent = interaction.id.split('_');
                if (interactionContent && interactionContent.length >= 3) {
                    part = parseInt(interactionContent[interactionContent.length - 1]) ?
                        parseInt(interactionContent[interactionContent.length - 1]) > 100 ?
                            0
                            :
                                parseInt(interactionContent[interactionContent.length - 1])
                        :
                            0;
                    section = parseInt(interactionContent[interactionContent.length - 2]) ?
                        parseInt(interactionContent[interactionContent.length - 2]) > 100 ?
                            0
                            :
                                parseInt(interactionContent[interactionContent.length - 2])
                        :
                            0;
                }
                var index = interactionContent.length && interactionContent.length > 2 ? 2 : 0;
                interactionId = interactionContent.slice(0, interactionContent.length - index).join('_');
            }
            else {
                interactionId += interaction.id.split(':').join('_');
                var prefix = interaction.filter_data.channel
                    + '_'
                    + interaction.filter_data.campaign
                    + '_'
                    + moment(interaction.filter_data.end_date).format("YYYY-MM-DD")
                    + '_';
                interactionId = interactionId.replace(prefix, '');
            }
            table.rows.add(vcc.substr(0, 50), processId.substr(0, 200), evaluationPlanId.substr(0, 50), interactionId.substr(0, 250), section, part, queryId.substr(0, 50), moment(interaction.filter_data.end_date).format("YYYY-MM-DD"), interaction.filter_data.channel, interaction.filter_data.campaign);
        }
        if (!interactions || !interactions.length)
            table.rows.add(vcc, processId, evaluationPlanId, '---', 0, 0, '---', '---', '---', '---');
        return table;
    }).then(function (table) {
        logger.info('[AudiosMSSql::BulkEvaluationPlanQueryTemp] CREATED', tempTableName);
        return TransactionBulk(table, BulkEvaluationPlanQuery, tempTableName);
    }).catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[AudiosMSSql::BulkEvaluationPlanQueryTemp] getting error:', errorMessage, 'after bulk insert operation into temporary table:', tempTableName);
        return Promise.resolve(false);
    });
}
exports.BulkEvaluationPlanQueryTemp = BulkEvaluationPlanQueryTemp;
function BulkEvaluationPlanQuery(request) {
    return request.execute('BulkEvaluationPlanQuery')
        .then(function (recordset) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[AudiosMSSql::BulkEvaluationPlanQuery] getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.BulkEvaluationPlanQuery = BulkEvaluationPlanQuery;
function BulkEvaluationPlanAbilityTemp(vcc, processId, evaluationPlanId, queryId, interactions) {
    var tempTableName = '#EvaluationPlanAbilityTemp';
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        var table = new sql.Table('#EvaluationPlanAbilityTemp');
        table.create = true;
        table.columns.add('VCC', sql.VarChar(50), { nullable: true });
        table.columns.add('ProcessId', sql.VarChar(200), { nullable: true });
        table.columns.add('EvaluationPlanId', sql.VarChar(200), { nullable: true });
        table.columns.add('AudioId', sql.VarChar(250), { nullable: true });
        table.columns.add('Section', sql.Numeric(5, 0), { nullable: true });
        table.columns.add('Part', sql.Numeric(5, 0), { nullable: true });
        table.columns.add('QueryId', sql.VarChar(50), { nullable: true });
        table.columns.add('EndDate', sql.VarChar(50), { nullable: true });
        table.columns.add('Channel', sql.VarChar(50), { nullable: true });
        table.columns.add('Campaign', sql.VarChar(50), { nullable: true });
        var interactionContent = [];
        for (var _i = 0, interactions_5 = interactions; _i < interactions_5.length; _i++) {
            var interaction = interactions_5[_i];
            var interactionId = '';
            var part = 0;
            var section = 0;
            if (interaction.filter_data.channel === 'CALL') {
                interactionContent = interaction.id.split('_');
                if (interactionContent && interactionContent.length >= 3) {
                    part = parseInt(interactionContent[interactionContent.length - 1]) ?
                        parseInt(interactionContent[interactionContent.length - 1]) > 100 ?
                            0
                            :
                                parseInt(interactionContent[interactionContent.length - 1])
                        :
                            0;
                    section = parseInt(interactionContent[interactionContent.length - 2]) ?
                        parseInt(interactionContent[interactionContent.length - 2]) > 100 ?
                            0
                            :
                                parseInt(interactionContent[interactionContent.length - 2])
                        :
                            0;
                }
                var index = interactionContent.length && interactionContent.length > 2 ? 2 : 0;
                interactionId = interactionContent.slice(0, interactionContent.length - index).join('_');
            }
            else {
                interactionId += interaction.id.split(':').join('_');
                var prefix = interaction.filter_data.channel
                    + '_'
                    + interaction.filter_data.campaign
                    + '_'
                    + moment(interaction.filter_data.end_date).format("YYYY-MM-DD")
                    + '_';
                interactionId = interactionId.replace(prefix, '');
            }
            table.rows.add(vcc.substr(0, 50), processId.substr(0, 200), evaluationPlanId.substr(0, 50), interactionId.substr(0, 250), section, part, queryId.substr(0, 50), moment(interaction.filter_data.end_date).format("YYYY-MM-DD"), interaction.filter_data.channel, interaction.filter_data.campaign);
        }
        if (!interactions || !interactions.length)
            table.rows.add(vcc, processId, evaluationPlanId, '---', 0, 0, queryId, '---', '---', '---');
        return table;
    }).then(function (table) {
        logger.info('[AudiosMSSql::BulkEvaluationPlanAbilityTemp] CREATED', tempTableName);
        return TransactionBulk(table, BulkEvaluationPlanAbility, tempTableName);
    }).catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[AudiosMSSql::BulkEvaluationPlanAbilityTemp] getting error:', errorMessage, 'after bulk insert operation into temporary table:', tempTableName);
        return Promise.resolve(false);
    });
}
exports.BulkEvaluationPlanAbilityTemp = BulkEvaluationPlanAbilityTemp;
function BulkEvaluationPlanAbility(request) {
    return request.execute('BulkEvaluationPlanAbility')
        .then(function (recordset) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[AudiosMSSql::BulkEvaluationPlanAbility] getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.BulkEvaluationPlanAbility = BulkEvaluationPlanAbility;
function BulkInteractionCognitiveTemp(interactions) {
    var tempTableName = '#InteractionCognitiveTemp';
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        var table = new sql.Table('#InteractionCognitiveTemp');
        table.create = true;
        table.columns.add('VCC', sql.VarChar(50), { nullable: true });
        table.columns.add('ProcessId', sql.VarChar(50), { nullable: true });
        table.columns.add('StudyId', sql.VarChar(200), { nullable: true });
        table.columns.add('ID', sql.VarChar(250), { nullable: true });
        table.columns.add('Section', sql.VarChar(50), { nullable: true });
        table.columns.add('Part', sql.VarChar(50), { nullable: true });
        table.columns.add('TemplateId', sql.VarChar(200), { nullable: true });
        table.columns.add('TemplateThemeId', sql.VarChar(200), { nullable: true });
        table.columns.add('TemplateThemeItemId', sql.VarChar(200), { nullable: true });
        table.columns.add('TranscriptionId', sql.VarChar(250), { nullable: true });
        table.columns.add('NluId', sql.VarChar(50), { nullable: true });
        table.columns.add('EndDate', sql.VarChar(50), { nullable: true });
        table.columns.add('Channel', sql.VarChar(50), { nullable: true });
        table.columns.add('Campaign', sql.VarChar(100), { nullable: true });
        var _loop_2 = function (interaction) {
            var transcriptionId = '';
            if (interaction.interactionType === 'CALL') {
                transcriptionId = interaction.id;
                transcriptionId += '_';
                transcriptionId += interaction.section ? interaction.section : '0';
                transcriptionId += '_';
                transcriptionId += interaction.part ? interaction.part : '0';
            }
            else {
                transcriptionId += interaction.interactionType;
                transcriptionId += '_';
                transcriptionId += interaction.campaign;
                transcriptionId += '_';
                transcriptionId += interaction.folder;
                transcriptionId += '_';
                transcriptionId += interaction.id.split(':').join('_');
            }
            if (interaction && interaction.succedItems && interaction.succedItems.length > 0) {
                interaction.succedItems.map(function (si) {
                    si.templateThemeItems.map(function (tti) {
                        table.rows.add(interaction.vcc, interaction.processId, interaction.studyId, interaction.id, interaction.section, interaction.part, interaction.templateId, si.templateThemeId, tti, transcriptionId, interaction.nluId, interaction.folder, interaction.interactionType, interaction.campaign);
                    });
                });
            }
        };
        for (var _i = 0, interactions_6 = interactions; _i < interactions_6.length; _i++) {
            var interaction = interactions_6[_i];
            _loop_2(interaction);
        }
        return table;
    })
        .then(function (table) {
        logger.info('[AudiosMSSql::BulkInteractionCognitiveTemp] CREATED', tempTableName);
        return TransactionBulk(table, BulkInteractionCognitive, tempTableName);
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[AudiosMSSql::BulkInteractionCognitiveTemp] getting error:', errorMessage, 'after bulk insert operation into temporary table:', tempTableName);
        return Promise.resolve(false);
    });
}
exports.BulkInteractionCognitiveTemp = BulkInteractionCognitiveTemp;
function BulkInteractionCognitive(request) {
    logger.info("[AudiosMSSQL::BulkInteractionCognitive] RUNNING STUDY");
    return request.execute('BulkInteractionCognitive')
        .then(function (recordset) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[AudiosMSSql::BulkInteractionCognitive] getting error:', errorMessage);
        return Promise.reject(errorMessage);
    });
}
exports.BulkInteractionCognitive = BulkInteractionCognitive;
function BulkAbilityCognitiveTemp(interactions) {
    var tempTableName = '#AbilityCognitiveTemp';
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        var table = new sql.Table('#AbilityCognitiveTemp');
        table.create = true;
        table.columns.add('VCC', sql.VarChar(50), { nullable: true });
        table.columns.add('ProcessId', sql.VarChar(50), { nullable: true });
        table.columns.add('EvaluationPlanId', sql.VarChar(200), { nullable: true });
        table.columns.add('ID', sql.VarChar(250), { nullable: true });
        table.columns.add('Section', sql.VarChar(50), { nullable: true });
        table.columns.add('Part', sql.VarChar(50), { nullable: true });
        table.columns.add('TranscriptionId', sql.VarChar(200), { nullable: true });
        table.columns.add('NluId', sql.VarChar(50), { nullable: true });
        table.columns.add('EndDate', sql.VarChar(50), { nullable: true });
        table.columns.add('Channel', sql.VarChar(50), { nullable: true });
        table.columns.add('Campaign', sql.VarChar(100), { nullable: true });
        table.columns.add('AbilityId', sql.VarChar(50), { nullable: true });
        var _loop_3 = function (interaction) {
            var transcriptionId = '';
            if (interaction.interactionType === 'CALL') {
                transcriptionId = interaction.id;
                transcriptionId += '_';
                transcriptionId += interaction.section ? interaction.section : '0';
                transcriptionId += '_';
                transcriptionId += interaction.part ? interaction.part : '0';
            }
            else {
                transcriptionId += interaction.interactionType;
                transcriptionId += '_';
                transcriptionId += interaction.campaign;
                transcriptionId += '_';
                transcriptionId += interaction.folder;
                transcriptionId += '_';
                transcriptionId += interaction.id.split(':').join('_');
            }
            if (interaction && interaction.succedItems && interaction.succedItems.length > 0) {
                interaction.succedItems.map(function (si) {
                    table.rows.add(interaction.vcc, interaction.processId, interaction.evaluationPlanId, interaction.id, interaction.section, interaction.part, transcriptionId, interaction.nluId, interaction.folder, interaction.interactionType, interaction.campaign, si.abilityId);
                });
            }
        };
        for (var _i = 0, interactions_7 = interactions; _i < interactions_7.length; _i++) {
            var interaction = interactions_7[_i];
            _loop_3(interaction);
        }
        return table;
    }).then(function (table) {
        logger.info('[AudiosMSSql::BulkAbilityCognitiveTemp] CREATED', tempTableName);
        return TransactionBulk(table, BulkAbilityCognitive, tempTableName);
    }).catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[BulkAbilityCognitiveTemp] getting error:', errorMessage, 'after bulk insert operation into temporary table:', tempTableName);
        return Promise.resolve(false);
    });
}
exports.BulkAbilityCognitiveTemp = BulkAbilityCognitiveTemp;
function BulkAbilityCognitive(request) {
    return request.execute('BulkAbilityCognitive')
        .then(function (recordset) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[AudiosMSSql::BulkAbilityCognitive] getting error:', errorMessage);
        return Promise.reject(errorMessage);
    });
}
exports.BulkAbilityCognitive = BulkAbilityCognitive;
