"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function EnvironmentMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.EnvironmentMSSql = EnvironmentMSSql;
function GetEnvironments(vcc, search_criteria, search_expression, includeAllegro) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .input('IncludeAllegro', sql.Bit, includeAllegro)
        .execute('GetEnvironments')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetEnvironments = GetEnvironments;
function GetEnvironmentsUsage(vcc, environments) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Environments', sql.VarChar(sql.MAX), environments)
        .execute('GetEnvironmentsUsage')
        .then(function (recordsets) {
        var plans = recordsets && recordsets.length > 0 ? recordsets[0] : [];
        var process = recordsets && recordsets.length > 1 ? recordsets[1] : [];
        return {
            plans: plans,
            process: process
        };
    });
}
exports.GetEnvironmentsUsage = GetEnvironmentsUsage;
function GetEnvironment(vcc, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetEnvironment')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var environment = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(environment);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetEnvironment = GetEnvironment;
function GetEnvironmentByName(vcc, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetEnvironmentByName')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var environment = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(environment);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetEnvironmentByName = GetEnvironmentByName;
function GetEnvironmentTypeByName(vcc, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetEnvironmentTypeByName')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var environmentType = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(environmentType);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetEnvironmentTypeByName = GetEnvironmentTypeByName;
function SaveEnvironment(vcc, name, type, description, ftpDirectory, apiHost, apiPort, apiUser, apiPassword, dbDisabled, dbServer, dbPort, dbUser, dbPassword, pbxDisabled, pbxServer, pbxPort, pbxUser, pbxPassword, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .input('Type', sql.VarChar(50), type)
        .input('Description', sql.VarChar(200), description)
        .input('FTPDirectory', sql.VarChar(sql.MAX), ftpDirectory)
        .input('ApiHost', sql.VarChar(50), apiHost)
        .input('ApiPort', sql.VarChar(50), apiPort)
        .input('ApiUser', sql.VarChar(50), apiUser)
        .input('ApiPassword', sql.VarChar(50), apiPassword)
        .input('DbDisabled', sql.Bit, dbDisabled)
        .input('DbServer', sql.VarChar(50), dbServer)
        .input('DbPort', sql.VarChar(50), dbPort)
        .input('DbUser', sql.VarChar(50), dbUser)
        .input('DbPassword', sql.VarChar(50), dbPassword)
        .input('PbxDisabled', sql.Bit, pbxDisabled)
        .input('PbxServer', sql.VarChar(50), pbxServer)
        .input('PbxPort', sql.VarChar(50), pbxPort)
        .input('PbxUser', sql.VarChar(50), pbxUser)
        .input('PbxPassword', sql.VarChar(50), pbxPassword)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveEnvironment')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveEnvironment = SaveEnvironment;
function DeleteEnvironment(vcc, name) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('DeleteEnvironment')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteEnvironment = DeleteEnvironment;
