"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var sql = require('mssql');
var interaction_model_1 = require("../../models/interaction.model");
var logger = log4js.getLogger('Sql');
var sqlConn;
function InteractionMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.InteractionMSSql = InteractionMSSql;
function GetNotEvaluatedInteractionsAuditor(instance, userId, campaigns, agents, evaluationPlans, interactionTypes, interactionResults, dateFin, evaluationStatus, interactionIds) {
    if (interactionIds === void 0) { interactionIds = ''; }
    logger.info('[InteractionMSSql::GetNotEvaluatedInteractionsAuditor] Calling SP GetNotEvaluatedInteractionsAuditor for VCC: ' + instance);
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('UserId', sql.VarChar(50), userId)
        .input('IdCampaigns', sql.VarChar(sql.MAX), campaigns)
        .input('IdAgents', sql.VarChar(sql.MAX), agents)
        .input('EvaluationPlans', sql.VarChar(sql.MAX), evaluationPlans)
        .input('InteractionTypes', sql.VarChar(100), interactionTypes)
        .input('InteractionResults', sql.VarChar(sql.MAX), interactionResults)
        .input('EvaluationDateTo', sql.VarChar(50), dateFin)
        .input('EvaluationStatus', sql.Bit(), evaluationStatus)
        .input('InteractionIds', sql.VarChar(sql.MAX), interactionIds)
        .execute('GetNotEvaluatedInteractionsAuditor')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[InteractionMSSql::GetNotEvaluatedInteractionsAuditor] Received result from SP GetNotEvaluatedInteractionsAuditor: ' + recordsets.length);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Error executing: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[InteractionMSSql::GetNotEvaluatedInteractionsAuditor] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNotEvaluatedInteractionsAuditor = GetNotEvaluatedInteractionsAuditor;
function GetEvaluatedInteractionsByAuditor(instance, auditorIds, agentIds, campaignIds, evaluationPlanIds, evaluationDateFrom, evaluationDateTo, evaluationHourFrom, evaluationHourTo, interactionMinDuration, interactionMaxDuration, interactionOrigins, interactionResults, interactionTags, interactionFormats, interactionIds) {
    if (interactionIds === void 0) { interactionIds = ''; }
    logger.info('[InteractionMSSql::GetEvaluatedInteractionsByAuditor] Calling GetEvaluatedInteractionsByAuditor SP');
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('IdAuditor', sql.VarChar(sql.MAX), auditorIds)
        .input('IdAgent', sql.VarChar(sql.MAX), agentIds)
        .input('IdCampaign', sql.VarChar(sql.MAX), campaignIds)
        .input('IdEvaluationPlan', sql.VarChar(sql.MAX), evaluationPlanIds)
        .input('EvaluationDateFrom', sql.VarChar(50), evaluationDateFrom)
        .input('EvaluationDateTo', sql.VarChar(50), evaluationDateTo)
        .input('EvaluationHourFrom', sql.VarChar(50), evaluationHourFrom)
        .input('EvaluationHourTo', sql.VarChar(50), evaluationHourTo)
        .input('InteractionMinDuration', sql.Int, interactionMinDuration)
        .input('InteractionMaxDuration', sql.Int, interactionMaxDuration)
        .input('InteractionOrigins', sql.VarChar(50), interactionOrigins)
        .input('InteractionResults', sql.VarChar(sql.MAX), interactionResults)
        .input('InteractionTags', sql.VarChar(sql.MAX), interactionTags)
        .input('InteractionFormats', sql.VarChar(sql.MAX), interactionFormats)
        .input("InteractionIds", sql.VarChar(sql.MAX), interactionIds)
        .execute('GetEvaluatedInteractionsByAuditor')
        .then(function (recordsets) {
        logger.info('[InteractionMSSql::GetEvaluatedInteractionsByAuditor] Received resulto from GetEvaluatedInteractionsByAuditor SP');
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Error requesting: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[InteractionMSSql::GetEvaluatedInteractionsByAuditor] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetEvaluatedInteractionsByAuditor = GetEvaluatedInteractionsByAuditor;
function GetEvaluatedInteractionsByCampaignsAuditorsAndAgents(instance, auditorIds, agentIds, campaignIds, evaluationDateFrom, evaluationDateTo, evaluationHourFrom, evaluationHourTo, interactionMinDuration, interactionMaxDuration, interactionOrigins, interactionResults, evaluationTypes, interactionTags, interactionFormats) {
    logger.info('[InteractionMSSql::GetEvaluatedInteractionsByCampaignsAuditorsAndAgents] Received Instance: ' + instance);
    logger.info('[InteractionMSSql::GetEvaluatedInteractionsByCampaignsAuditorsAndAgents] Calling SP GetEvaluatedInteractionsByCampaignsAuditorsAndAgents');
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('IdAuditor', sql.VarChar(sql.MAX), auditorIds)
        .input('IdAgent', sql.VarChar(sql.MAX), agentIds)
        .input('IdCampaign', sql.VarChar(sql.MAX), campaignIds)
        .input('EvaluationDateFrom', sql.VarChar(sql.MAX), evaluationDateFrom)
        .input('EvaluationDateTo', sql.VarChar(sql.MAX), evaluationDateTo)
        .input('EvaluationHourFrom', sql.VarChar(50), evaluationHourFrom)
        .input('EvaluationHourTo', sql.VarChar(50), evaluationHourTo)
        .input('InteractionMinDuration', sql.Int, interactionMinDuration)
        .input('InteractionMaxDuration', sql.Int, interactionMaxDuration)
        .input('InteractionOrigins', sql.VarChar(50), interactionOrigins)
        .input('InteractionResults', sql.VarChar(sql.MAX), interactionResults)
        .input('EvaluationTypes', sql.VarChar(sql.MAX), evaluationTypes)
        .input('InteractionTags', sql.VarChar(sql.MAX), interactionTags)
        .input('InteractionFormats', sql.VarChar(sql.MAX), interactionFormats)
        .execute('GetEvaluatedInteractionsByCampaignsAuditorsAndAgents')
        .then(function (recordsets) {
        logger.info('[InteractionMSSql::GetEvaluatedInteractionsByCampaignsAuditorsAndAgents] Received result from GetEvaluatedInteractionsByCampaignsAuditorsAndAgents SP');
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Requesting error: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[InteractionMSSql::GetEvaluatedInteractionsByCampaignsAuditorsAndAgents] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetEvaluatedInteractionsByCampaignsAuditorsAndAgents = GetEvaluatedInteractionsByCampaignsAuditorsAndAgents;
function GetEvaluatedInteractions(instance, auditorIds, agentIds, campaignIds, evaluationDateFrom, evaluationDateTo, evaluationHourFrom, evaluationHourTo, interactionMinDuration, interactionMaxDuration, interactionOrigins, interactionResults, interactionTags, interactionFormats) {
    logger.info('[InteractionMSSql::GetEvaluatedInteractions] Received Instance: ' + instance);
    logger.info('[InteractionMSSql::GetEvaluatedInteractions] Calling SP GetEvaluatedInteractions');
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('IdAuditor', sql.VarChar(sql.MAX), auditorIds)
        .input('IdAgent', sql.VarChar(sql.MAX), agentIds)
        .input('IdCampaign', sql.VarChar(sql.MAX), campaignIds)
        .input('EvaluationDateFrom', sql.VarChar(50), evaluationDateFrom)
        .input('EvaluationDateTo', sql.VarChar(50), evaluationDateTo)
        .input('EvaluationHourFrom', sql.VarChar(50), evaluationHourFrom)
        .input('EvaluationHourTo', sql.VarChar(50), evaluationHourTo)
        .input('InteractionMinDuration', sql.Int, interactionMinDuration)
        .input('InteractionMaxDuration', sql.Int, interactionMaxDuration)
        .input('InteractionOrigins', sql.VarChar(50), interactionOrigins)
        .input('InteractionResults', sql.VarChar(sql.MAX), interactionResults)
        .input('InteractionTags', sql.VarChar(sql.MAX), interactionTags)
        .input('InteractionFormats', sql.VarChar(sql.MAX), interactionFormats)
        .execute('GetEvaluatedInteractions')
        .then(function (recordsets) {
        logger.info('[InteractionMSSql::GetEvaluatedInteractions] Received result from GetEvaluatedInteractions SP');
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Requesting error: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[InteractionMSSql::GetEvaluatedInteractions] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetEvaluatedInteractions = GetEvaluatedInteractions;
function GetInteractionsEvaluationPlan(instance, idEvaluationPlan, idAuditor) {
    logger.info('[InteractionMSSql::GetInteractionsEvaluationPlan] Received Instance: ' + instance);
    logger.info('[InteractionMSSql::GetInteractionsEvaluationPlan] Calling SP GetInteractionsEvaluationPlan');
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('IdEvaluationPlan', sql.VarChar(50), idEvaluationPlan)
        .input('IdAuditor', sql.VarChar(50), idAuditor)
        .execute('GetInteractionsEvaluationPlan')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[InteractionMSSql::GetInteractionsEvaluationPlan] Received interactions from SP: ' + recordset.length);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Error requesting: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[InteractionMSSql::GetInteractionsEvaluationPlan] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetInteractionsEvaluationPlan = GetInteractionsEvaluationPlan;
function GetEvaluationTypesInteraction(vcc, interactionId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('InteractionId', sql.VarChar(250), interactionId)
        .execute('GetEvaluationTypesInteraction')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetEvaluationTypesInteraction = GetEvaluationTypesInteraction;
function GetInteractions(instance, limit, agents, campaigns, dateIni, dateFin, minDuration, maxDuration, origins, hourIni, hourFin, tags, evaluationTypes, results, formats, interactionIds, evaluated, withSectionPart) {
    if (interactionIds === void 0) { interactionIds = ''; }
    if (evaluated === void 0) { evaluated = ''; }
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('IdAgent', sql.VarChar(sql.MAX), agents)
        .input('IdCampaign', sql.VarChar(sql.MAX), campaigns)
        .input('EvaluationDateFrom', sql.VarChar(50), dateIni)
        .input('EvaluationDateTo', sql.VarChar(50), dateFin)
        .input('EvaluationHourFrom', sql.VarChar(50), hourIni)
        .input('EvaluationHourTo', sql.VarChar(50), hourFin)
        .input('InteractionMinDuration', sql.Int, minDuration)
        .input('InteractionMaxDuration', sql.Int, maxDuration)
        .input('InteractionOrigins', sql.VarChar(50), origins)
        .input('InteractionsLimit', sql.Int, limit)
        .input('InteractionTags', sql.VarChar(sql.MAX), tags)
        .input('InteractionEvaluationTypes', sql.VarChar(sql.MAX), evaluationTypes)
        .input('InteractionResults', sql.VarChar(sql.MAX), results)
        .input('InteractionFormats', sql.VarChar(50), formats)
        .input('InteractionIds', sql.VarChar(sql.MAX), interactionIds)
        .input('Evaluated', sql.VarChar(sql.MAX), evaluated)
        .input('InteractionWithSectionPart', sql.Bit, withSectionPart)
        .execute('GetInteractionsSearch')
        .then(function (recordsets) {
        var recorddata = (recordsets && recordsets.length > 0 ? recordsets[0] : "No result");
        logger.info('[InteractionMSSql::GetInteractions] Interaccions search result: ', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[1] : null);
        var result = new interaction_model_1.InteractionView;
        result.interactionType = 'CALL';
        result.totalTime = recorddata[0].totalTime;
        result.totalRecords = recorddata[0].totalRecords;
        result.interactions = recordset;
        return Promise.resolve(result);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Request executing error: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[InteractionMSSql::GetInteractions] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetInteractions = GetInteractions;
function GetAsyncInteractions(ip, vcc, type, interactionType, startDate, endDate, startHour, endHour, campaigns, agents, dispositionCodes, origins, interactionIds, evaluated, uniqueIds, resultType) {
    if (interactionIds === void 0) { interactionIds = ''; }
    if (evaluated === void 0) { evaluated = ''; }
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.Int, type)
        .input('InteractionType', sql.VarChar(50), interactionType)
        .input('StartDate', sql.DateTime, startDate)
        .input('EndDate', sql.DateTime, endDate)
        .input('StartHour', sql.VarChar(50), startHour)
        .input('EndHour', sql.VarChar(50), endHour)
        .input('Campaigns', sql.VarChar(sql.MAX), campaigns)
        .input('Agents', sql.VarChar(sql.MAX), agents)
        .input('DispositionCodes', sql.VarChar(sql.MAX), dispositionCodes)
        .input('Origins', sql.VarChar(50), origins)
        .input("InteractionIds", sql.VarChar(sql.MAX), interactionIds)
        .input("Evaluated", sql.VarChar(sql.MAX), evaluated)
        .input('UniqueIds', sql.Bit, uniqueIds)
        .input('ResultType', sql.VarChar(50), resultType)
        .execute('GetAsyncInteractions')
        .then(function (recordsets) {
        var recorddata = (recordsets && recordsets.length > 0 ? recordsets[0] : "No result");
        var result = new interaction_model_1.AsyncInteractionView;
        if (uniqueIds === true) {
        }
        else {
        }
        if (resultType === 'SUMMARIZED') {
            var recordset = (recorddata && recorddata.length > 0 ? recorddata[0] : null);
            result.interactionType = recordset && recordset.interactionType ? recordset.interactionType : interactionType;
            result.totalInteractions = recordset && recordset.interactions ? recordset.interactions : 0;
            result.totalInteractionDetails = recordset && recordset.details ? recordset.details : 0;
        }
        else if (resultType === 'DETAILED') {
            result.interactionType = interactionType;
            result.totalInteractions = recorddata.length ?
                recorddata.map(function (item) { return item.id; }).filter(function (value, index, self) { return self.indexOf(value) === index; }).length
                :
                    0;
            result.totalInteractionDetails = recorddata.length ? recorddata.length : 0;
            result.interactions = recorddata ? recorddata : [];
        }
        return Promise.resolve(result);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Request executing error: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[InteractionMSSql::GetAsyncInteractions] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAsyncInteractions = GetAsyncInteractions;
function GetAsyncBillingInteractions(ip, vcc, type, interactionType, startDate, endDate, startHour, endHour, campaigns, agents, dispositionCodes, origins, interactionIds, cognitiveServiceCharsPerPhrase) {
    if (interactionIds === void 0) { interactionIds = ''; }
    return new sql.Request(sqlConn)
        .input('Ip', sql.VarChar(50), ip)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Type', sql.Int, type)
        .input('InteractionType', sql.VarChar(50), interactionType)
        .input('StartDate', sql.DateTime, startDate)
        .input('EndDate', sql.DateTime, endDate)
        .input('StartHour', sql.VarChar(50), startHour)
        .input('EndHour', sql.VarChar(50), endHour)
        .input('Campaigns', sql.VarChar(sql.MAX), campaigns)
        .input('Agents', sql.VarChar(sql.MAX), agents)
        .input('DispositionCodes', sql.VarChar(sql.MAX), dispositionCodes)
        .input('Origins', sql.VarChar(50), origins)
        .input("InteractionIds", sql.VarChar(sql.MAX), interactionIds)
        .input('CharsPerPhrase', sql.Int, cognitiveServiceCharsPerPhrase)
        .execute('GetAsyncBillingInteractions')
        .then(function (recordsets) {
        var recorddata = (recordsets && recordsets.length > 0 ? recordsets[0] : "No result");
        var recordset = (recorddata && recorddata.length > 0 ? recorddata[0] : null);
        return Promise.resolve(recordset);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Request executing error: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[InteractionMSSql::GetAsyncBillingInteractions] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAsyncBillingInteractions = GetAsyncBillingInteractions;
function GetInteractionTranscription(vcc, evaluationPlanId, processId, interactionType, interactionId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('EvaluationPlanId', sql.VarChar(50), evaluationPlanId)
        .input('ProcessId', sql.VarChar(50), processId)
        .input('InteractionType', sql.VarChar(250), interactionType)
        .input('InteractionId', sql.VarChar(250), interactionId)
        .execute('GetInteractionTranscription')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetInteractionTranscription = GetInteractionTranscription;
