"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var sql = require('mssql');
var Promise = require('bluebird');
var logger = log4js.getLogger('Sql');
var sqlConn;
function NotificationMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.NotificationMSSql = NotificationMSSql;
function GetNotifications(type) {
    return new sql.Request(sqlConn)
        .input('Type', sql.VarChar(50), type)
        .execute('GetNotifications')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNotifications = GetNotifications;
function GetUserNotifications(instance, userId, type) {
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), instance)
        .input('UserId', sql.VarChar(50), userId)
        .input('Type', sql.VarChar(50), type)
        .execute('GetUserNotifications')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetUserNotifications = GetUserNotifications;
function SaveNotification(userIndicatorId, instance, indicatorId, userId, type, opened, sendDate, title, description, createdDate) {
    return new sql.Request(sqlConn)
        .input('UserIndicatorId', sql.VarChar(50), userIndicatorId)
        .input('Instance', sql.VarChar(50), instance)
        .input('IndicatorId', sql.VarChar(50), indicatorId)
        .input('UserId', sql.VarChar(50), userId)
        .input('Opened', sql.Bit, opened)
        .input('Type', sql.VarChar(50), type)
        .input('SendDate', sql.DateTime, sendDate)
        .input('Title', sql.VarChar(150), title)
        .input('Description', sql.VarChar(500), description)
        .input('CreatedDate', sql.DateTime, createdDate)
        .execute('SaveUserNotification')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveNotification = SaveNotification;
