"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function UtteranceMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.UtteranceMSSql = UtteranceMSSql;
function CatchHandler(err) {
    var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
    logger.error(errorMsg);
    return Promise.reject(err);
}
function GetUtterance(vcc, appId, intentId, text) {
    logger.info('[UtteranceMSSql::GetUtterance] getting vcc:', vcc, 'text:', text);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('IntentId', sql.VarChar(50), intentId)
        .input('Text', sql.VarChar(50), text)
        .execute('GetUtterance')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var utterance = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(utterance);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[UtteranceMSSql::GetUtterance] getting errorMsg:', errorMsg, 'of GetUtterance');
        return Promise.reject(err);
    });
}
exports.GetUtterance = GetUtterance;
function GetUtterances(vcc, appId, intentId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('IntentId', sql.VarChar(50), intentId)
        .execute('GetUtterances')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch(CatchHandler);
}
exports.GetUtterances = GetUtterances;
function GetUtterancesByIntentId(vcc, intentId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('IntentId', sql.VarChar(50), intentId)
        .execute('GetUtterancesByIntentId')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('GetUtterancesByIntentId: ', recordset);
        return Promise.resolve(recordset);
    }).catch(CatchHandler);
}
exports.GetUtterancesByIntentId = GetUtterancesByIntentId;
function GetUtterancesByInteraction(vcc, audioId, section, part) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AudioId', sql.VarChar(50), audioId)
        .input('Section', sql.Numeric(5, 0), section)
        .input('Part', sql.Numeric(5, 0), part)
        .execute('GetUtterancesByInteraction')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('GetUtterancesByInteraction: ', recordset);
        return Promise.resolve(recordset);
    }).catch(CatchHandler);
}
exports.GetUtterancesByInteraction = GetUtterancesByInteraction;
function GetUtteranceEntities(vcc, utteranceId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('UtteranceId', sql.VarChar(50), utteranceId)
        .execute('GetUtteranceEntities')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    })
        .catch(CatchHandler);
}
exports.GetUtteranceEntities = GetUtteranceEntities;
function SaveUtteranceEntity(vcc, utteranceId, entityName, start, end, startTokenIndex, endTokenIndex) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('UtteranceId', sql.VarChar(50), utteranceId)
        .input('EntityName', sql.VarChar(50), entityName)
        .input('Start', sql.Int, start)
        .input('End', sql.Int, end)
        .input('EndTokenIndex', sql.Int, endTokenIndex)
        .input('StartTokenIndex', sql.Int, startTokenIndex)
        .execute('SaveUtteranceEntity')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[UtteranceMSSql::SaveUtteranceEntity] Output: ', output);
        return Promise.resolve(output && output.result === 1);
    })
        .catch(CatchHandler);
}
exports.SaveUtteranceEntity = SaveUtteranceEntity;
function SaveUtterance(vcc, appId, intentName, id, text, userId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('IntentName', sql.VarChar(50), intentName)
        .input('Id', sql.VarChar(50), id)
        .input('Text', sql.VarChar(500), text)
        .input('UserId', sql.VarChar(50), userId)
        .execute('SaveUtterance')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('SaveUtterance - recordset: ', recordset);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('SaveUtterance - output: ', output);
        if (output) {
            return Promise.resolve(output.utteranceId);
        }
        return Promise.resolve(null);
    })
        .catch(CatchHandler);
}
exports.SaveUtterance = SaveUtterance;
function DeleteUtterance(vcc, appId, intentId, utteranceId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('IntentId', sql.VarChar(50), intentId)
        .input('Id', sql.VarChar(50), utteranceId)
        .execute('DeleteUtterance')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('DeleteUtterance - recordset: ', recordset);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('DeleteUtterance - output: ', output);
        return Promise.resolve(output && output.result === 1);
    }).catch(CatchHandler);
}
exports.DeleteUtterance = DeleteUtterance;
function DeleteUtteranceEntity(vcc, utteranceId, entityName, start, end, startTokenIndex, endTokenIndex) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('UtteranceId', sql.VarChar(50), utteranceId)
        .input('EntityName', sql.VarChar(50), entityName)
        .input('Start', sql.Int, start)
        .input('End', sql.Int, end)
        .input('EndTokenIndex', sql.Int, endTokenIndex)
        .input('StartTokenIndex', sql.Int, startTokenIndex)
        .execute('DeleteUtteranceEntity')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    })
        .catch(CatchHandler);
}
exports.DeleteUtteranceEntity = DeleteUtteranceEntity;
function SaveUnderlined(vcc, appId, intentId, audioId, section, part, speaker, anchorOffset, focusOffset, currentTime, endCurrentTime, utteranceId, textContent, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('IntentId', sql.VarChar(50), intentId)
        .input('AudioId', sql.VarChar(50), audioId)
        .input('Section', sql.Numeric(5, 0), section)
        .input('Part', sql.Numeric(5, 0), part)
        .input('Speaker', sql.VarChar(3), speaker)
        .input('AnchorOffset', sql.Numeric(5, 0), anchorOffset)
        .input('FocusOffset', sql.Numeric(5, 0), focusOffset)
        .input('CurrentTime', sql.Float, currentTime)
        .input('EndCurrentTime', sql.Float, endCurrentTime)
        .input('UtteranceId', sql.Numeric(18, 0), utteranceId)
        .input('TextContent', sql.VarChar(500), textContent)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveUnderlined')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('SaveUnderlined - recordset: ', recordset);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('SaveUnderlined - output: ', output);
        return Promise.resolve(output && output.result === 1);
    }).catch(CatchHandler);
}
exports.SaveUnderlined = SaveUnderlined;
