"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var redis = require("redis");
var bluebird = require("bluebird");
var log4js = require("log4js");
var redis_config_1 = require("../config/redis.config");
var domain_redis_1 = require("./redis/domain.redis");
var evaluation_redis_1 = require("./redis/evaluation.redis");
var evaluation_plan_ability_redis_1 = require("./redis/evaluation-plan-ability.redis");
var vcc_redis_1 = require("./redis/vcc.redis");
var role_redis_1 = require("./redis/role.redis");
var session_redis_1 = require("./redis/session.redis");
var user_redis_1 = require("./redis/user.redis");
var interaction_redis_1 = require("./redis/interaction.redis");
var process_redis_1 = require("./redis/process.redis");
var cognitive_redis_1 = require("./redis/cognitive.redis");
var queue_redis_1 = require("./redis/queue.redis");
var platform_redis_1 = require("./redis/platform.redis");
var nlu_redis_1 = require("./redis/nlu.redis");
var study_template_theme_redis_1 = require("./redis/study-template-theme.redis");
var cleaner_redis_1 = require("./redis/cleaner.redis");
var logger = log4js.getLogger('Redis');
function RedisInit(app) {
    return new Promise(function (resolve, reject) {
        logger.info('Iniciando conexión a Redis');
        console.log('Iniciando conexión a Redis');
        var redisClient = redis.createClient(redis_config_1.REDIS_OPTIONS)
            .on('error', function (err) {
            var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('No se pudo inicializar cliente Redis: ' + errMsg);
            console.error('No se pudo inicializar cliente Redis: ' + errMsg);
            reject(err);
        })
            .on('connect', function () {
            logger.info('Conexión a Redis establecida');
            console.log('Conexión a Redis establecida');
            resolve(true);
        });
        bluebird.promisifyAll(redisClient);
        app.set('redisClient', redisClient);
        domain_redis_1.DomainRedis(app);
        evaluation_redis_1.EvaluationRedis(app);
        evaluation_plan_ability_redis_1.EvaluationPlanAbilityRedis(app);
        vcc_redis_1.VccRedis(app);
        role_redis_1.RoleRedis(app);
        session_redis_1.SessionRedis(app);
        study_template_theme_redis_1.StudyTemplateThemeRedis(app);
        user_redis_1.UserRedis(app);
        interaction_redis_1.InteractionRedis(app);
        process_redis_1.ProcessRedis(app);
        cognitive_redis_1.CognitiveRedis(app);
        queue_redis_1.QueueRedis(app);
        platform_redis_1.PlatformRedis(app);
        nlu_redis_1.NluRedis(app);
        cleaner_redis_1.CleanerRedis(app);
    });
}
exports.RedisInit = RedisInit;
