"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var redis_config_1 = require("../../config/redis.config");
var promises_shared_1 = require("../../shared/promises.shared");
var logger = log4js.getLogger('Redis');
var redisClient;
function CleanerRedis(app) {
    redisClient = app.get('redisClient');
}
exports.CleanerRedis = CleanerRedis;
function CleanerRedisGenericErrorHandler(err) {
    var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
    logger.error(errorMsg);
    return Promise.reject(err);
}
function SaveCleanerData(key, idColumn, vcc, mixin, rows) {
    return promises_shared_1.DummyPromise().then(function (_) {
        var promises = [];
        var keys = [];
        for (var _i = 0, rows_1 = rows; _i < rows_1.length; _i++) {
            var row = rows_1[_i];
            var _key = vcc + ":" + row[idColumn];
            promises.push(redisClient.hsetAsync(key, _key, JSON.stringify(Object.assign({ vcc: vcc }, mixin, row))));
            keys.push(_key);
        }
        promises.push(redisClient.lpushAsync(key + ":Keys", keys));
        return Promise.all(promises);
    })
        .then(function (_) {
        return redisClient.expireAsync(key, redis_config_1.REDIS_DATA_PERSISTENCE);
    }).then(function () {
        return redisClient.expireAsync(key + ":Keys", redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .catch(CleanerRedisGenericErrorHandler);
}
exports.SaveCleanerData = SaveCleanerData;
function SaveCleanerError(key, data) {
    return promises_shared_1.DummyPromise().then(function (_) {
        return redisClient.lpushAsync(key, JSON.stringify(data));
    }).then(function (_) {
        return redisClient.expireAsync(key, redis_config_1.REDIS_DATA_PERSISTENCE);
    }).catch(CleanerRedisGenericErrorHandler);
}
exports.SaveCleanerError = SaveCleanerError;
function GetCleanerData(key, keys) {
    return promises_shared_1.DummyPromise().then(function (_) {
        return redisClient.hmgetAsync(key, keys);
    }).catch(CleanerRedisGenericErrorHandler);
}
exports.GetCleanerData = GetCleanerData;
function DeleteCleanerData(key, id) {
    return promises_shared_1.DummyPromise().then(function (_) {
        return redisClient.hdelAsync(key, id);
    }).catch(CleanerRedisGenericErrorHandler);
}
exports.DeleteCleanerData = DeleteCleanerData;
function DeleteAllCleanerTasks(keys) {
    return promises_shared_1.DummyPromise().then(function (_) {
        return redisClient.delAsync(keys);
    }).catch(CleanerRedisGenericErrorHandler);
}
exports.DeleteAllCleanerTasks = DeleteAllCleanerTasks;
