"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var redis_config_1 = require("../../config/redis.config");
var logger = log4js.getLogger('Redis');
var redisClient;
function CognitiveRedis(app) {
    redisClient = app.get('redisClient');
}
exports.CognitiveRedis = CognitiveRedis;
function GetCognitiveInteraction(key) {
    var storage = 'Cognitive';
    return redisClient.hgetAsync(storage, key)
        .then(function (result) {
        var interaction;
        try {
            interaction = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return interaction;
    })
        .catch(function (err) {
        logger.error('[GetInteractionCognitive] getting err value:', err, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetCognitiveInteraction = GetCognitiveInteraction;
function SaveCognitiveInteraction(key, interaction) {
    var storage = 'Cognitive';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(interaction))
        .then(function (result) {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[SaveInteractionCognitive] getting error:', err, 'key:', key);
        return Promise.resolve(err);
    });
}
exports.SaveCognitiveInteraction = SaveCognitiveInteraction;
function DeleteCognitiveInteraction(key) {
    var storage = 'Cognitive';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[DeleteCognitiveInteraction] getting error:', err, 'key:', key);
        return Promise.resolve(false);
    });
}
exports.DeleteCognitiveInteraction = DeleteCognitiveInteraction;
function SaveReadyCognitiveInteraction(key, interaction, executionCode) {
    var storage = 'Cognitive';
    storage += ':';
    storage += executionCode;
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(interaction))
        .then(function (result) {
        return redisClient.expireAsync(key + ':' + storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[SaveReadyCognitiveInteraction] getting error:', err, 'with vcc:', interaction.vcc, 'processId:', interaction.processId, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.SaveReadyCognitiveInteraction = SaveReadyCognitiveInteraction;
function GetReadyCognitiveInteractions(executionCode) {
    var hash = 'Cognitive:';
    hash += executionCode;
    return redisClient.hgetallAsync(hash)
        .then(function (result) {
        var interactions = [];
        try {
            var interactionsToSendResults = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && interactionsToSendResults) {
                for (var key in interactionsToSendResults) {
                    interactions.push(JSON.parse(interactionsToSendResults[key]));
                }
            }
        }
        catch (err) {
            logger.error('[GetReadyCognitiveInteractions] getting err value:', result);
            return Promise.resolve(null);
        }
        return interactions;
    })
        .catch(function (err) {
        logger.error('[GetReadyCognitiveInteractions] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetReadyCognitiveInteractions = GetReadyCognitiveInteractions;
function SaveReadyCognitiveAbility(key, interaction, executionCode) {
    var storage = 'AbilityCognitive';
    storage += ':';
    storage += executionCode;
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(interaction))
        .then(function (result) {
        return redisClient.expireAsync(key + ':' + storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[SaveReadyAbilityCognitive] getting error:', err, 'with vcc:', interaction.vcc, 'processId:', interaction.processId, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.SaveReadyCognitiveAbility = SaveReadyCognitiveAbility;
function GetReadyCognitiveAbilities(executionCode) {
    var hash = 'AbilityCognitive:';
    hash += executionCode;
    return redisClient.hgetallAsync(hash)
        .then(function (result) {
        var interactions = [];
        try {
            var interactionsToSendResults = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && interactionsToSendResults) {
                for (var key in interactionsToSendResults) {
                    interactions.push(JSON.parse(interactionsToSendResults[key]));
                }
            }
        }
        catch (err) {
            logger.error('[GetReadyAbilitiesCognitive] getting err value:', result);
            return Promise.resolve(null);
        }
        return interactions;
    })
        .catch(function (err) {
        logger.error('[GetReadyAbilitiesCognitive] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetReadyCognitiveAbilities = GetReadyCognitiveAbilities;
function SaveRunningRasaModel(key, runningModel) {
    var storage = 'RasaModel';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(runningModel))
        .then(function (result) {
        return redisClient.expireAsync(key + ':' + storage, 60 * 60);
    }).then(function (result) {
        return Promise.resolve(true);
    }).catch(function (err) {
        logger.error('[cognitive.redis::SaveRunningRasaModel] nluId:', key, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.SaveRunningRasaModel = SaveRunningRasaModel;
function GetRunningRasaModel(key) {
    var storage = 'RasaModel';
    return redisClient.hgetAsync(storage, key)
        .then(function (result) {
        var interactions = [];
        try {
            var nlu = (typeof result === 'string') ? JSON.parse(result) : result;
            return Promise.resolve(nlu);
        }
        catch (err) {
            logger.error('[cognitive.redis:GetRunningRasaModel] getting err value:', result);
            return Promise.resolve(null);
        }
    }).catch(function (err) {
        logger.error('[cognitive.redis:GetRunningRasaModel] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetRunningRasaModel = GetRunningRasaModel;
function DeleteRunningRasaModel(key) {
    var storage = 'RasaModel';
    if (!key)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then(function (result) {
        return Promise.resolve(true);
    }).catch(function (err) {
        logger.error('[cognitive.redis::DeleteRunningRasaModel] getting error: ', err, 'key: ', key);
        return Promise.resolve(false);
    });
}
exports.DeleteRunningRasaModel = DeleteRunningRasaModel;
function GetRunningRasaModels() {
    var storage = 'RasaModel';
    return redisClient.hgetallAsync(storage)
        .then(function (result) {
        var runningRasaModelsList = [];
        try {
            runningRasaModelsList = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            logger.error('[cognitive.redis::GetRunningRasaModels] getting err value:', result);
            return Promise.resolve(null);
        }
        return Promise.resolve(runningRasaModelsList);
    }).catch(function (err) {
        logger.error('[cognitive.redis::GetRunningRasaModels] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetRunningRasaModels = GetRunningRasaModels;
