"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var redis_config_1 = require("../../config/redis.config");
var logger = log4js.getLogger('Redis');
var redisClient;
function DomainRedis(app) {
    redisClient = app.get('redisClient');
}
exports.DomainRedis = DomainRedis;
function SaveDomain(vcc, domain) {
    if (!domain || !domain.domain)
        throw 'Se intento guardar un registro en una clave nula';
    return redisClient.hsetAsync(vcc + ':Domain', domain.domain, JSON.stringify(domain))
        .then(function (result) {
        return redisClient.expireAsync(vcc + ':Domain', redis_config_1.REDIS_DATA_PERSISTENCE);
    }).then(function (result) {
        return redisClient.hgetAsync(vcc + ':MainDomain', vcc);
    }).then(function (result) {
        try {
            var mainDomain = void 0;
            mainDomain = JSON.parse(result);
            if (mainDomain.domain === domain.domain || JSON.stringify(mainDomain) === JSON.stringify({})) {
                redisClient.hdel(vcc + ':MainDomain', vcc);
            }
        }
        catch (err) {
        }
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveDomain = SaveDomain;
function DeleteDomain(vcc, domain) {
    if (!domain || !domain.domain)
        throw 'Se intento eliminar un registro en una clave nula';
    return redisClient.hdelAsync(vcc + ':Domain', domain.domain)
        .then(function (result) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteDomain = DeleteDomain;
function GetDomain(vcc, domainDomain) {
    if (!domainDomain)
        throw 'Se intento consultar un registro en una clave nula';
    return redisClient.hgetAsync(vcc + ':Domain', domainDomain)
        .then(function (result) {
        var domain;
        try {
            domain = JSON.parse(result);
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return Promise.resolve(domain);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetDomain = GetDomain;
function SaveInstanceByDomain(domain, vcc) {
    if (!domain)
        throw 'Se intento guardar un registro en una clave nula';
    return redisClient.hsetAsync('InstanceByDomain', domain, vcc)
        .then(function (result) {
        return redisClient.expireAsync('InstanceByDomain', redis_config_1.REDIS_DATA_PERSISTENCE);
    }).then(function (result) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveInstanceByDomain = SaveInstanceByDomain;
function DeleteInstanceByDomain(domain) {
    if (!domain)
        throw 'Se intento eliminar un registro en una clave nula';
    return redisClient.hdelAsync('InstanceByDomain', domain)
        .then(function (result) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteInstanceByDomain = DeleteInstanceByDomain;
function GetInstanceByDomain(domain) {
    if (!domain)
        throw 'Se intento consultar un registro en una clave nula';
    return redisClient.hgetAsync('InstanceByDomain', domain)
        .then(function (vcc) {
        return Promise.resolve(vcc);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetInstanceByDomain = GetInstanceByDomain;
function SaveMainDomain(vcc, domain) {
    if (!vcc)
        throw 'Se intento guardar un registro en una clave nula';
    return redisClient.hsetAsync(vcc + ':MainDomain', vcc, JSON.stringify(domain))
        .then(function (result) {
        return redisClient.expireAsync(vcc + ':MainDomain', redis_config_1.REDIS_DATA_PERSISTENCE);
    }).then(function (result) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveMainDomain = SaveMainDomain;
function DeleteMainDomain(vcc) {
    if (!vcc)
        throw 'Se intento eliminar un registro en una clave nula';
    return redisClient.hdelAsync(vcc + ':MainDomain', vcc)
        .then(function (result) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteMainDomain = DeleteMainDomain;
function GetMainDomain(vcc) {
    if (!vcc)
        throw 'Se intento consultar un registro en una clave nula';
    return redisClient.hgetAsync(vcc + ':MainDomain', vcc)
        .then(function (result) {
        var mainDomain;
        try {
            mainDomain = JSON.parse(result);
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return Promise.resolve(mainDomain);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetMainDomain = GetMainDomain;
function GetMailingDomainByInstance(vcc) {
    return redisClient.hgetAsync(vcc + ':MailingDomainByInstance', vcc)
        .then(function (result) {
        var mailDomain;
        try {
            mailDomain = JSON.parse(result);
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return Promise.resolve(mailDomain);
    }).catch(function (err) {
        console.dir(err);
        return Promise.reject(err);
    });
}
exports.GetMailingDomainByInstance = GetMailingDomainByInstance;
function SaveMailingDomainByInstance(vcc, domain) {
    try {
        if (vcc && domain) {
            redisClient.hset(vcc + ':MailingDomainByInstance', vcc, JSON.stringify(domain));
            redisClient.expire(vcc + ':MailingDomainByInstance', redis_config_1.REDIS_DATA_PERSISTENCE);
            return Promise.resolve(true);
        }
        else {
            return Promise.resolve(false);
        }
    }
    catch (err) {
        console.dir(err);
        throw err;
    }
}
exports.SaveMailingDomainByInstance = SaveMailingDomainByInstance;
function DeleteMailingDomainByInstance(vcc) {
    return redisClient.hdelAsync(vcc + ':MailingDomainByInstance', vcc)
        .then(function (result) {
        return Promise.resolve(true);
    }).catch(function (err) {
        console.dir(err);
        return Promise.reject(err);
    });
}
exports.DeleteMailingDomainByInstance = DeleteMailingDomainByInstance;
