"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var redis_config_1 = require("../../config/redis.config");
var logger = log4js.getLogger('Redis');
var redisClient;
function ProcessRedis(app) {
    redisClient = app.get('redisClient');
}
exports.ProcessRedis = ProcessRedis;
function GetProcessInteraction(storage, key) {
    return redisClient.hgetAsync(storage, key)
        .then(function (result) {
        var interaction;
        try {
            interaction = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return interaction;
    })
        .catch(function (err) {
        logger.error('[GetInteraction] getting err value:', err, 'hash:', storage, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetProcessInteraction = GetProcessInteraction;
function SaveProcessInteraction(storage, key, processInteraction) {
    if (!key || !storage)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(processInteraction))
        .then(function (result) {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[SaveProcessInteraction] getting error:', err, 'with key:', key);
        return Promise.resolve(false);
    });
}
exports.SaveProcessInteraction = SaveProcessInteraction;
function DeleteProcessInteraction(storage, key) {
    if (!key || !storage)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage, key)
        .then(function (result) {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[DeleteProcessInteraction] getting error:', err, 'with key:', key, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.DeleteProcessInteraction = DeleteProcessInteraction;
function SaveReadyProcessInteraction(executionCode, key, processInteraction) {
    var storage = 'Audios';
    storage += ':';
    storage += executionCode;
    if (!key)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(processInteraction))
        .then(function (result) {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[SaveReadyInteractionProcess] getting error:', err, 'with vcc:', processInteraction.vcc, 'processId:', processInteraction.processId, 'processInteraction:', processInteraction, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.SaveReadyProcessInteraction = SaveReadyProcessInteraction;
function GetReadyProcessInteractions(executionCode) {
    var storage = 'Audios:';
    storage += executionCode;
    return redisClient.hgetallAsync(storage)
        .then(function (result) {
        var interactionProcessToSendList = [];
        try {
            var interactionsToSendResults = (typeof result === 'string') ? JSON.parse(result) : result;
            if (result && interactionsToSendResults) {
                for (var key in interactionsToSendResults) {
                    interactionProcessToSendList.push(JSON.parse(interactionsToSendResults[key]));
                }
            }
        }
        catch (err) {
            logger.error('[GetReadyProcessInteractions] getting err value:', result);
            return Promise.resolve(null);
        }
        return interactionProcessToSendList;
    })
        .catch(function (err) {
        logger.error('[GetReadyProcessInteractions] getting err value:', err);
        return Promise.resolve(err);
    });
}
exports.GetReadyProcessInteractions = GetReadyProcessInteractions;
function GetCsvProcess(storage, key) {
    return redisClient.hgetAsync(storage + ":CsvProcess", key)
        .then(function (result) {
        var interactions;
        try {
            interactions = (typeof result === 'string') ? JSON.parse(result) : result;
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return interactions;
    })
        .catch(function (err) {
        logger.error('[GetCsvProcess] getting err value:', err, 'hash:', storage, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetCsvProcess = GetCsvProcess;
function SaveCsvProcess(storage, key, records) {
    if (!key || !storage)
        return Promise.resolve(false);
    var body = '{"timeInit": "' + new Date() + '","data":' + records + '}';
    return redisClient.hsetAsync(storage + ":CsvProcess", key, body)
        .then(function (result) {
        return redisClient.expireAsync(storage + ":CsvProcess", redis_config_1.REDIS_DATA_PERSISTENCE_ONE_HOUR);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[SaveCsvProcess] getting error:', err, 'with key:', key);
        return Promise.resolve(false);
    });
}
exports.SaveCsvProcess = SaveCsvProcess;
function DeleteCsvProcess(storage, key) {
    if (!key || !storage)
        return Promise.resolve(false);
    return redisClient.hdelAsync(storage + ":CsvProcess", key)
        .then(function (result) {
        return redisClient.expireAsync(storage + ":CsvProcess", redis_config_1.REDIS_DATA_PERSISTENCE);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[DeleteCsvProcess] getting error:', err, 'with key:', key, 'with storage:', storage);
        return Promise.resolve(false);
    });
}
exports.DeleteCsvProcess = DeleteCsvProcess;
