"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var redis_config_1 = require("../../config/redis.config");
var logger = log4js.getLogger('Redis');
var redisClient;
function RoleRedis(app) {
    redisClient = app.get('redisClient');
}
exports.RoleRedis = RoleRedis;
function SaveRole(vcc, role) {
    if (!role || !role.id)
        throw 'Se intento guardar un registro en una clave nula';
    return redisClient.hsetAsync(vcc + ':Role', role.id, JSON.stringify(role))
        .then(function (result) {
        return redisClient.expireAsync(vcc + ':Role', redis_config_1.REDIS_DATA_PERSISTENCE);
    }).then(function (result) {
        return redisClient.hgetallAsync(vcc + ':User');
    }).then(function (users) {
        for (var userId in users) {
            try {
                users[userId] = JSON.parse(users[userId]);
            }
            catch (err) {
                users[userId] = {};
            }
            if (users[userId].role === role.id || JSON.stringify(users[userId]) === JSON.stringify({})) {
                redisClient.hdel(vcc + ':User', userId);
            }
        }
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveRole = SaveRole;
function DeleteRole(vcc, role) {
    if (!role || !role.id)
        throw 'Se intento eliminar un registro en una clave nula';
    return redisClient.hdelAsync(vcc + ':Role', role.id)
        .then(function (result) {
        return redisClient.hgetallAsync(vcc + ':User');
    }).then(function (users) {
        for (var userId in users) {
            try {
                users[userId] = JSON.parse(users[userId]);
            }
            catch (err) {
                users[userId] = {};
            }
            if (users[userId].role === role.id || JSON.stringify(users[userId]) === JSON.stringify({})) {
                redisClient.hdel(vcc + ':User', userId);
            }
        }
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteRole = DeleteRole;
function GetRole(vcc, roleId) {
    if (!roleId)
        throw 'Se intento consultar un registro en una clave nula';
    return redisClient.hgetAsync(vcc + ':Role', roleId)
        .then(function (result) {
        var role;
        try {
            role = JSON.parse(result);
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return Promise.resolve(role);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetRole = GetRole;
