"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var redis_config_1 = require("../../config/redis.config");
var logger = log4js.getLogger('Redis');
var redisClient;
function SessionRedis(app) {
    redisClient = app.get('redisClient');
}
exports.SessionRedis = SessionRedis;
function SaveSession(vcc, userId, session) {
    if (!userId)
        throw 'Se intento guardar un registro en una clave nula';
    return redisClient.hsetAsync(vcc + ':Session', userId, session)
        .then(function (result) {
        return redisClient.expireAsync(vcc + ':Session', redis_config_1.REDIS_SESSION_PERSISTENCE);
    }).then(function (result) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveSession = SaveSession;
function DeleteSession(vcc, userId) {
    if (!userId)
        throw 'Se intento eliminar un registro en una clave nula';
    return redisClient.hdelAsync(vcc + ':Session', userId)
        .then(function (result) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteSession = DeleteSession;
function GetSession(vcc, userId) {
    if (!userId)
        throw 'Se intento consultar un registro en una clave nula';
    return redisClient.hgetAsync(vcc + ':Session', userId)
        .then(function (session) {
        return Promise.resolve(session);
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetSession = GetSession;
