"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var billing_batch_implementation_1 = require("../implementation/billing-batch.implementation");
var log4js = require("log4js");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('Task');
var BillingBatchStatusImplementation = (function () {
    function BillingBatchStatusImplementation() {
    }
    BillingBatchStatusImplementation.prototype.Execute = function (executionCode) {
        var _this = this;
        var executeIndex = 1;
        var executeResult = true;
        var genericResult = {};
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.debug('[BillingBatchStatusImplementation::Execute] Requesting data');
            var billing = new billing_batch_implementation_1.BillingBatchImplementation();
            return billing.GetCurrentBatches();
        })
            .then(function (result) {
            if (result) {
                if (result.status === true) {
                    genericResult.status = true;
                    genericResult.data = {
                        allowedBatches: _this.ProcessResult(result.data['allowedBatches']),
                        cancelledBatches: _this.ProcessResult(result.data['cancelledBatches']),
                    };
                }
                else {
                    genericResult.status = false;
                    genericResult.description = 'SERVER_ERROR_CURRENT_BATCHES_REQUEST_FAILED';
                }
            }
            else {
                genericResult.status = false;
                genericResult.description = 'SERVER_ERROR_CURRENT_BATCHES_REQUEST_FAILED';
            }
            return Promise.resolve(genericResult);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[BillingBatchStatusImplementation::Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            genericResult.status = false;
            genericResult.description = errorMessage;
            return Promise.resolve(genericResult);
        });
    };
    BillingBatchStatusImplementation.prototype.ProcessResult = function (batches) {
        var allowedBatches = {};
        allowedBatches['transcription'] = batches.filter(function (b) { return b.SystemService === 'TRANSCRIPTION'; });
        allowedBatches['semanticAnalysis'] = batches.filter(function (b) { return b.SystemService === 'SEMANTIC_ANALYSIS'; });
        allowedBatches['cognitiveAnalysis'] = batches.filter(function (b) { return b.SystemService === 'COGNITIVE_ANALYSIS'; });
        return allowedBatches;
    };
    return BillingBatchStatusImplementation;
}());
exports.BillingBatchStatusImplementation = BillingBatchStatusImplementation;
