"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var billing_implementation_1 = require("./billing.implementation");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var platform_service_1 = require("../services/platform.service");
var icModule = require('intermediate-component-sa');
var logger = log4js.getLogger('ServerScripts');
var BillingBatchImplementation = (function () {
    function BillingBatchImplementation() {
    }
    BillingBatchImplementation.prototype.CancelBatchSummary = function (batchManage) {
        var genericResult = {};
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        })
            .then(function (result) {
            logger.info('[BillingBatchImplementation::CancelBatchSummary] Getting result after GetVersion');
            if (result && result.installationId) {
                var body = {
                    batchManage: batchManage
                };
                return icModule.InconcertRequest(result.installationId, 'IC_PARAM_URL_BATCH_SUMMARY_CANCEL', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(function (result) {
            if (result) {
                if (result.status === true) {
                    genericResult.status = true;
                    genericResult.data = result.data;
                }
                else {
                    genericResult.status = false;
                    genericResult.description = 'ERROR_FAILED_SEARCH';
                }
            }
            else {
                genericResult.status = false;
                genericResult.description = 'ERROR_FAILED_SEARCH';
            }
            return Promise.resolve(genericResult);
        })
            .catch(function (err) {
            logger.error('[BillingBatchImplementation::CancelBatchSummary] Getting error:', err);
            genericResult.status = false;
            genericResult.description = err;
            return Promise.resolve(genericResult);
        });
    };
    BillingBatchImplementation.prototype.GetCurrentBatches = function () {
        var genericResult = {};
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        })
            .then(function (result) {
            logger.info('[BillingBatchImplementation::GetCurrentBatches] Getting result after GetVersion');
            if (result && result.installationId) {
                var body = {};
                return icModule.InconcertRequest(result.installationId, 'IC_PARAM_URL_CURRENT_BATCHES_GET', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(function (result) {
            if (result) {
                if (result.status === true) {
                    genericResult.status = true;
                    genericResult.data = result.data;
                }
                else {
                    genericResult.status = false;
                    genericResult.description = 'ERROR_FAILED_SEARCH';
                }
            }
            else {
                genericResult.status = false;
                genericResult.description = 'ERROR_FAILED_SEARCH';
            }
            return Promise.resolve(genericResult);
        })
            .catch(function (err) {
            logger.error('[BillingBatchImplementation::GetCurrentBatches] Getting error:', err);
            genericResult.status = false;
            genericResult.description = err;
            return Promise.resolve(genericResult);
        });
    };
    BillingBatchImplementation.prototype.GetCancelledBatches = function () {
        var genericResult = {};
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        })
            .then(function (result) {
            logger.info('[BillingBatchImplementation::GetCurrentBatches] Getting result after GetVersion');
            if (result && result.installationId) {
                var body = {};
                return icModule.InconcertRequest(result.installationId, 'IC_PARAM_URL_CANCELLED_BATCHES_GET', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(function (result) {
            if (result) {
                if (result.status === true) {
                    genericResult.status = true;
                    genericResult.data = result.data;
                }
                else {
                    genericResult.status = false;
                    genericResult.description = 'ERROR_FAILED_SEARCH';
                }
            }
            else {
                genericResult.status = false;
                genericResult.description = 'ERROR_FAILED_SEARCH';
            }
            return Promise.resolve(genericResult);
        })
            .catch(function (err) {
            logger.error('[BillingBatchImplementation::GetCancelledBatches] Getting error:', err);
            genericResult.status = false;
            genericResult.description = err;
            return Promise.resolve(genericResult);
        });
    };
    BillingBatchImplementation.prototype.GetBatchBalanceDetail = function (batchDetail) {
        var genericResult = {};
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        })
            .then(function (result) {
            logger.info('[BillingBatchImplementation::GetBatchBalanceDetail] Getting result after GetVersion');
            if (result && result.installationId) {
                var body = {
                    batchDetail: batchDetail
                };
                return icModule.InconcertRequest(result.installationId, 'IC_PARAM_URL_BATCH_BALANCE_DETAIL', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(function (result) {
            if (result) {
                if (result.status === true) {
                    genericResult.status = true;
                    genericResult.data = result.data;
                }
                else {
                    genericResult.status = false;
                    genericResult.description = 'ERROR_FAILED_SEARCH';
                }
            }
            else {
                genericResult.status = false;
                genericResult.description = 'ERROR_FAILED_SEARCH';
            }
            return Promise.resolve(genericResult);
        })
            .catch(function (err) {
            logger.error('[BillingBatchImplementation::GetBatchBalanceDetail] Getting error:', err);
            genericResult.status = false;
            genericResult.description = err;
            return Promise.resolve(genericResult);
        });
    };
    BillingBatchImplementation.prototype.FindBatchId = function (billingBatch) {
        var genericResult = {};
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        })
            .then(function (result) {
            logger.info('[BillingBatchImplementation::FindBatchId] Getting result after GetVersion');
            if (result && result.installationId) {
                var body = {
                    batch: billingBatch
                };
                return icModule.InconcertRequest(result.installationId, 'IC_PARAM_URL_BATCH_ID_SEARCH', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(function (result) {
            if (result) {
                if (result.data && result.data.length > 0) {
                    genericResult.status = true;
                    genericResult.result = result.data;
                }
                else {
                }
            }
            else {
                genericResult.status = false;
                genericResult.description = 'ERROR_FAILED_SEARCH';
            }
            return Promise.resolve(genericResult);
        })
            .catch(function (err) {
            logger.error('[BillingBatchImplementation::FindBatchId] Getting error:', err);
            genericResult.status = false;
            genericResult.description = err;
            return Promise.resolve(genericResult);
        });
    };
    BillingBatchImplementation.prototype.ResumeBatchSummary = function (batchManage) {
        var genericResult = {};
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        })
            .then(function (result) {
            logger.info('[BillingBatchImplementation::ResumeBatchSummary] Getting result after GetVersion');
            if (result && result.installationId) {
                var body = {
                    batchManage: batchManage
                };
                return icModule.InconcertRequest(result.installationId, 'IC_PARAM_URL_BATCH_SUMMARY_RESUME', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(function (result) {
            if (result) {
                if (result.status === true) {
                    genericResult.status = true;
                    genericResult.data = result.data;
                }
                else {
                    genericResult.status = false;
                    genericResult.description = 'ERROR_FAILED_SEARCH';
                }
            }
            else {
                genericResult.status = false;
                genericResult.description = 'ERROR_FAILED_SEARCH';
            }
            return Promise.resolve(genericResult);
        })
            .catch(function (err) {
            logger.error('[BillingBatchImplementation::ResumeBatchSummary] Getting error:', err);
            genericResult.status = false;
            genericResult.description = err;
            return Promise.resolve(genericResult);
        });
    };
    BillingBatchImplementation.prototype.SaveBatchSummary = function (billingBatchSummary) {
        var billing = new billing_implementation_1.BillingImplementation();
        var installationId = '';
        var systemInformationData = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        })
            .then(function (result) {
            logger.info('[BillingBatchImplementation::SaveBatchSummary] Getting result:', result, 'after GetVersion');
            if (result && result.installationId) {
                installationId = result.installationId;
                return billing.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(function (result) {
            systemInformationData = result;
            var plaformHash = '';
            if (systemInformationData) {
                plaformHash = billing.GenerateMD5Content(systemInformationData);
            }
            var body = {
                platformHash: plaformHash,
                batch: billingBatchSummary
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_BATCH_SUMMARY_CREATE', body);
        })
            .then(function (result) {
            return Promise.resolve(result);
        })
            .catch(function (err) {
            logger.error('[BillingBatchImplementation::SaveBatchSummary] Getting error:', err);
            return Promise.resolve(err);
        });
    };
    BillingBatchImplementation.prototype.CloseBatchSummary = function (billingBatchFinished) {
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        })
            .then(function (result) {
            logger.info('[BillingBatchImplementation::CloseBatchSummary] Getting result after GetVersion');
            if (result && result.installationId) {
                var body = {
                    batch: billingBatchFinished
                };
                return icModule.InconcertRequest(result.installationId, 'IC_PARAM_URL_BATCH_SUMMARY_CLOSE', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(function (result) {
            if (result && result.data && result.data.result) {
                return mssql_service_1.MSSql.FinishedBatchClose(billingBatchFinished.vcc, billingBatchFinished.batchTypeName, billingBatchFinished.batchTypeId);
            }
            return Promise.resolve(true);
        })
            .catch(function (err) {
            logger.error('[BillingBatchImplementation::CloseBatchSummary] Getting error:', err);
            return Promise.resolve(false);
        });
    };
    return BillingBatchImplementation;
}());
exports.BillingBatchImplementation = BillingBatchImplementation;
