"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var config_1 = require("../config/config");
var promises_shared_1 = require("../shared/promises.shared");
var redis_service_1 = require("../db/redis.service");
var mssql_service_1 = require("../db/mssql.service");
var file_manager_1 = require("../shared/file-manager");
var logger = log4js.getLogger('Task');
var CleanerImplementation = (function () {
    function CleanerImplementation() {
    }
    CleanerImplementation.prototype.Execute = function (executionCode) {
        var totalProgress = [];
        var progress = [];
        var errorProgress = [];
        var summaryProgress = [];
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return redis_service_1.Redis.GetReadyProcessInteractions(executionCode);
        }).then(function (interactions) {
            logger.info('[CleanerImplementation:Execute] executionCode:', executionCode, 'getting: interactions:', (interactions && interactions.length ? interactions.length : 0), 'after Redis.GetReadyProcessInteractions');
            if (interactions && interactions.length) {
                return mssql_service_1.MSSql.BulkProcessInteractionTemp(interactions);
            }
            return true;
        }).then(function (result) {
            var hash = 'Audios:' + executionCode;
            return redis_service_1.Redis.Del(hash, executionCode);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation:Execute] executionCode:', executionCode, 'BulkProcessInteractionTemp getting error:', errorMessage);
            return false;
        }).then(function (result) {
            return redis_service_1.Redis.GetReadyCognitiveInteractions(executionCode);
        }).then(function (interactions) {
            logger.info('[CleanerImplementation:Execute] executionCode:', executionCode, 'getting: interactions:', (interactions && interactions.length ? interactions.length : 0), 'after Redis.GetReadyCognitiveInteractions');
            if (interactions && interactions.length) {
                return mssql_service_1.MSSql.BulkInteractionCognitiveTemp(interactions);
            }
            return true;
        }).then(function (result) {
            var hash = 'Cognitive:' + executionCode;
            return redis_service_1.Redis.Del(hash, executionCode);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation:Execute] executionCode:', executionCode, 'BulkInteractionCognitiveTemp getting error:', errorMessage);
            return false;
        }).then(function (result) {
            return redis_service_1.Redis.GetReadyCognitiveAbilities(executionCode);
        }).then(function (interactions) {
            logger.info('[CleanerImplementation:Execute] executionCode:', executionCode, 'getting: interactions:', (interactions && interactions.length ? interactions.length : 0), 'after Redis.GetReadyCognitiveAbilities');
            if (interactions && interactions.length) {
                logger.debug("[CleanerImplementation:Execute] BulkAbilityCognitiveTemp:", JSON.stringify(interactions));
                return mssql_service_1.MSSql.BulkAbilityCognitiveTemp(interactions);
            }
            return true;
        }).then(function (result) {
            var hash = 'AbilityCognitive:' + executionCode;
            return redis_service_1.Redis.Del(hash, executionCode);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation:Execute] executionCode:', executionCode, 'BulkAbilityCognitiveTemp getting error:', errorMessage);
            return false;
        }).then(function (result) {
            return redis_service_1.Redis.GetTotalProgress();
        }).then(function (result) {
            totalProgress = result;
            return redis_service_1.Redis.GetErrorProgress();
        }).then(function (result) {
            errorProgress = result;
            return redis_service_1.Redis.GetProgress();
        }).then(function (result) {
            progress = result;
            if (totalProgress && totalProgress.length) {
                totalProgress.forEach(function (total) {
                    var currentProgress = progress.find(function (item) { return item.key === total.key; });
                    var currentErrorProgress = errorProgress.find(function (item) { return item.key === total.key; });
                    total.value = total.value ? total.value : 0;
                    currentProgress.value = currentProgress && currentProgress.value ? currentProgress.value : 0;
                    var realProgress = total.value - currentProgress.value;
                    var summaryItem = JSON.parse(JSON.stringify(total));
                    summaryItem['progress'] = realProgress;
                    summaryItem['percentage'] = currentProgress.value <= 0 ? 1 : Math.round((realProgress / total.value) * 100000) / 100000;
                    summaryItem['error'] = currentErrorProgress && currentErrorProgress.value ? currentErrorProgress.value : 0;
                    logger.debug('[CleanerImplementation:Execute] executionCode:', executionCode, 'RealProgress: ', realProgress);
                    logger.debug('[CleanerImplementation:Execute] executionCode:', executionCode, 'TotalValue: ', total.value);
                    logger.debug('[CleanerImplementation:Execute] executionCode:', executionCode, 'CurrentProgress: ', currentProgress.value);
                    logger.debug('[CleanerImplementation:Execute] executionCode:', executionCode, 'CurrentErrorProgress: ', currentErrorProgress && currentErrorProgress.value ? currentErrorProgress.value : 0);
                    logger.debug('[CleanerImplementation:Execute] executionCode:', executionCode, 'Progress: ', summaryItem['progress']);
                    logger.debug('[CleanerImplementation:Execute] executionCode:', executionCode, 'Percentaje: ', summaryItem['percentage']);
                    logger.debug('[CleanerImplementation:Execute] executionCode:', executionCode, 'Error: ', summaryItem['error']);
                    summaryProgress.push(summaryItem);
                });
            }
            return mssql_service_1.MSSql.BulkProgressTemp(summaryProgress);
        }).then(function (result) {
            var promiseArray = [];
            var readyProgress = summaryProgress.filter(function (item) { return item.percentage === 1; });
            if (readyProgress && readyProgress.length) {
                readyProgress.forEach(function (item) {
                    logger.info('[CleanerImplementation:Execute] executionCode:', executionCode, 'FINALIZED', item);
                    promiseArray.push(redis_service_1.Redis.DeleteTotalProgress(item.key));
                    promiseArray.push(redis_service_1.Redis.DeleteProgress(item.key));
                    promiseArray.push(redis_service_1.Redis.DeleteErrorProgress(item.key));
                    var batchName = null;
                    if (item.type === null) {
                        batchName = 'TRANSCRIPTION';
                        item.typeId = item.processId;
                    }
                    if (item.type === 'study') {
                        batchName = 'STUDY';
                        promiseArray.push(redis_service_1.Redis.RemoveStudyTemplateThemes(item.vcc, item.typeId));
                    }
                    if (item.type === 'ability') {
                        batchName = 'EVALUATION_PLAN';
                        promiseArray.push(redis_service_1.Redis.RemoveEvaluationPlanAbilities(item.vcc, item.typeId));
                    }
                    if (batchName)
                        promiseArray.push(redis_service_1.Redis.DeleteBatchRasaModels(batchName.toUpperCase() + '::' + item.typeId));
                });
            }
            return Promise.all(promiseArray);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation:Execute] executionCode:', executionCode, 'Progress getting error:', errorMessage);
            return false;
        }).then(function (result) {
            logger.info('[CleanerImplementation:Execute] executionCode: CleanupTempFile', executionCode, 'getting result:', 'DELETED FILES');
            var promiseArray = [];
            promiseArray.push(file_manager_1.CleanupTempFile(config_1.PATH_FOLDER_TEMP_AUDIO, config_1.PERSISTENCE_AUDIO_TEMP));
            return Promise.all(promiseArray);
        }).then(function (result) {
            logger.info('[CleanerImplementation:Execute] executionCode:', executionCode, 'getting result:', 'TASK FINISHED');
            return Promise.resolve(true);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation:Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    return CleanerImplementation;
}());
exports.CleanerImplementation = CleanerImplementation;
