"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var fs = require("fs");
var mssql_service_1 = require("../db/mssql.service");
var config_1 = require("../config/config");
var redis_service_1 = require("../db/redis.service");
var file_manager_1 = require("../shared/file-manager");
var promises_shared_1 = require("../shared/promises.shared");
var process_implementation_1 = require("../implementation/process.implementation");
var billing_implementation_1 = require("../implementation/billing.implementation");
var Client = require('ftp');
var logger = log4js.getLogger('Task');
var DiarizationConvertImplementation = (function () {
    function DiarizationConvertImplementation() {
    }
    DiarizationConvertImplementation.prototype.Execute = function (executionCode, rateRecursive) {
        var _this = this;
        var executeIndex = 1;
        var ratio = config_1.RATIO_DIARIZATION_CONVERT;
        var executeResult = true;
        return promises_shared_1.SleepPromise(config_1.SLEEP_DIARIZATION_CONVERT)
            .then(function (result) {
            return redis_service_1.Redis.LRange('DiarizationConvert', 0, ratio - 1);
        }).then(function (result) {
            var promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            while (ratio >= executeIndex) {
                promiseArray.push(_this.DiarizationConvertExecute(executionCode));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('DiarizationConvert', 0, ratio - 1);
        }).then(function (result) {
            if (result && result.length && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1);
            return executeResult;
        }).then(function (result) {
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[DiarizationConvertImplementation::Execute] executionCode:', executionCode, 'getting error results:', errorMessage);
            return Promise.resolve(false);
        });
    };
    DiarizationConvertImplementation.prototype.DiarizationConvertExecute = function (executionCode) {
        var _this = this;
        var currentBlockPromise = 'BLOCK_BEFORE_PROMISE';
        var storage = '';
        var currentQueueId = '';
        var processInteraction;
        var isCancel = false;
        var vcc = '';
        var processId = '';
        var interactionType = '';
        var interactionId = '';
        var interactionSection = 0;
        var interactionPart = 0;
        var batchTypeName = '';
        var batchTypeId = '';
        return redis_service_1.Redis.RPopLPush('DiarizationConvert', 'DiarizationConvertExecute')
            .then(function (queueId) {
            currentBlockPromise = 'BLOCK_TO_GET_PROCESS_INTERACTION';
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                var item = queueId.split(':');
                if (item && item.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    interactionId = item[3];
                    interactionSection = Number(item[4]);
                    interactionPart = Number(item[5]);
                    storage = vcc + ':' + processId;
                    logger.info('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
                }
                else {
                    logger.error('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return null;
        }).then(function (processInteractionResult) {
            currentBlockPromise = 'BLOCK_TO_LAUNCH_MAIN_FUNCTION';
            if (processInteractionResult) {
                processInteraction = processInteractionResult;
                if (processInteraction.statusDescription === 'pendingUploadRawAudio') {
                    return _this.UploadRawAudio(processInteraction, currentQueueId, executionCode);
                }
                else if (processInteraction.statusDescription === 'pendingLaunchAudioConverter') {
                    return _this.LaunchAudioConverter(processInteraction, currentQueueId, executionCode);
                }
                else if (processInteraction.statusDescription === 'pendingDownloadConvertedAudio') {
                    return _this.DownloadConvertedAudio(processInteraction, currentQueueId, executionCode);
                }
                return null;
            }
            else {
                if (currentQueueId)
                    logger.warn('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return null;
            }
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_TO_UPDATE_INTERACTION';
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return null;
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_TO_LAUNCH_FINISHING_FUNCTION';
            var promiseArray = [];
            var billing = new billing_implementation_1.BillingImplementation();
            if (result
                && processInteraction
                && processInteraction.status === 2
                && processInteraction.statusDescription === 'pendingUpload') {
                logger.info('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_QUEUE');
                promiseArray.push(redis_service_1.Redis.CreateQueueL('Transcriptions', currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationConvertExecute', currentQueueId));
                var process_1 = new process_implementation_1.ProcessImplementation();
                promiseArray.push(process_1.RunCurrentBatchesRasaModels(vcc, 'TRANSCRIPTION', processId, ['diarization-' + processInteraction.language], executionCode, currentQueueId));
            }
            else if (result
                && processInteraction
                && (processInteraction.status === 1
                    ||
                        (processInteraction.status === 2 && processInteraction.statusDescription !== 'pendingUpload'))
                && processInteraction.retry < config_1.RETRY_DIARIZATION_CONVERT) {
                promiseArray.push(redis_service_1.Redis.InsertQueue('DiarizationConvert', currentQueueId, config_1.RATE_RECURSIVE_DIARIZATION_CONVERT * config_1.RATIO_DIARIZATION_CONVERT * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationConvertExecute', currentQueueId));
            }
            else if (result
                && processInteraction
                && processInteraction.status === -2
                && processInteraction.statusDescription === 'REMOVED_BY_CANCELLED') {
                logger.error('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationConvertExecute', currentQueueId));
            }
            else if (processInteraction && processInteraction.retry >= config_1.RETRY_DIARIZATION_CONVERT) {
                logger.error('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationConvertExecute', currentQueueId));
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                var interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, batchTypeName, batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationConvertExecute', currentQueueId));
            }
            else {
                logger.error('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.resolve(true);
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_TO_UPDATE_REDIS';
            var promiseArray = [];
            if (processInteraction && processInteraction.retry >= config_1.RETRY_DIARIZATION_CONVERT) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_TO_FINISH_EXECUTION';
            return Promise.resolve(true);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[DiarizationConvertImplementation::DiarizationConvertExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    };
    DiarizationConvertImplementation.prototype.UploadRawAudio = function (interaction, currentQueueId, executionCode) {
        var filePath = config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + interaction.vcc + '/' + interaction.fileName;
        var uploadFtpFolder = config_1.CONVERTER_FTP_UPLOAD_FOLDER_DIARIZATION_CONVERT;
        var uploadFtpPath = uploadFtpFolder + '/' + interaction.fileName;
        var downloadFtpFolder = config_1.CONVERTER_FTP_DOWNLOAD_FOLDER_DIARIZATION_CONVERT;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return file_manager_1.ExistInFileSystem(filePath);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_AUDIO_NOT_FOUND';
            return new Promise(function (resolve, reject) {
                var clientFtp = new Client();
                clientFtp.connect({
                    host: config_1.CONVERTER_FTP_IP_DIARIZATION_CONVERT,
                    port: config_1.CONVERTER_FTP_PORT_DIARIZATION_CONVERT,
                    user: config_1.CONVERTER_FTP_USER_DIARIZATION_CONVERT,
                    password: config_1.CONVERTER_FTP_PASSWORD_DIARIZATION_CONVERT,
                    connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
                    pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT
                });
                if (clientFtp) {
                    clientFtp.on('greeting', function (message) {
                        logger.info('[DiarizationConvertImplementation::UploadRawAudio] executionCode:', executionCode, 'getting on greeting event message:', message);
                    });
                    clientFtp.on('ready', function () {
                        clientFtp.mkdir(uploadFtpFolder, true, function (err) {
                            if (err) {
                                clientFtp.end();
                                logger.error('[DiarizationConvertImplementation::UploadRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_INPUT_FOLDER_CREATE_FAILED:', err);
                                return resolve('SERVER_ERROR_INPUT_FOLDER_CREATE_FAILED');
                            }
                            else {
                                logger.info('[DiarizationConvertImplementation::UploadRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_EVENT_INPUT_FOLDER_CREATE_SUCCED:', err);
                                clientFtp.put(filePath, uploadFtpPath, function (err) {
                                    if (err) {
                                        clientFtp.end();
                                        logger.error('[DiarizationConvertImplementation::UploadRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_AUDIO_UPLOAD_FAILED:', err);
                                        return resolve('SERVER_ERROR_AUDIO_UPLOAD_FAILED');
                                    }
                                    else {
                                        logger.info('[DiarizationConvertImplementation::UploadRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_EVENT_AUDIO_UPLOAD_SUCCED:', err);
                                        clientFtp.mkdir(uploadFtpFolder, true, function (err) {
                                            if (err) {
                                                clientFtp.end();
                                                logger.error('[DiarizationConvertImplementation::UploadRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_OUTPUT_FOLDER_CREATE_FAILED:', err);
                                                return resolve('SERVER_ERROR_INPUT_FOLDER_CREATE_FAILED');
                                            }
                                            else {
                                                logger.info('[DiarizationConvertImplementation::UploadRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_EVENT_OUTPUT_FOLDER_CREATE_SUCCED:', err);
                                                clientFtp.end();
                                                return resolve('OK');
                                            }
                                        });
                                    }
                                });
                            }
                        });
                    });
                    clientFtp.on('end', function () {
                        var eventMessage = 'FTP_CONNECTION_ENDED';
                        logger.warn('[DiarizationConvertImplementation::UploadRawAudio] executionCode:', executionCode, 'getting eventMessage:', eventMessage);
                        return resolve(eventMessage);
                    });
                    clientFtp.on('close', function (hadError) {
                        var eventMessage = hadError ? 'FTP_CONNECTION_CLOSE_DUE_ERROR' : 'FTP_CONNECTION_CLOSE';
                        logger.warn('[DiarizationConvertImplementation::UploadRawAudio] executionCode:', executionCode, 'getting eventMessage:', eventMessage, 'hadError:', hadError);
                        return resolve(eventMessage);
                    });
                    clientFtp.on('error', function (err) {
                        logger.error('[DiarizationConvertImplementation::UploadRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_NOT_FTP_CLIENT', err);
                        return resolve('SERVER_ERROR_NOT_FTP_CLIENT');
                    });
                }
                else {
                    logger.error('[DiarizationConvertImplementation::UploadRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_NOT_FTP_CLIENT');
                    return resolve('SERVER_ERROR_NOT_FTP_CLIENT');
                }
            });
        }).then(function (result) {
            if (result === 'OK') {
                interaction['status'] = 2;
                interaction['statusDescription'] = 'pendingLaunchAudioConverter';
                interaction['retry'] = 0;
                interaction['errorMessage'] = null;
            }
            else {
                logger.error('[DiarizationConvertImplementation::UploadRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -2;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorMessage'] = result;
            }
            return Promise.resolve(interaction);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[DiarizationConvertImplementation::UploadRawAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -2;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorMessage'] = errorMessage;
            return Promise.resolve(interaction);
        });
    };
    DiarizationConvertImplementation.prototype.LaunchAudioConverter = function (interaction, currentQueueId, executionCode) {
        var ftpConverterApplicationFolder = config_1.CONVERTER_FTP_CONVERTER_APP_PATH;
        var ftpInputFileFolder = config_1.CONVERTER_FTP_ROOT_FOLDER_DIARIZATION_CONVERT + config_1.CONVERTER_FTP_UPLOAD_FOLDER_DIARIZATION_CONVERT;
        var ftpOutputFileFolder = config_1.CONVERTER_FTP_ROOT_FOLDER_DIARIZATION_CONVERT + config_1.CONVERTER_FTP_DOWNLOAD_FOLDER_DIARIZATION_CONVERT;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.LaunchAudioConverter(interaction.vcc, ftpConverterApplicationFolder, ftpInputFileFolder + '/' + interaction.fileName, ftpOutputFileFolder);
        }).then(function (result) {
            if (result && (result.errcode || result.errcode === 0)) {
                switch (result.errcode) {
                    case 0: return Promise.resolve('OK');
                    case -1: throw 'SERVER_ERROR_DIARIZATION_FAILED';
                    case -2: throw 'SERVER_ERROR_CHANNEL_SPLIT_FAILED';
                    case -3: throw 'SERVER_ERROR_FFMPEG_FUNCTION_FAILED';
                    case -100: throw 'SERVER_ERROR_FILE_NOT_FOUND';
                    default: throw 'SERVER_ERROR_UNKNOWN_ERROR';
                }
            }
            else {
                throw 'SERVER_ERROR_CONVERTER_AUDIO_SP_FAILED';
            }
        }).then(function (result) {
            if (result === 'OK') {
                interaction['status'] = 2;
                interaction['statusDescription'] = 'pendingDownloadConvertedAudio';
                interaction['retry'] = 0;
                interaction['errorMessage'] = null;
            }
            else {
                logger.error('[DiarizationConvertImplementation::LaunchAudioConverter] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -2;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorMessage'] = result;
            }
            return Promise.resolve(interaction);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[DiarizationConvertImplementation::LaunchAudioConverter] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -2;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorMessage'] = errorMessage;
            return Promise.resolve(interaction);
        });
    };
    DiarizationConvertImplementation.prototype.DownloadConvertedAudio = function (interaction, currentQueueId, executionCode) {
        var filePath = config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + interaction.vcc + '/' + interaction.fileName;
        var tmpPath = filePath + '-TMP';
        var uploadFtpFolder = config_1.CONVERTER_FTP_UPLOAD_FOLDER_DIARIZATION_CONVERT;
        var uploadFtpPath = uploadFtpFolder + '/' + interaction.fileName;
        var downloadFtpFolder = config_1.CONVERTER_FTP_DOWNLOAD_FOLDER_DIARIZATION_CONVERT;
        var downloadFtpPath = downloadFtpFolder + '/' + interaction.fileName;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return file_manager_1.ExistInFileSystem(filePath);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_AUDIO_NOT_FOUND';
            return file_manager_1.RenameFile(filePath, tmpPath);
        }).then(function (result) {
            return new Promise(function (resolve, reject) {
                var clientFtp = new Client();
                clientFtp.connect({
                    host: config_1.CONVERTER_FTP_IP_DIARIZATION_CONVERT,
                    port: config_1.CONVERTER_FTP_PORT_DIARIZATION_CONVERT,
                    user: config_1.CONVERTER_FTP_USER_DIARIZATION_CONVERT,
                    password: config_1.CONVERTER_FTP_PASSWORD_DIARIZATION_CONVERT,
                    connTimeout: config_1.FTP_CONNECTION_TIMEOUT,
                    pasvTimeout: config_1.FTP_CONNECTION_TIMEOUT
                });
                if (clientFtp) {
                    clientFtp.on('greeting', function (message) {
                        logger.info('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting on greeting event message:', message);
                    });
                    clientFtp.on('ready', function () {
                        clientFtp.list(downloadFtpFolder, function (err, list) {
                            if (err)
                                return resolve(err.message);
                            if (list.find(function (l) { return l && l.name && l.name === interaction.fileName; })) {
                                clientFtp.get(downloadFtpPath, function (err, stream) {
                                    if (err) {
                                        clientFtp.end();
                                        logger.error('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_AUDIO_FILE_GET_FTP:', err);
                                        return resolve('SERVER_ERROR_AUDIO_FILE_GET_FTP');
                                    }
                                    else {
                                        var file_1 = fs.createWriteStream(filePath);
                                        stream.pipe(fs.createWriteStream(filePath));
                                        stream.once('close', function () {
                                            logger.info('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, ' DOWNLOADED_CONVERTED_AUDIO');
                                            clientFtp.delete(uploadFtpPath, function (err) {
                                                if (err) {
                                                    clientFtp.end();
                                                    logger.error('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_FTP_INPUT_AUDIO_DELETE_FAILED:', err);
                                                    return resolve('SERVER_ERROR_FTP_INPUT_AUDIO_DELETE_FAILED');
                                                }
                                                else {
                                                    logger.info('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, ' DELETED_RAW_AUDIO');
                                                    clientFtp.delete(downloadFtpPath, function (err) {
                                                        if (err) {
                                                            clientFtp.end();
                                                            logger.error('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_FTP_OUTPUT_AUDIO_DELETE_FAILED:', err);
                                                            return resolve('SERVER_ERROR_FTP_OUTPUT_AUDIO_DELETE_FAILED');
                                                        }
                                                        else {
                                                            logger.info('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, ' DELETED_CONVERTED_AUDIO');
                                                            clientFtp.end();
                                                            file_1.close();
                                                            return resolve('OK');
                                                        }
                                                    });
                                                }
                                            });
                                        });
                                    }
                                });
                            }
                            else {
                                return resolve('SERVER_ERROR_AUDIO_TO_DOWNLOAD_NOT_FOUND');
                            }
                        });
                    });
                    clientFtp.on('end', function () {
                        var eventMessage = 'FTP_CONNECTION_ENDED';
                        logger.warn('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting eventMessage:', eventMessage);
                        return resolve(eventMessage);
                    });
                    clientFtp.on('close', function (hadError) {
                        var eventMessage = hadError ? 'FTP_CONNECTION_CLOSE_DUE_ERROR' : 'FTP_CONNECTION_CLOSE';
                        logger.warn('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting eventMessage:', eventMessage, 'hadError:', hadError);
                        return resolve(eventMessage);
                    });
                    clientFtp.on('error', function (err) {
                        logger.error('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_NOT_FTP_CLIENT', err);
                        return resolve('SERVER_ERROR_NOT_FTP_CLIENT');
                    });
                }
                else {
                    logger.error('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'SERVER_ERROR_NOT_FTP_CLIENT');
                    return resolve('SERVER_ERROR_NOT_FTP_CLIENT');
                }
            });
        }).then(function (result) {
            if (result === 'OK') {
                interaction['status'] = 2;
                interaction['statusDescription'] = 'pendingUpload';
                interaction['isDiariced'] = true;
                interaction['retry'] = 0;
                interaction['errorMessage'] = null;
                return file_manager_1.DeleteFile(tmpPath);
            }
            else {
                logger.error('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -2;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorMessage'] = result;
                return file_manager_1.RenameFile(tmpPath, filePath);
            }
        }).then(function (result) {
            return Promise.resolve(interaction);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[DiarizationConvertImplementation::DownloadConvertedAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -2;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorMessage'] = errorMessage;
            return Promise.resolve(interaction);
        });
    };
    return DiarizationConvertImplementation;
}());
exports.DiarizationConvertImplementation = DiarizationConvertImplementation;
