"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var sy = require("systeminformation");
var promises_shared_1 = require("../shared/promises.shared");
var md5 = require("md5");
var platform_service_1 = require("../services/platform.service");
var logger = log4js.getLogger('Task');
var icModule = require('intermediate-component-sa');
var ReportImplementation = (function () {
    function ReportImplementation() {
    }
    ReportImplementation.prototype.GetSystemInformation = function (guid) {
        var systemInformationData = {
            installationId: guid,
            macAddress: '',
            publicIp: ''
        };
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[ReportImplementation::GetSystemInformation] guid:', guid);
            var sys = sy.networkInterfaces();
            return sys;
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_NETWORK_INTERFACE_INFORMATION';
            if (result && result.length) {
                logger.info('[ReportImplementation::GetSystemInformation] guid:', guid, 'getting result[0].mac:', result[0].mac);
                var myNetworkInterface = null;
                myNetworkInterface = result.filter(function (r) { return r.operstate === 'up'; })[0];
                systemInformationData.macAddress = myNetworkInterface.mac;
                systemInformationData.publicIp = myNetworkInterface.ip4;
            }
            logger.info('[ReportImplementation::GetSystemInformation] guid:', guid, 'getting systemInformationData:', systemInformationData);
            return Promise.resolve(systemInformationData);
        }).catch(function (err) {
            logger.error('[ReportImplementation::GetSystemInformation] guid:', guid, 'getting error:', err);
            return Promise.resolve(false);
        });
    };
    ReportImplementation.prototype.GenerateMD5Content = function (data) {
        var result = '';
        if (data) {
            result += data.installationId;
            result += data.macAddress;
            logger.info('[ReportImplementation::GenerateMD5Content] getting result:', result);
            result = md5(result);
        }
        return result;
    };
    ReportImplementation.prototype.GetReportProceduraParameters = function (procedure) {
        var _this = this;
        var systemInformationData = null;
        var installationId;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[ReportImplementation::GetReportProceduraParameters] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            var body = { procedure: procedure };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_REPORT_GET_PARAMETERS', body);
        }).then(function (result) {
            logger.info('[ReportImplementation::GetReportProceduraParameters] Getting result:', result.length, 'from external server');
            if (result.status) {
                return Promise.resolve(result.data);
            }
            else {
                return Promise.resolve(null);
            }
        }).catch(function (err) {
            logger.error('[ReportImplementation::GetReportProceduraParameters] Getting error:', err);
            return Promise.resolve(null);
        });
    };
    ReportImplementation.prototype.ReportExecution = function (procedureParameters, procedure) {
        var _this = this;
        var systemInformationData = null;
        var installationId;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[ReportImplementation::GetReportProceduraParameters] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                procedureParameters.push({
                    column: "InstallationId",
                    value: installationId
                });
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            if (!installationId)
                throw 'MISSING_PARAMETER_INSTALLATION_ID';
            var body = { procedure: procedure,
                procedureParameters: procedureParameters };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_REPORT_EXECUTE', body);
        }).then(function (result) {
            logger.info('[ReportImplementation::GetReportProceduraParameters] Getting result:', result.length, 'from external server');
            if (result.status) {
                return Promise.resolve(result.data);
            }
            else {
                return Promise.resolve(null);
            }
        }).catch(function (err) {
            logger.error('[ReportImplementation::GetReportProceduraParameters] Getting error:', err);
            return Promise.resolve(null);
        });
    };
    return ReportImplementation;
}());
exports.ReportImplementation = ReportImplementation;
