"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rp = require("request-promise-native");
var log4js = require("log4js");
var fs = require("fs");
var mssql_service_1 = require("../db/mssql.service");
var config_1 = require("../config/config");
var redis_service_1 = require("../db/redis.service");
var file_manager_1 = require("../shared/file-manager");
var moment = require("moment");
var promises_shared_1 = require("../shared/promises.shared");
var billing_implementation_1 = require("../implementation/billing.implementation");
var logger = log4js.getLogger('Task');
var TranscriptionsImplementation = (function () {
    function TranscriptionsImplementation() {
    }
    TranscriptionsImplementation.prototype.Execute = function (executionCode, rateRecursive, availableBatch, cancelledBatch) {
        var _this = this;
        var executeIndex = 1;
        var ratio = config_1.RATIO_TRANSCRIPTION;
        var alowedRatio = 0;
        var notAlowedRatio = 0;
        var executeResult = true;
        return promises_shared_1.SleepPromise(config_1.SLEEP_TRANSCRIPTION)
            .then(function (result) {
            return redis_service_1.Redis.LRange('Transcriptions', 0, ratio - 1);
        })
            .then(function (result) {
            var promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            result.map(function (r) {
                var arrQueue = r.split(':');
                if (availableBatch.find(function (b) { return b.VCC === arrQueue[0] && b.BatchTypeId === arrQueue[1]; })) {
                    alowedRatio++;
                }
                else {
                    notAlowedRatio++;
                }
            });
            while (ratio >= executeIndex) {
                promiseArray.push(_this.TranscribeAudioExecute(executionCode, availableBatch, cancelledBatch));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > notAlowedRatio) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('Transcriptions', 0, ratio - 1);
        }).then(function (result) {
            if (result && result.length && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1, availableBatch, cancelledBatch);
            return executeResult;
        }).then(function (result) {
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[TranscriptionsImplementation::Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    TranscriptionsImplementation.prototype.TranscribeAudioExecute = function (executionCode, availableBatch, cancelledBatch) {
        var _this = this;
        var storage = '';
        var currentQueueId = '';
        var processInteraction;
        var vcc = '';
        var processId = '';
        var interactionType = '';
        var interactionId = '';
        var interactionSection = 0;
        var interactionPart = 0;
        var batchTypeName = '';
        var batchTypeId = '';
        var currentBlockPromise;
        return redis_service_1.Redis.RemoveQueueL('Transcriptions')
            .then(function (result) {
            currentBlockPromise = 'BLOCK_01';
            currentQueueId = result;
            var arrQueue = currentQueueId.split(':');
            return mssql_service_1.MSSql.GetEvaluationPlanByProcessId(arrQueue[0], arrQueue[1]);
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_02';
            var arrQueue = currentQueueId.split(':');
            if (result && result.id) {
                batchTypeName = 'EVALUATION_PLAN';
                batchTypeId = result.id;
            }
            else {
                batchTypeName = 'TRANSCRIPTION';
                batchTypeId = arrQueue[1];
            }
            var promiseArray = [];
            if (availableBatch.find(function (b) { return b.VCC === arrQueue[0] && b.BatchTypeId === batchTypeId; })) {
                promiseArray.push(Promise.resolve('ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL('TranscriptionsExecute', currentQueueId));
                promiseArray.push(Promise.resolve(currentQueueId));
            }
            else {
                promiseArray.push(Promise.resolve('NOT_ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL('Transcriptions', currentQueueId));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchStatus(arrQueue[0], arrQueue[1], 'TRANSCRIPTION', 'PAUSED'));
            }
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_03';
            if (result && result[0] === 'NOT_ALLOWED')
                throw 'SERVER_ERROR_NOT_ALLOWED_QUEUE';
            var queueId = result[2];
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[TranscriptionsImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                var item = queueId.split(':');
                if (item && item.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    interactionId = item[3];
                    interactionSection = Number(item[4]);
                    interactionPart = Number(item[5]);
                    storage = vcc + ':' + processId;
                    logger.info('[TranscriptionsImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
                }
                else {
                    logger.error('[TranscriptionsImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return null;
        }).then(function (processInteractionResult) {
            currentBlockPromise = 'BLOCK_04';
            if (processInteractionResult) {
                if (cancelledBatch.find(function (b) { return b.VCC === vcc && b.BatchTypeId === batchTypeId; })) {
                    processInteractionResult.status = -3;
                    processInteractionResult.isCancel = true;
                    processInteractionResult.statusDescription = 'REMOVED_BY_CANCELLED';
                    processInteractionResult.errorMessage = 'REMOVED_BY_CANCELLED';
                    processInteractionResult.retry = config_1.RETRY_TRANSCRIPTION;
                }
                logger.info('[TranscriptionsImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1');
                processInteraction = processInteractionResult;
                return file_manager_1.CreateFolder(config_1.TRANSCRIPTION_FOLDER + '/' + processInteraction.vcc);
            }
            else {
                if (currentQueueId)
                    logger.warn('[TranscriptionsImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return null;
            }
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_05';
            if (processInteraction) {
                if (batchTypeName === 'TRANSCRIPTION')
                    batchTypeId = processInteraction.processId;
                if (processInteraction.statusDescription === 'pendingUpload') {
                    if (processInteraction.interactionType === 'CALL')
                        return _this.UploadAudio(processInteraction, currentQueueId, executionCode);
                    else {
                        processInteraction.status = 3;
                        processInteraction.statusDescription = 'pendingAnalysis';
                    }
                }
                else if (processInteraction.statusDescription === 'pendingTranscription') {
                    var validTime = moment().format('x');
                    validTime = validTime * 1;
                    var nextExecutionTime = processInteraction.uploadedDate;
                    nextExecutionTime = nextExecutionTime * 1 + config_1.NEXT_EXECUTION_TIME_TRANSCRIPTION;
                    if (validTime > nextExecutionTime) {
                        return _this.DownloadTranscription(processInteraction, currentQueueId, executionCode);
                    }
                    else {
                        logger.info('[TranscriptionsImplementation::AudioTransferExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_EXECUTION', nextExecutionTime, 'CURRENT_TIME', validTime);
                    }
                }
            }
            return null;
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_06';
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return null;
        })
            .then(function (result) {
            currentBlockPromise = 'BLOCK_07';
            var promiseArray = [];
            var billing = new billing_implementation_1.BillingImplementation();
            if (result
                && processInteraction
                && processInteraction.status === 3
                && (processInteraction.statusDescription === 'pendingAnalysis'
                    ||
                        processInteraction.statusDescription === 'pendingIdentifyAgent')) {
                logger.info('[TranscriptionsImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_QUEUE');
                if (processInteraction.interactionType === 'CALL') {
                    billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'ValidDurationTime', batchTypeName, batchTypeId);
                    billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'SpeechmaticsApi', batchTypeName, batchTypeId);
                }
                if (processInteraction.statusDescription === 'pendingIdentifyAgent') {
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('DiarizationAnalysis', currentQueueId));
                }
                else {
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('SemanticAnalysis', currentQueueId));
                }
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsExecute', currentQueueId));
            }
            else if (result && processInteraction && processInteraction.status === 3 && processInteraction.statusDescription === 'pendingTranscription' && processInteraction.retry < config_1.RETRY_TRANSCRIPTION) {
                promiseArray.push(redis_service_1.Redis.InsertQueue('Transcriptions', currentQueueId, config_1.RATE_RECURSIVE_TRANSCRIPTION * config_1.RATIO_TRANSCRIPTION * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsExecute', currentQueueId));
            }
            else if (result && processInteraction && processInteraction.status === -3 && processInteraction.statusDescription === 'REMOVED_BY_CANCELLED') {
                logger.error('[TranscriptionsImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_CANCELLED');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, batchTypeName, batchTypeId, 0, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsExecute', currentQueueId));
            }
            else if (processInteraction && processInteraction.status === -3 && processInteraction.errorMessage === 'UPLOAD_ERROR' && processInteraction.retry >= 10) {
                logger.error('[TranscriptionsImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'UPLOAD_ERROR');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, batchTypeName, batchTypeId, 0, 'UPLOAD_ERROR');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsExecute', currentQueueId));
                processInteraction.retry = config_1.RETRY_TRANSCRIPTION;
            }
            else if (processInteraction && processInteraction.retry >= config_1.RETRY_TRANSCRIPTION) {
                logger.error('[TranscriptionsImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, batchTypeName, batchTypeId, 0, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsExecute', currentQueueId));
                processInteraction.retry = config_1.RETRY_TRANSCRIPTION;
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[TranscriptionsImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                var interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, batchTypeName, batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsExecute', currentQueueId));
            }
            else {
                logger.error('[TranscriptionsImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_08';
            var promiseArray = [];
            if (processInteraction && processInteraction.retry >= config_1.RETRY_TRANSCRIPTION) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_09';
            return Promise.resolve(true);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    TranscriptionsImplementation.prototype.UploadAudio = function (interaction, currentQueueId, executionCode) {
        var _this = this;
        var fullPath = '';
        var dictionaryContent = [];
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return _this.GetDictionaryContent(interaction, currentQueueId, executionCode);
        })
            .then(function (result) {
            dictionaryContent = result;
            return new Promise(function (resolve, reject) {
                var baseFolder = interaction.environmentType === 'allegro' ? config_1.PATH_FOLDER_TEMP_AUDIO : config_1.PATH_FOLDER_PROCESS_AUDIO;
                fullPath = baseFolder;
                fullPath += '/';
                fullPath += interaction.vcc;
                fullPath += '/';
                fullPath += interaction.fileName;
                logger.info('[TranscriptionsImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'audio:', fullPath);
                fs.stat(fullPath, function (err, stat) {
                    resolve({ fullPath: fullPath, stat: stat });
                });
            });
        })
            .then(function (result) {
            if (result && result.stat && result.stat.size > 0) {
                return new Promise(function (resolve, reject) {
                    var stream = fs.createReadStream(result.fullPath);
                    stream.on('readable', function () {
                        if (stream !== null && stream !== undefined) {
                            return resolve(stream);
                        }
                        else {
                            logger.error('[TranscriptionsImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NOT_GET_FILE_STREAM');
                            return resolve(null);
                        }
                    });
                });
            }
            logger.error('[TranscriptionsImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NOT_GET_FILE_STAT fullPath:', fullPath);
            return null;
        }).then(function (stream) {
            if (stream) {
                logger.debug('[TranscriptionsImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Process language:', interaction.language);
                var url = '';
                url += 'http://';
                url += config_1.SPEECHMATICS_IP;
                url += ':';
                url += config_1.SPEECHMATICS_PORT;
                url += '/v1/user/1/jobs/';
                var postOptions = {
                    uri: url,
                    method: 'POST',
                    headers: {
                        'Content-Type': 'multipart/form-data',
                        'Connection': 'keep-alive'
                    },
                    formData: {
                        'data_file': stream,
                        'config': JSON.stringify({
                            'type': 'transcription',
                            'transcription_config': {
                                'diarization': 'channel',
                                'channel_diarization_labels': ['Agent', 'Caller'],
                                'language': interaction.language,
                                'additional_vocab': dictionaryContent
                            }
                        })
                    }
                };
                logger.debug('[TranscriptionsImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Request to speechmatics:', JSON.stringify(postOptions));
                return rp(postOptions);
            }
            return null;
        }).then(function (result) {
            var jobStatus = null;
            if (result) {
                try {
                    jobStatus = JSON.parse(result);
                }
                catch (err) {
                    jobStatus = null;
                }
            }
            return jobStatus;
        }).then(function (jobStatus) {
            var jobId = jobStatus && jobStatus.id ? jobStatus.id : null;
            if (jobId) {
                logger.info('[TranscriptionsImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'UPLOADED');
                interaction['status'] = 3;
                interaction['statusDescription'] = 'pendingTranscription';
                interaction['retry'] = 0;
                interaction['jobId'] = jobId;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorMessage'] = null;
            }
            else {
                logger.error('[TranscriptionsImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'JOBID_IS_NOT_FOUND');
                interaction['status'] = -3;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorMessage'] = 'JOBID_IS_NOT_FOUND';
            }
            return Promise.resolve(interaction);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            interaction['status'] = -3;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorMessage'] = 'UPLOAD_ERROR';
            return Promise.resolve(interaction);
        });
    };
    TranscriptionsImplementation.prototype.DownloadTranscription = function (interaction, currentQueueId, executionCode) {
        var _this = this;
        var retry = false;
        return this.DownloadSpeechMatics(interaction, currentQueueId, executionCode)
            .then(function (result) {
            if (result && result.statusDescription === 'jobInProgress') {
                retry = true;
                return null;
            }
            var regex = /\.(mp3|wav)$/;
            var jsonFileName = interaction.fileName.replace(regex, '.json');
            var jsonContent = JSON.stringify(result);
            var targetLocation = config_1.TRANSCRIPTION_FOLDER;
            targetLocation += '/';
            targetLocation += interaction.vcc;
            targetLocation += '/';
            targetLocation += jsonFileName;
            return _this.SaveTranscriptionJSONFile(targetLocation, jsonContent, currentQueueId, executionCode);
        }).then(function (result) {
            var jobStatus = null;
            if (result) {
                try {
                    jobStatus = JSON.parse(result);
                    var validChar_1 = 0;
                    if (jobStatus.results && jobStatus.results.length > 0) {
                        jobStatus.results.map(function (r) {
                            if (r.alternatives && r.alternatives.length > 0) {
                                validChar_1 += r.alternatives[0].content ? r.alternatives[0].content.length + 1 : 0;
                            }
                        });
                    }
                    interaction['durationLinguisticTime'] = jobStatus && jobStatus.job && jobStatus.job.duration ? jobStatus.job.duration : null;
                    interaction['validChar'] = validChar_1;
                    interaction['validWords'] = jobStatus && jobStatus.results ? jobStatus.results.length : 0;
                    interaction['speechmaticsApi'] = config_1.SPEECHMATICS_IP + ':' + config_1.SPEECHMATICS_PORT;
                }
                catch (err) {
                    jobStatus = null;
                }
            }
            return jobStatus;
        }).then(function (result) {
            if (result) {
                interaction['status'] = 3;
                interaction['statusDescription'] = interaction.isDiariced === true ? 'pendingIdentifyAgent' : 'pendingAnalysis';
                interaction['retry'] = 0;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorMessage'] = null;
            }
            else if (retry) {
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorMessage'] = 'JOB_IN_PROGRESS';
            }
            else {
                logger.error('[TranscriptionsImplementation::DownloadSpeechMatics] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'DOWNLOAD_ERROR');
                interaction['status'] = -3;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorMessage'] = 'DOWNLOAD_ERROR';
                interaction['uploadedDate'] = moment().format('x');
            }
            return Promise.resolve(interaction);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsImplementation::DownloadTranscription] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            interaction['status'] = -3;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorMessage'] = errorMessage;
            interaction['uploadedDate'] = moment().format('x');
            return Promise.resolve(interaction);
        });
    };
    TranscriptionsImplementation.prototype.SaveTranscriptionJSONFile = function (fileName, data, currentQueueId, executionCode) {
        return file_manager_1.WriteFileContent(fileName, data, 'utf-8', true)
            .then(function (result) {
            return Promise.resolve(data);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[TranscriptionsImplementation::SaveTranscriptionJSONFile] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(null);
        });
    };
    TranscriptionsImplementation.prototype.DownloadSpeechMatics = function (interaction, currentQueueId, executionCode) {
        var url = '';
        url += 'http://';
        url += config_1.SPEECHMATICS_IP;
        url += ':';
        url += config_1.SPEECHMATICS_PORT;
        url += '/v1/user/1/jobs/';
        url += interaction.jobId;
        url += '/transcript?format=json-v2';
        var options = {
            method: 'GET',
            uri: url,
            json: true,
            resolveWithFullResponse: false
        };
        var data;
        return rp(options)
            .then(function (result) {
            data = result;
            return Promise.resolve(result);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            var pending = {};
            if (typeof errorMessage === 'string' && errorMessage.indexOf('Job In Progress') > -1) {
                logger.warn('[TranscriptionsImplementation::DownloadSpeechMatics] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                pending['statusDescription'] = 'jobInProgress';
            }
            else {
                logger.error('[TranscriptionsImplementation::DownloadSpeechMatics] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage, 'url:', url);
            }
            data = pending && pending.statusDescription ? pending : null;
            return Promise.resolve(data);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsImplementation::DownloadSpeechMatics] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage, 'url:', url);
            return Promise.resolve(null);
        });
    };
    TranscriptionsImplementation.prototype.GetDictionaryContent = function (interaction, currentQueueId, executionCode) {
        var dictionaryWords = [];
        var dictionaryContent = [];
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetDictionaryWords(interaction.vcc, null, null, interaction.dictionaryId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_GET_DICTIONARY_WORDS';
            dictionaryWords = result;
            var dictionaryWordIds = [];
            dictionaryWords.map(function (dw) { return dictionaryWordIds.push(dw.id); });
            return mssql_service_1.MSSql.GetDictionaryWordSoundLikes(interaction.vcc, dictionaryWordIds.join('\',\''));
        })
            .then(function (result) {
            logger.error('[TranscriptionsImplementation::GetDictionaryContent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'found words:', dictionaryWords.length);
            dictionaryWords.map(function (dw) {
                dw.soundLikes = [];
                dw.soundLikes = result.filter(function (r) { return r.dictionaryWordId === dw.id; });
            });
            dictionaryWords.map(function (dw) {
                if (dw.soundLikes && dw.soundLikes.length) {
                    var soundLikeWords_1 = [];
                    dw.soundLikes.map(function (sl) { return soundLikeWords_1.push(sl.soundLike); });
                    dictionaryContent.push({
                        content: dw.word,
                        sounds_like: soundLikeWords_1
                    });
                }
                else {
                    dictionaryContent.push(dw.word);
                }
            });
            return Promise.resolve(dictionaryContent);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsImplementation::GetDictionaryContent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve([]);
        });
    };
    return TranscriptionsImplementation;
}());
exports.TranscriptionsImplementation = TranscriptionsImplementation;
