"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var promises_shared_1 = require("../shared/promises.shared");
var billing_implementation_1 = require("../implementation/billing.implementation");
var billing_account_implementation_1 = require("../implementation/billing-account.implementation");
var logger = log4js.getLogger('ServerScripts');
function AccountService(app) {
    app.post('/api/account/get_account_status', function (req, res) {
        var vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var billingAccount = new billing_account_implementation_1.BillingAccountImplementation();
            return billingAccount.GetAccountStatusByVCC(vcc);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            if (result) {
                var accountStatus = {};
                if (result.status && result.status === true) {
                    accountStatus.enabled = result.status;
                    accountStatus.description = result.description;
                }
                else {
                    accountStatus.enabled = result.status;
                    accountStatus.description = result.error;
                }
                res.json({ status: true, description: 'OK', data: accountStatus });
            }
            else
                throw 'CANNOT_PROCESS_REQUEST';
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[AccountService::get_account_status] Getting error:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_accounts_list', function (req, res) {
        var vcc = res.locals.vcc;
        var updated = req.body.updated || false;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var billing = new billing_implementation_1.BillingImplementation();
            return billing.GetAccountsSubAccounts(vcc, updated);
        }).then(function (result) {
            return Promise.resolve(result);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[AccountService::/api/account/get_accounts_list] Getting error:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_vccs_by_account', function (req, res, next) {
        var vcc = res.locals.vcc;
        var accountId = req.body.accountId;
        var type = req.body.type;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var billingAccount = new billing_account_implementation_1.BillingAccountImplementation();
            return billingAccount.GetVCCsByAccount(accountId, type);
        }).then(function (result) {
            return Promise.resolve(result);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[AccountService::/api/account/get_vccs_by_account] Getting error:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_movements', function (req, res, next) {
        var vcc = res.locals.vcc;
        var billingMovement = (req.body || {});
        if (res.locals.vcc === 'system'
            && res.locals.userId === 'admin') {
        }
        else
            billingMovement.vcc = vcc;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var billingAccount = new billing_account_implementation_1.BillingAccountImplementation();
            return billingAccount.GetMovements(billingMovement);
        }).then(function (result) {
            return Promise.resolve(result);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            logger.error('[AccountService::/api/account/get_movements] Getting error:', errorMsg, 'params:', billingMovement);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_movement_currencies', function (req, res, next) {
        var vcc = res.locals.vcc;
        var billingMovement = (req.body || {});
        if (res.locals.vcc === 'system'
            && res.locals.userId === 'admin') {
        }
        else
            billingMovement.vcc = vcc;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var billingAccount = new billing_account_implementation_1.BillingAccountImplementation();
            return billingAccount.GetMovementCurrencies(billingMovement);
        })
            .then(function (result) {
            return Promise.resolve(result);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            logger.error('[AccountService::/api/account/get_movements] Getting error:', errorMsg, 'params:', billingMovement);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/account/get_costs', function (req, res, next) {
        var vcc = res.locals.vcc;
        var billingMovement = (req.body || {});
        if (res.locals.vcc === 'system'
            && res.locals.userId === 'admin') {
        }
        else
            billingMovement.vcc = vcc;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var billingAccount = new billing_account_implementation_1.BillingAccountImplementation();
            return billingAccount.GetCosts(billingMovement);
        }).then(function (result) {
            return Promise.resolve(result);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_REQUESTING_ERROR');
            logger.error('[AccountService::/api/account/get_cost] Getting error:', errorMsg, 'params:', billingMovement);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AccountService = AccountService;
