"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var api_user_implementation_1 = require("../implementation/api-user.implementation");
var logger = log4js.getLogger('ServerScripts');
function ApiUserService(app) {
    app.get('/api/api_user/get_api_users/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetApiUsers(vcc, search_criteria, search_expression);
        }).then(function (users) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: users });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/api_user/get_api_users_external', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var apiUser = new api_user_implementation_1.ApiUserImplementation();
            return apiUser.GetApiUsers(vcc);
        }).then(function (users) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: users });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/api_user/create_api_user', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var apiUserI = new api_user_implementation_1.ApiUserImplementation();
        var apiUser = (req.body || {});
        apiUser.isLocked = false;
        apiUser.createdDate = nowUtc;
        apiUser.createdByUserId = userId;
        apiUser.vcc = vcc;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (vcc && apiUser.user && apiUser.pin) {
                return apiUserI.CreateApiUser(vcc, apiUser.user, apiUser.pin.toString(), userId);
            }
            else {
                throw 'MISSING_PARAMETERS';
            }
        })
            .then(function (result) {
            if (!result)
                throw 'ERROR_SAVE_API_USER_SERVER_EXTERNAL';
            return mssql_service_1.MSSql.SaveApiUser(apiUser);
        })
            .then(function (resultSaveUser) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveUser, description: (resultSaveUser ? 'OK' : resultSaveUser), data: resultSaveUser });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/api_user/get_api_user/:user_id', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = req.params.user_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!userId)
                throw 'Missing user input parameter';
            return mssql_service_1.MSSql.GetApiUser(vcc, userId);
        }).then(function (resultUser) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: resultUser });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/api_user/get_current_user', function (req, res, next) {
        logger.info('[ApiUserService::get_current_user] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var genericResult = {};
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!userId)
                throw 'SERVER_ERROR_MISSING_PARAMETER';
            return mssql_service_1.MSSql.GetApiUser(vcc, userId);
        })
            .then(function (resultUser) {
            if (!resultUser)
                throw 'SERVER_ERROR_API_USER_NOT_FOUND';
            if (res.headersSent)
                return;
            genericResult.status = true;
            genericResult.description = 'OK';
            res.json(genericResult);
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[ApiUserService::get_current_user] Error al procesar \'' + req.path + '\': ' + errorMsg);
            genericResult.status = false;
            genericResult.description = clientMsg;
            genericResult.error = errorMsg;
            if (res.headersSent)
                return;
            res.json(genericResult);
        });
    });
    app.post('/api/api_user/save_api_user', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var apiUserI = new api_user_implementation_1.ApiUserImplementation();
        var apiUser = (req.body || {});
        apiUser.createdDate = new Date(apiUser.createdDate);
        apiUser.lastModifiedDate = nowUtc;
        apiUser.lastModifiedByUserId = userId;
        apiUser.vcc = vcc;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (vcc && apiUser.user && apiUser.pin) {
                return apiUserI.SaveApiUser(vcc, apiUser.user, apiUser.pin.toString(), userId);
            }
            else {
                throw 'MISSING_PARAMETERS';
            }
        })
            .then(function (result) {
            if (!result)
                throw 'ERROR_SAVE_API_USER_SERVER_EXTERNAL';
            return mssql_service_1.MSSql.SaveApiUser(apiUser);
        })
            .then(function (resultSaveUser) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveUser, description: (resultSaveUser ? 'OK' : resultSaveUser), data: resultSaveUser });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/api_user/delete_api_user', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var user = req.body.user;
        var userId = user.user;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!userId)
                throw 'Missing user input parameter';
            var apiUser = new api_user_implementation_1.ApiUserImplementation();
            return apiUser.DeleteApiUser(vcc, userId);
        })
            .then(function (resultDeleteUser) {
            if (!resultDeleteUser)
                throw 'FAILED_DELETE_API_USER_EXTERNAL_SERVER';
            return DeleteApiUser(vcc, userId);
        })
            .then(function (resultDeleteUser) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: '', data: resultDeleteUser });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/api_user/delete_api_users', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var users = req.body.users || [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var usersApi = [];
            users.forEach(function (us) {
                usersApi.push(us.user);
            });
            var apiUser = new api_user_implementation_1.ApiUserImplementation();
            return apiUser.DeleteApiUsers(vcc, usersApi);
        })
            .then(function (result) {
            if (!result)
                throw 'FAILED_DELETE_API_USERS';
            var promiseArray = [];
            users.forEach(function (user) {
                promiseArray.push(DeleteApiUser(vcc, user.user));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteUsers) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: '', data: resultDeleteUsers });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.ApiUserService = ApiUserService;
function DeleteApiUser(vcc, userId) {
    var user, reason = '';
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        if (!userId) {
            reason = 'MISSING_USER_ID_PARAMETER';
            throw reason;
        }
        return mssql_service_1.MSSql.GetApiUser(vcc, userId);
    }).then(function (result) {
        user = result;
        if (!result) {
            reason = 'ERROR_USER_NOT_FOUND';
            throw reason;
        }
        return mssql_service_1.MSSql.DeleteApiUser(vcc, userId);
    }).then(function (result) {
        if (!result) {
            reason = 'ERROR_DELETING_API_USER';
            throw reason;
        }
        return Promise.resolve(result);
    }).catch(function (error) {
        return SetError(userId, error);
    });
}
function SetError(userId, reason) {
    var details = null;
    if ((userId && userId.length) && (reason && reason.length)) {
        details = {};
        details.error = {
            userId: userId,
            reason: reason
        };
    }
    return Promise.resolve(details);
}
