"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function AuditorService(app) {
    var _this = this;
    app.get('/api/auditor/get_auditors/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[AuditorService::get_auditors] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        logger.info('[AuditorService::get_auditors] Received search_criteria: ' + _this.search_criteria);
        logger.info('[AuditorService::get_auditors] Received search_expression: ' + _this.search_expression);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[AuditorService::get_auditors] Calling GetAuditors SP');
            return mssql_service_1.MSSql.GetAuditors(instance, search_criteria, search_expression);
        }).then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[AuditorService::get_auditors] Received results from GetAuditors SP: ' + result.length);
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var groupMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[AuditorService::get_auditors] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: groupMsg, error: errorMsg });
        });
    });
    app.get('/api/auditor/get_auditors_interactions/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibió p' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('Instance ' + instance);
            logger.info('sear. criterial  ' + search_criteria);
            logger.info('sear. expression  ' + search_expression);
            return mssql_service_1.MSSql.GetAuditorsInteractions(instance, search_criteria, search_expression);
        }).then(function (result) {
            logger.info('Datos prueba ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var groupMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: groupMsg, error: errorMsg });
        });
    });
    app.get('/api/auditor/get_auditor/:auditor_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var auditorId = req.params.auditor_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!auditorId)
                throw 'Missing group input parameter';
            return mssql_service_1.MSSql.GetAuditor(instance, auditorId);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/auditor/get_auditor_evaluationplan_evaluation', function (req, res, next) {
        logger.info('[AuditorService::get_auditor_evaluationplan_evaluation] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var userRole = res.locals.user.role;
        var auditorId = userRole && userRole === ('administrator' || 'quality_admin') ? req.body.auditorId : res.locals.userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!auditorId)
                throw 'Missing auditor ID parameter';
            return mssql_service_1.MSSql.GetAuditorEvaluationPlanEvaluation(vcc, auditorId, req.body.evaluationPlanId, req.body.interactionId, req.body.interactionType);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AuditorService = AuditorService;
