"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function CustomTemplateService(app) {
    app.get('/api/custom_template/get_custom_templates/:type/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var type = req.params.type || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetCustomTemplates(vcc, type, search_criteria, search_expression);
        }).then(function (customTemplates) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: customTemplates });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/custom_template/get_custom_template/:custom_template_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var customTemplateId = req.params.custom_template_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!customTemplateId)
                throw 'Missing custom template input parameter';
            return mssql_service_1.MSSql.GetCustomTemplate(vcc, customTemplateId);
        }).then(function (resultCustomTemplate) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: resultCustomTemplate });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/custom_template/get_custom_template_theme_items/:custom_template_id?', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var customTemplateId = req.params.custom_template_id || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetCustomTemplateThemeItems(vcc, customTemplateId);
        }).then(function (resultCustomTemplateThemeItems) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: resultCustomTemplateThemeItems });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/custom_template/create_custom_template', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newCustomTemplate = (req.body || {});
        var customTemplateId;
        var SaveCustomTemplateValue;
        var templateThemes;
        newCustomTemplate.createdDate = nowUtc;
        newCustomTemplate.createdByUserId = userId;
        newCustomTemplate.lastModifiedDate = nowUtc;
        newCustomTemplate.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetCustomTemplate(vcc, newCustomTemplate.name);
        }).then(function (resultCustomTemplate) {
            if (resultCustomTemplate)
                throw 'Ya existe una plantilla personalizada con el identificador \'' + newCustomTemplate.name + '\'';
            return SaveCustomTemplate(vcc, newCustomTemplate);
        }).then(function (resultSaveCustomTemplate) {
            customTemplateId = newCustomTemplate.name;
            SaveCustomTemplateValue = resultSaveCustomTemplate;
            if (newCustomTemplate.template !== null) {
                return mssql_service_1.MSSql.GetCustomTemplateThemes(vcc, newCustomTemplate.template);
            }
            else {
                return Promise.resolve(false);
            }
        }).then(function (resultTemplateThemes) {
            if (resultTemplateThemes && resultTemplateThemes.length > 0) {
                return mssql_service_1.MSSql.BulkTemplateThemeTemp(vcc, newCustomTemplate, customTemplateId, resultTemplateThemes);
            }
            else {
                return Promise.resolve(false);
            }
        }).then(function (resultBulkTemplateThemes) {
            if (resultBulkTemplateThemes) {
                return mssql_service_1.MSSql.GetTemplateThemeItems(vcc, newCustomTemplate.template, null);
            }
            else {
                return Promise.resolve(false);
            }
        }).then(function (resultTemplateThemeItems) {
            if (resultTemplateThemeItems && resultTemplateThemeItems.length > 0) {
                return mssql_service_1.MSSql.BulkTemplateThemeItemTemp(vcc, newCustomTemplate, customTemplateId, resultTemplateThemeItems);
            }
            else {
                return Promise.resolve(true);
            }
        }).then(function (resultBulkTemplateThemeItems) {
            if (!resultBulkTemplateThemeItems)
                throw 'Se produjo un error al guardar la plantilla personalizada';
            return SaveCustomTemplateValue;
        }).then(function (resultSaveCustomTemplate) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSaveCustomTemplate ? 'OK' : 'Template save failed'), data: resultSaveCustomTemplate });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/custom_template/save_custom_template', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newCustomTemplate = (req.body || {});
        newCustomTemplate.createdDate = null;
        newCustomTemplate.createdByUserId = null;
        newCustomTemplate.lastModifiedDate = nowUtc;
        newCustomTemplate.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.DeleteCustomTemplateTheme(vcc, newCustomTemplate.name);
        }).then(function (resultDeleteCustomTemplate) {
            if (!resultDeleteCustomTemplate)
                throw 'Se produjo un error al guardar la plantilla personalizada';
            return SaveCustomTemplate(vcc, newCustomTemplate);
        }).then(function (resultSaveCustomemplate) {
            if (!resultSaveCustomemplate)
                throw 'Se produjo un error al guardar la plantilla personalizada';
            return Promise.resolve(true);
        }).then(function (resultSaveCustomemplate) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveCustomemplate, description: (resultSaveCustomemplate ? 'OK' : 'template save failed'), data: resultSaveCustomemplate });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/custom_template/delete_custom_template', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var customTemplate = req.body.custom_template;
        var customTemplateId = customTemplate.name;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!customTemplate)
                throw 'Missing template input parameter';
            return mssql_service_1.MSSql.DeleteCustomTemplate(vcc, customTemplateId);
        }).then(function (resultDeleteCustomTemplate) {
            if (!resultDeleteCustomTemplate)
                throw 'Se produjo un error al eliminar la plantilla personalizada';
            return Promise.resolve(true);
        }).then(function (resultDeleteCustomTemplate) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteCustomTemplate, description: (resultDeleteCustomTemplate ? 'OK' : 'Template delete failed'), data: resultDeleteCustomTemplate });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/custom_template/delete_custom_templates', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var customTemplates = req.body.custom_templates || [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!customTemplates)
                throw 'Missing templates input parameter';
            var promiseArray = [];
            customTemplates.forEach(function (customTemplate) {
                promiseArray.push(mssql_service_1.MSSql.DeleteCustomTemplate(vcc, customTemplate.name));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteCustomTemplates) {
            var result;
            if (resultDeleteCustomTemplates.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Plantillas eliminadas correctamente');
            }
            return result;
        }).then(function (resultDeleteCustomTemplates) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteCustomTemplates, description: (resultDeleteCustomTemplates ? 'OK' : 'Custom templates delete failed'), data: resultDeleteCustomTemplates });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.CustomTemplateService = CustomTemplateService;
function GetCustomTemplate(vcc, name) {
    var cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(function () {
        return mssql_service_1.MSSql.GetCustomTemplate(vcc, name);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('Error al ejecutar GetCustomTemplate: ' + errMsg);
    });
}
exports.GetCustomTemplate = GetCustomTemplate;
function SaveCustomTemplate(vcc, customTemplate) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveCustomTemplate(vcc, customTemplate.name, customTemplate.type, customTemplate.description, customTemplate.template, customTemplate.category, customTemplate.createdDate, customTemplate.createdByUserId, customTemplate.lastModifiedDate, customTemplate.lastModifiedByUserId));
    if (customTemplate.templateThemes) {
        customTemplate.templateThemes.forEach(function (customTemplateTheme) {
            promiseArray.push(mssql_service_1.MSSql.SaveCustomTemplateTheme(vcc, customTemplateTheme.templateId, customTemplateTheme.id, customTemplateTheme.order, customTemplateTheme.description, customTemplateTheme.category1, customTemplateTheme.category2, moment().utc().toDate(), customTemplateTheme.createdByUserId, moment().utc().toDate(), customTemplateTheme.lastModifiedByUserId));
        });
    }
    if (customTemplate.templateThemeItems) {
        customTemplate.templateThemeItems.forEach(function (customTemplateThemeItem) {
            promiseArray.push(mssql_service_1.MSSql.SaveCustomTemplateThemeItem(vcc, customTemplateThemeItem.templateId, customTemplateThemeItem.templateThemeId, customTemplateThemeItem.id, moment().utc().toDate(), customTemplateThemeItem.createdByUserId, moment().utc().toDate(), customTemplateThemeItem.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
