"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var integration_config_1 = require("../config/integration.config");
var mssql_config_1 = require("../config/mssql.config");
var http_request_occ_1 = require("../shared/http-request-occ");
var logger = log4js.getLogger('ServerScripts');
function DispositionCodeService(app) {
    app.post('/api/dispositioncode/get', function (req, res, next) {
        logger.info('[DispositionCodeService::get] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var type = res.locals.type;
        var campaigns = req.body.campaigns ? req.body.campaigns : [];
        var allegroManagementResult = [];
        logger.info('[DispositionCodeService::get] Received vcc::campaigns => ' + vcc + '::' + campaigns);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var promiseArray = [];
            switch (type) {
                case 'ALLEGRO':
                    var flagAllegro = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroManagementResults(integration_config_1.IP_BD, vcc, flagAllegro));
                    break;
                case 'I6':
                    promiseArray.push(http_request_occ_1.ApiI6SearchCampaigns(vcc));
                    break;
                case 'OCC':
                    var flagOCC = integration_config_1.IP_BD === mssql_config_1.MSSQL_CONFIG.server ? 1 : 0;
                    promiseArray.push(mssql_service_1.MSSql.GetAllegroManagementResults(integration_config_1.IP_BD, vcc, flagOCC));
                    promiseArray.push(http_request_occ_1.ApiI6SearchCampaigns(vcc));
                    break;
                case 'OTHER':
                    break;
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            switch (type) {
                case 'ALLEGRO':
                    if (!result[0])
                        throw 'SERVER_ERROR_GET_SQL_DISPOSITION_CODES_FAILED';
                    allegroManagementResult = result[0];
                    break;
                case 'I6':
                    if ((!result[0]) || (result[0] && result[0].status === false))
                        throw 'SERVER_ERROR_GET_API_I6_DISPOSITION_CODES_FAILED';
                    var campaignsApiI6 = result[0].body;
                    campaignsApiI6.map(function (c) {
                        if (c.dispositionCodes) {
                            c.dispositionCodes.map(function (dc) {
                                var dispositionCode = {};
                                dispositionCode.campaignId = c.id;
                                dispositionCode.description = dc.parent ? dc.parent : null;
                                dispositionCode.description2 = dc.child ? dc.child : null;
                                dispositionCode.description3 = dc.child2 ? dc.child2 : null;
                                dispositionCode.description4 = dc.child3 ? dc.child3 : null;
                                allegroManagementResult.push(dispositionCode);
                            });
                        }
                    });
                    break;
                case 'OCC':
                    if (!result[0]
                        &&
                            ((!result[1]) || (result[1] && result[1].status === false)))
                        throw 'SERVER_ERROR_GET_DISPOSITION_CODES_FAILED';
                    allegroManagementResult = result[0] ? result[0] : [];
                    var campaignsApi = result[1] && result[1].status ? result[1].body : [];
                    campaignsApi.map(function (c) {
                        if (c.dispositionCodes) {
                            c.dispositionCodes.map(function (dc) {
                                var dispositionCode = {};
                                dispositionCode.vcc = vcc;
                                dispositionCode.code = null;
                                dispositionCode.campaignId = c.id;
                                dispositionCode.description = dc.parent ? dc.parent : null;
                                dispositionCode.description2 = dc.child ? dc.child : null;
                                dispositionCode.description3 = dc.child2 ? dc.child2 : null;
                                dispositionCode.description4 = dc.child3 ? dc.child3 : null;
                                allegroManagementResult.push(dispositionCode);
                            });
                        }
                    });
                    break;
                case 'OTHER':
                    break;
            }
            if (campaigns && campaigns.length > 0) {
                allegroManagementResult = allegroManagementResult.filter(function (amr) { return campaigns.includes(amr.campaignId); });
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: allegroManagementResult });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/dispositioncode/evaluation', function (req, res, next) {
        logger.info('[DispositionCodeService::evaluation] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var type = res.locals.type;
        var userId = res.locals.userId;
        var referrer = res.locals.pageReferrer.split('/').pop();
        var searchType = null;
        var campaigns = req.body.campaigns ? req.body.campaigns : [];
        switch (referrer) {
            case 'auditorinbox':
                searchType = 'PENDING';
                break;
            case 'evaluation':
                searchType = 'HISTORY';
                break;
            case 'evaluation-agent':
                searchType = 'HISTORY';
                break;
            case 'evaluation-admin':
                searchType = 'HISTORY';
                break;
        }
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetDispositionCodesFromEvaluation(vcc, userId, campaigns, searchType);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_DISPOSITION_CODES_SEARCH_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[DispositionCodeService::evaluation] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.DispositionCodeService = DispositionCodeService;
