"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function DomainService(app) {
    app.get('/api/domain/get_domains/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var user = res.locals.user;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetDomains(vcc, search_criteria, search_expression);
        }).then(function (resultDomains) {
            if (!resultDomains)
                throw 'No se pudo ejecutar consulta de dominios';
            if (user && user.limitClients) {
                var userClients_1 = (user.userClients || []).map(function (userClient) { return userClient.clientId; });
                resultDomains = resultDomains.filter(function (domain) {
                    return userClients_1.includes(domain.client);
                });
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: resultDomains });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/domain/get_domain/:domain', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var domain = req.params.domain;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!domain)
                throw 'Missing domain input parameter';
            return mssql_service_1.MSSql.GetDomain(vcc, domain);
        }).then(function (resultDomain) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: resultDomain });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/domain/create_domain', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newDomain = (req.body || {});
        newDomain.createdDate = nowUtc;
        newDomain.createdByUserId = userId;
        newDomain.lastModifiedDate = nowUtc;
        newDomain.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetDomains(vcc, 'domain', '');
        }).then(function (resultDomains) {
            if (resultDomains.find(function (domain) { return domain.domain === newDomain.domain; })) {
                throw 'Ya existe un dominio con el identificador \'' + newDomain.domain + '\'.';
            }
            if (newDomain.isMainDomain && resultDomains.find(function (domain) { return domain.isMainDomain && domain.domain !== newDomain.domain; })) {
                throw 'Ya existe un dominio definido como principal.';
            }
            return mssql_service_1.MSSql.SaveDomain(vcc, newDomain.domain, newDomain.description, newDomain.client, newDomain.hasHttps, newDomain.contentPort, newDomain.prefix, newDomain.isMainDomain, newDomain.isContentDomain, newDomain.contentDefaultRedirect, newDomain.isDistributedService, newDomain.associatedDistributedService, newDomain.customCode, newDomain.createdDate, newDomain.createdByUserId, newDomain.lastModifiedDate, newDomain.lastModifiedByUserId);
        }).then(function (resultSaveDomain) {
            if (!resultSaveDomain)
                throw 'Error al ejecutar solicitud';
            return redis_service_1.Redis.SaveDomain(vcc, newDomain);
        }).then(function (resultSaveDomain) {
            if (res.headersSent)
                return;
            logger.info('Dominio \'' + newDomain.domain + '\' creado correctamente');
            res.json({ status: resultSaveDomain, description: (resultSaveDomain ? 'OK' : 'Domain save failed'), data: resultSaveDomain });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/domain/save_domain', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newDomain = (req.body || {});
        newDomain.createdDate = null;
        newDomain.createdByUserId = null;
        newDomain.lastModifiedDate = nowUtc;
        newDomain.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetDomains(vcc, 'domain', '');
        }).then(function (resultDomains) {
            if (newDomain.isMainDomain && resultDomains.find(function (domain) { return domain.isMainDomain && domain.domain !== newDomain.domain; })) {
                throw 'Ya existe un dominio definido como principal.';
            }
            return mssql_service_1.MSSql.SaveDomain(vcc, newDomain.domain, newDomain.description, newDomain.client, newDomain.hasHttps, newDomain.contentPort, newDomain.prefix, newDomain.isMainDomain, newDomain.isContentDomain, newDomain.contentDefaultRedirect, newDomain.isDistributedService, newDomain.associatedDistributedService, newDomain.customCode, newDomain.createdDate, newDomain.createdByUserId, newDomain.lastModifiedDate, newDomain.lastModifiedByUserId);
        }).then(function (resultSaveDomain) {
            if (!resultSaveDomain)
                throw 'Error al ejecutar solicitud';
            return redis_service_1.Redis.SaveDomain(vcc, newDomain);
        }).then(function (resultSaveDomain) {
            if (res.headersSent)
                return;
            logger.info('Dominio \'' + newDomain.domain + '\' guardado correctamente');
            res.json({ status: resultSaveDomain, description: (resultSaveDomain ? 'OK' : 'Domain save failed'), data: resultSaveDomain });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/domain/delete_domain', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var domain = req.body.domain || {};
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!domain)
                throw 'Missing domain input parameter';
            return mssql_service_1.MSSql.DeleteDomain(vcc, domain.domain);
        }).then(function (resultDeleteDomain) {
            if (!resultDeleteDomain)
                throw 'Se produjo un error al eliminar el dominio';
            return redis_service_1.Redis.DeleteDomain(vcc, domain);
        }).then(function (resultDeleteDomain) {
            if (res.headersSent)
                return;
            logger.info('Dominio \'' + domain.domain + '\' eliminado correctamente');
            res.json({ status: resultDeleteDomain, description: (resultDeleteDomain ? 'OK' : 'Domain delete failed'), data: resultDeleteDomain });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/domain/delete_domains', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var domains = req.body.domains || [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!domains)
                throw 'Missing domains input parameter';
            var promiseArray = [];
            domains.forEach(function (domain) {
                promiseArray.push(mssql_service_1.MSSql.DeleteDomain(vcc, domain.domain));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteDomains) {
            var result;
            if (resultDeleteDomains.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Dominios eliminados correctamente');
                domains.forEach(function (domain) {
                    redis_service_1.Redis.DeleteDomain(vcc, domain);
                });
            }
            return result;
        }).then(function (resultDeleteDomains) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteDomains, description: (resultDeleteDomains ? 'OK' : 'Domains delete failed'), data: resultDeleteDomains });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.DomainService = DomainService;
