"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var template_service_1 = require("./template.service");
var promises_shared_1 = require("../shared/promises.shared");
var utilities_1 = require("../shared/utilities");
var template_model_1 = require("../models/template.model");
var custom_field_service_1 = require("./custom-field.service");
var ability_service_1 = require("./ability.service");
var logger = log4js.getLogger('ServerScripts');
function EvaluationFormatService(app) {
    app.get('/api/evaluationformat/get_evaluationformats/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[EvaluationFormatService::get_evaluationformats] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var disabled = req.query.disabled === 'true' || false;
        logger.info('[EvaluationFormatService::get_evaluationformats] Instance received: ' + instance);
        logger.info('[EvaluationFormatService::get_evaluationformats] Search_criteria received: ' + search_criteria);
        logger.info('[EvaluationFormatService::get_evaluationformats] Search_expression received: ' + search_expression);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (search_criteria === 'idGroup' || search_criteria === 'idSubGroup' || search_criteria === 'idAbility') {
                logger.info('[EvaluationFormatService::get_evaluationformats] Calling SP GetEvaluationFormatDescriptionsBySearchCriteria');
                return mssql_service_1.MSSql.GetEvaluationFormatDescriptionsBySearchCriteria(instance, search_criteria, search_expression);
            }
            else {
                logger.info('[EvaluationFormatService::get_evaluationformats] Calling SP GetEvaluationFormats');
                return mssql_service_1.MSSql.GetEvaluationFormats(instance, search_criteria, search_expression, disabled);
            }
        }).then(function (evaluationformats) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationFormatService::get_evaluationformats] Received restult from SP GetEvaluationFormats: ' + evaluationformats.length);
            res.json({ status: true, description: 'OK', data: evaluationformats });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::get_evaluationformats] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationformat/create_evaluationformat', function (req, res, next) {
        logger.info('[EvaluationFormatService::create_evaluationformat] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newEvaluationFormat = (req.body || {});
        newEvaluationFormat.createdDate = nowUtc;
        newEvaluationFormat.createdByUserId = userId;
        newEvaluationFormat.lastModifiedDate = nowUtc;
        newEvaluationFormat.lastModifiedByUserId = userId;
        logger.info('[EvaluationFormatService::create_evaluationformat] Received instance: ' + instance);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[EvaluationFormatService::create_evaluationformat] Calling GetEvaluationFormatDescription SP');
            return mssql_service_1.MSSql.GetEvaluationFormatDescription(instance, newEvaluationFormat.description);
        }).then(function (result) {
            logger.info('[EvaluationFormatService::create_evaluationformat] Received result from GetEvaluationFormatDescription SP: ' + JSON.stringify(result));
            if (result)
                throw 'EVALUATION_FORMAT_DESCRIPTION_EXISTS \'' + newEvaluationFormat.description + '\'';
            if (newEvaluationFormat.type === 'MANUAL_AUTOMATIC' || newEvaluationFormat.type === 'MIXED') {
                logger.info('[EvaluationFormatService::create_evaluationformat] Calling GetTemplate SP for MANUAL_AUTOMATIC type');
                return mssql_service_1.MSSql.GetTemplate(instance, newEvaluationFormat.id);
            }
            return false;
        }).then(function (result) {
            if (result)
                throw 'TEMPLATE_DESCRIPTION_EXISTS \'' + newEvaluationFormat.description + '\'';
            newEvaluationFormat.id = utilities_1.guid();
            newEvaluationFormat.templateId = (newEvaluationFormat.type === 'MANUAL_AUTOMATIC' || newEvaluationFormat.type === 'MIXED') ? ('qty-tmpl-' + utilities_1.guid()) : null;
            logger.info('[EvaluationFormatService::create_evaluationformat] Saving ev format ' + JSON.stringify(newEvaluationFormat));
            return mssql_service_1.MSSql.SaveEvaluationFormat(instance, newEvaluationFormat.id, newEvaluationFormat.description, newEvaluationFormat.type, newEvaluationFormat.environmentType, newEvaluationFormat.minQualificationAcceptable, newEvaluationFormat.minAutomaticQualification, newEvaluationFormat.managementFormURL, newEvaluationFormat.createdDate, newEvaluationFormat.createdByUserId, newEvaluationFormat.lastModifiedDate, newEvaluationFormat.lastModifiedByUserId, newEvaluationFormat.enabled, newEvaluationFormat.templateId);
        }).then(function (result) {
            if (!result)
                throw 'FORMAT_EVALUATION_SAVING_ERROR';
            logger.info('[EvaluationFormatService::create_evaluationformat] Saving SaveAbilityAndCAmpainEF for EV Format');
            return SaveAbilityAndCAmpainEF(instance, newEvaluationFormat);
        })
            .then(function (result) {
            if (newEvaluationFormat.type === 'MANUAL' || newEvaluationFormat.type === 'MIXED') {
                logger.info('[EvaluationFormatService::create_evaluationformat] Saving CustomFields for EV Format');
                return SaveEvaluationFormatCustomFields(instance, newEvaluationFormat, userId);
            }
            return Promise.resolve(true);
        })
            .then(function (result) {
            logger.info('[EvaluationFormatService::create_evaluationformat] Saving AttachedSignatures for EV Format');
            return SaveEvaluationFormatAttachedSignatures(instance, newEvaluationFormat, userId);
        })
            .then(function (result) {
            if (newEvaluationFormat.type === 'MANUAL_AUTOMATIC' || newEvaluationFormat.type === 'MIXED') {
                logger.info('[EvaluationFormatService::create_evaluationformat] Requesting AbilityQueries');
                return SetAbilityQueriesIntoAbilityEvaluationFormat(instance, newEvaluationFormat);
            }
            return newEvaluationFormat;
        })
            .then(function (result) {
            if (newEvaluationFormat.type === 'MANUAL_AUTOMATIC' || newEvaluationFormat.type === 'MIXED') {
                newEvaluationFormat = result;
                logger.info('[EvaluationFormatService::create_evaluationformat] Generating Template for EV Format');
                return GenerateTemplate(instance, newEvaluationFormat);
            }
            return true;
        })
            .then(function (resultSave) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationFormatService::create_evaluationformat] Sending result');
            res.json({ status: resultSave, description: (resultSave ? 'OK' : 'Formato de evaluación save failed'), data: resultSave });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::create_evaluationformat] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationformat/get_evaluationformat/:evaluationformat_id', function (req, res, next) {
        logger.info('[EvaluationFormatService::get_evaluationformat] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var evaluationFormat = {};
        var evaluationFormatAbilities = [];
        var evaluationFormatId = req.params.evaluationformat_id;
        logger.info('[EvaluationFormatService::get_evaluationformat] Looking for EV ' + evaluationFormatId);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!evaluationFormatId)
                throw 'ERROR_EVALUATION_FORMAT_ID_MISSING';
            logger.info('[EvaluationFormatService::get_evaluationformat] Calling GetEvaluationFormat SP');
            return mssql_service_1.MSSql.GetEvaluationFormat(instance, evaluationFormatId);
        }).then(function (result) {
            evaluationFormat = result || evaluationFormat;
            logger.info('[EvaluationFormatService::get_evaluationformat] Calling GetAbilityEvaluationFormats SP');
            return mssql_service_1.MSSql.GetAbilityEvaluationFormats(instance, evaluationFormatId);
        }).then(function (result) {
            evaluationFormatAbilities = result || evaluationFormatAbilities;
            evaluationFormatAbilities.forEach(function (element) {
                if (element.idSubGroup === null) {
                    element.idSubGroup = '';
                    element.descriptionSubGroup = '';
                }
            });
            var promiseArray = [];
            evaluationFormatAbilities.map(function (efa) {
                promiseArray.push(ability_service_1.GetAbilityQueries(instance, efa.idAbility, false));
            });
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            result.map(function (r, i) {
                evaluationFormatAbilities[i].queries = r;
            });
            evaluationFormat.abilityEvaluationFormats = evaluationFormatAbilities;
            logger.info('[EvaluationFormatService::get_evaluationformat] Calling GetEvaluationFormatCustomFields SP');
            return mssql_service_1.MSSql.GetEvaluationFormatCustomFields(instance, evaluationFormatId);
        })
            .then(function (result) {
            logger.info('[EvaluationFormatService::get_evaluationformat] Calling GetEvaluationFormatAttachedSignatures SP');
            evaluationFormat.evaluationFormatCustomFields = result;
            if (evaluationFormat.evaluationFormatCustomFields && evaluationFormat.evaluationFormatCustomFields.length) {
                var promiseArray_1 = [];
                evaluationFormat.evaluationFormatCustomFields.map(function (efcf) { return promiseArray_1.push(custom_field_service_1.GetCustomField(instance, efcf.customFieldId)); });
                return Promise.all(promiseArray_1);
            }
            else {
                return Promise.resolve([null]);
            }
        })
            .then(function (result) {
            if (evaluationFormat.evaluationFormatCustomFields && evaluationFormat.evaluationFormatCustomFields.length) {
                evaluationFormat.evaluationFormatCustomFields.map(function (efcf, index) { return efcf.customField = result[index]; });
            }
            return mssql_service_1.MSSql.GetEvaluationFormatAttachedSignatures(instance, evaluationFormatId);
        })
            .then(function (result) {
            logger.info('[EvaluationFormatService::get_evaluationformat] Calling GetCampainEvaluationFormats SP');
            evaluationFormat.evaluationFormatAttachedSignatures = result;
            return mssql_service_1.MSSql.GetCampainEvaluationFormats(instance, evaluationFormatId);
        }).then(function (result) {
            if (res.headersSent)
                return;
            evaluationFormat.campainEvaluationFormats = result || evaluationFormat.campainEvaluationFormats;
            logger.info('[EvaluationFormatService::get_evaluationformat] Sending response');
            res.json({ status: true, description: 'OK', data: evaluationFormat });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::get_evaluationformat] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationformat/save_evaluationformat', function (req, res, next) {
        logger.info('[EvaluationFormatService::save_evaluationformat] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var userId = req.app.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newEvaluationFormat = (req.body || {});
        newEvaluationFormat.createdDate = null;
        newEvaluationFormat.createdByUserId = null;
        newEvaluationFormat.lastModifiedDate = nowUtc;
        newEvaluationFormat.lastModifiedByUserId = userId;
        var oldTemplateId = newEvaluationFormat.templateId ? newEvaluationFormat.templateId : '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[EvaluationFormatService::save_evaluationformat] Looking for a Evaluation format: ' + newEvaluationFormat.description);
            return mssql_service_1.MSSql.GetEvaluationFormatDescription(instance, newEvaluationFormat.description);
        }).then(function (result) {
            var evaluationFormatDesc = {};
            evaluationFormatDesc = result || evaluationFormatDesc;
            if (result && evaluationFormatDesc.id !== newEvaluationFormat.id)
                throw 'Ya existe el formato de evaluación con la descripción \'' + newEvaluationFormat.description + '\'';
            logger.info('[EvaluationFormatService::save_evaluationformat] Saving evaluation format');
            newEvaluationFormat.templateId = (newEvaluationFormat.type === 'MANUAL_AUTOMATIC' || newEvaluationFormat.type === 'MIXED') ? ('qty-tmpl-' + utilities_1.guid()) : null;
            return mssql_service_1.MSSql.SaveEvaluationFormat(instance, newEvaluationFormat.id, newEvaluationFormat.description, newEvaluationFormat.type, newEvaluationFormat.environmentType, newEvaluationFormat.minQualificationAcceptable, newEvaluationFormat.minAutomaticQualification, newEvaluationFormat.managementFormURL, newEvaluationFormat.createdDate, newEvaluationFormat.createdByUserId, newEvaluationFormat.lastModifiedDate, newEvaluationFormat.lastModifiedByUserId, newEvaluationFormat.enabled, newEvaluationFormat.templateId);
        })
            .then(function (result) {
            logger.info('[EvaluationFormatService::save_evaluationformat] Result: ', result);
            if (result && result.length) {
                var status_1;
                var idEvaluationFormat = result[0].idEvaluationFormat;
                logger.info('[EvaluationFormatService::save_evaluationformat] idEvaluationFormat: ', idEvaluationFormat);
                if (idEvaluationFormat === -1) {
                    status_1 = 'EVALUATION_FORMAT_BUSY';
                    logger.info('[EvaluationFormatService::save_evaluationformat] result - status -1:', status_1);
                    return Promise.resolve(status_1);
                }
                else {
                    status_1 = idEvaluationFormat;
                    logger.info('[EvaluationFormatService::save_evaluationformat] result - status: ', status_1);
                    return mssql_service_1.MSSql.DeleteAbilityEvaluationFormat(instance, newEvaluationFormat.id)
                        .then(function (result) {
                        logger.info('[EvaluationFormatService::save_evaluationformat] Calling DeleteCampainEvaluationFormat SP');
                        return mssql_service_1.MSSql.DeleteCampainEvaluationFormat(instance, newEvaluationFormat.id);
                    }).then(function (result) {
                        if (oldTemplateId.length > 0) {
                            logger.info('[EvaluationFormatService::save_evaluationformat] Calling DeleteTemplate SP');
                            return mssql_service_1.MSSql.DeleteTemplate(instance, oldTemplateId);
                        }
                        return true;
                    }).then(function (resultDelete) {
                        logger.info('[EvaluationFormatService::save_evaluationformat] Calling SaveAbilityAndCAmpainEF function');
                        return SaveAbilityAndCAmpainEF(instance, newEvaluationFormat);
                    })
                        .then(function (result) {
                        if (newEvaluationFormat.type === 'MANUAL_AUTOMATIC' || newEvaluationFormat.type === 'MIXED') {
                            logger.info('[EvaluationFormatService::save_evaluationformat] Requesting AbilityQueries');
                            return SetAbilityQueriesIntoAbilityEvaluationFormat(instance, newEvaluationFormat);
                        }
                        return newEvaluationFormat;
                    })
                        .then(function (result) {
                        logger.info('[EvaluationFormatService::save_evaluationformat] Calling GetTemplate SP');
                        newEvaluationFormat = result;
                        return mssql_service_1.MSSql.GetTemplate(instance, newEvaluationFormat.templateId);
                    })
                        .then(function (result) {
                        if (!result) {
                            if (newEvaluationFormat.type === 'MANUAL_AUTOMATIC' || newEvaluationFormat.type === 'MIXED') {
                                logger.info('[EvaluationFormatService::save_evaluationformat] Calling GenerateTemplate function for MANUAL_AUTOMATIC EF');
                                return GenerateTemplate(instance, newEvaluationFormat);
                            }
                        }
                        return true;
                    })
                        .then(function (result) {
                        logger.info('[EvaluationFormatService::save_evaluationformat] Calling DeleteEvaluationFormatCustomField SP');
                        return mssql_service_1.MSSql.DeleteEvaluationFormatCustomFields(instance, newEvaluationFormat.id);
                    })
                        .then(function (result) {
                        if (newEvaluationFormat.type === 'MANUAL' || newEvaluationFormat.type === 'MIXED') {
                            logger.info('[EvaluationFormatService::save_evaluationformat] Calling SaveEvaluationFormatCustomFields function for MANUAL/MIXED EF');
                            return SaveEvaluationFormatCustomFields(instance, newEvaluationFormat, userId);
                        }
                        return Promise.resolve(true);
                    })
                        .then(function (result) {
                        logger.info('[EvaluationFormatService::save_evaluationformat] Calling DeleteEvaluationFormatCustomField SP');
                        return mssql_service_1.MSSql.DeleteEvaluationFormatAttachedSignatures(instance, newEvaluationFormat.id);
                    })
                        .then(function (result) {
                        logger.info('[EvaluationFormatService::save_evaluationformat] Calling SaveEvaluationFormatAttachedSignatures function');
                        return SaveEvaluationFormatAttachedSignatures(instance, newEvaluationFormat, userId);
                    });
                }
            }
            else {
                logger.error('[EvaluationFormatService::save_evaluationformat] result error: ', result);
                return Promise.reject(result);
            }
        })
            .then(function (resultSave) {
            logger.info('[EvaluationFormatService::save_evaluationformat] resultSave: ', resultSave);
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? resultSave === 'EVALUATION_FORMAT_BUSY' ? 'EVALUATION_FORMAT_BUSY' : 'OK' : 'Evaluation Format save failed'), data: resultSave });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::save_evaluationformat] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationformat/delete_evaluationformat', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var evaluationFormat = (req.body || {});
        var evaluationFormatId = evaluationFormat.id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!evaluationFormat)
                throw 'Missing evaluation format input parameter';
            return mssql_service_1.MSSql.DeleteAbilityEvaluationFormat(instance, evaluationFormatId);
        }).then(function (result) {
            return mssql_service_1.MSSql.DeleteEvaluationFormat(instance, evaluationFormatId);
        }).then(function (result) {
            if (!result)
                throw 'Se produjo un error al eliminar formato de evaluación';
            return Promise.resolve(true);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'OK' : 'Evaluation format delete failed'), data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationformat/delete_evaluationformats', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var evaluationFormats = req.body.evaluationFormats || [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!evaluationFormats)
                throw 'Missing evaluation formats input parameter';
            var promiseArray = [];
            evaluationFormats.forEach(function (evaluationFormat) {
                promiseArray.push(mssql_service_1.MSSql.DeleteAbilityEvaluationFormat(instance, evaluationFormat.id));
            });
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (!evaluationFormats)
                throw 'Missing evaluation formats input parameter';
            var promiseArray = [];
            evaluationFormats.forEach(function (evaluationFormat) {
                promiseArray.push(mssql_service_1.MSSql.DeleteEvaluationFormat(instance, evaluationFormat.id));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeletes) {
            var result;
            if (resultDeletes.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Evaluation formats eliminados correctamente');
            }
            return result;
        }).then(function (resultDeletes) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeletes, description: (resultDeletes ? 'OK' : 'Evaluation formats delete failed'), data: resultDeletes });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationformat/get_evaluationformat_qualification_average/:evaluationFormatId', function (req, res, next) {
        logger.info('[EvaluationFormatService::get_evaluationformat_qualification_average] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var evaluationFormatStr = req.params.evaluationFormatId || '';
        logger.info('[EvaluationFormatService::get_evaluationformat_qualification_average] Received instance: ' + instance);
        logger.info('[EvaluationFormatService::get_evaluationformat_qualification_average] Received evaluationFormatId: ' + evaluationFormatStr);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[EvaluationFormatService::get_evaluationformat_qualification_average] Calling GetEvaluationFormatQualificationAverage SP');
            return mssql_service_1.MSSql.GetEvaluationFormatQualificationAverage(instance, evaluationFormatStr);
        }).then(function (result) {
            logger.info('[EvaluationFormatService::get_evaluationformat_qualification_average] Received result from GetEvaluationFormatQualificationAverage SP ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::get_evaluationformat_qualification_average] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationformat/isQueryBeingUsedByQuality/:queryName', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var queryName = req.params.queryName;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!queryName)
                throw 'Missing queryName input parameter';
            return mssql_service_1.MSSql.IsQueryBeingUsedByQuality(instance, queryName);
        }).then(function (result) {
            if (res.headersSent)
                return;
            var data = result > 0 ? true : false;
            res.json({ status: true, description: 'OK', data: data });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationformat/isTemplateBeingUsedByQuality/:TemplateId', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var templateId = req.params.TemplateId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!templateId)
                throw 'Missing queryName input parameter';
            return mssql_service_1.MSSql.IsTemplateBeingUsedByQuality(instance, templateId);
        }).then(function (result) {
            if (res.headersSent)
                return;
            var data = result > 0 ? true : false;
            res.json({ status: true, description: 'OK', data: data });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationformat/get_evaluationformat_attachedsignatures/:evaluationformat_id', function (req, res, next) {
        logger.info('[EvaluationFormatService::get_evaluationformat_attachedsignatures] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var evaluationFormatAttachedSignatures = [];
        var evaluationFormatId = req.params.evaluationformat_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!evaluationFormatId)
                throw 'SERVER_ERROR_EVALUATION_FORMAT_ID_MISSING';
            logger.info('[EvaluationFormatService::get_evaluationformat_attachedsignatures] Calling GetEvaluationFormatAttachedSignatues SP');
            return mssql_service_1.MSSql.GetEvaluationFormatAttachedSignatures(vcc, evaluationFormatId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationFormatService::get_evaluationformat_attachedsignatures] Received restult from SP GetEvaluationFormatAttachedSignatues: ' + result.length);
            res.json({ status: true, description: 'OK', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationFormatService::get_evaluationformat_attachedsignatures] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.EvaluationFormatService = EvaluationFormatService;
function SaveAbilityAndCAmpainEF(instance, evaluationFormat) {
    var promiseArray = [];
    logger.info('[EvaluationFormatService::SaveAbilityAndCAmpainEF] Saving ' + JSON.stringify(evaluationFormat.abilityEvaluationFormats));
    if (evaluationFormat.abilityEvaluationFormats) {
        evaluationFormat.abilityEvaluationFormats.forEach(function (abilityEvaluationFormat) {
            promiseArray.push(mssql_service_1.MSSql.SaveAbilityEvaluationFormat(instance, evaluationFormat.id, abilityEvaluationFormat.idAbility, abilityEvaluationFormat.idGroup, abilityEvaluationFormat.idSubGroup, abilityEvaluationFormat.penalizes, abilityEvaluationFormat.score, abilityEvaluationFormat.order, abilityEvaluationFormat.conditional));
        });
    }
    if (evaluationFormat.campainEvaluationFormats) {
        evaluationFormat.campainEvaluationFormats.forEach(function (campainEvaluationFormat) {
            promiseArray.push(mssql_service_1.MSSql.SaveCampainEvaluationFormat(instance, evaluationFormat.id, campainEvaluationFormat.idCampain));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
function SaveEvaluationFormatCustomFields(vcc, evaluationFormat, userId) {
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        var promiseArray = [];
        evaluationFormat.evaluationFormatCustomFields.map(function (efcf) {
            promiseArray.push(mssql_service_1.MSSql.SaveEvaluationFormatCustomField(vcc, efcf.id, evaluationFormat.id, efcf.customField.id, efcf.order, userId, userId));
        });
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        if (result.filter(function (r) { return (r !== true); }).length > 0) {
            return Promise.reject(false);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[CustomFieldService::SaveEvaluationFormatCustomFields] Error: ' + errorMsg);
        return Promise.reject(false);
    });
}
function SaveEvaluationFormatAttachedSignatures(vcc, evaluationFormat, userId) {
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        var promiseArray = [];
        evaluationFormat.evaluationFormatAttachedSignatures.map(function (efas) {
            if (efas && efas.description && efas.description !== '')
                promiseArray.push(mssql_service_1.MSSql.SaveEvaluationFormatAttachedSignature(vcc, evaluationFormat.id, efas.description, efas.order, userId, userId));
        });
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        if (result.filter(function (r) { return (r !== true); }).length > 0) {
            return Promise.reject(false);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[CustomFieldService::SaveEvaluationFormatAttachedSignatures] Error: ' + errorMsg);
        return Promise.reject(false);
    });
}
function SetAbilityQueriesIntoAbilityEvaluationFormat(vcc, evaluationFormat) {
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        var promiseArray = [];
        evaluationFormat.abilityEvaluationFormats.map(function (aef) {
            promiseArray.push(ability_service_1.GetAbilityQueries(vcc, aef.idAbility, false));
        });
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        result.map(function (r, i) {
            evaluationFormat.abilityEvaluationFormats[i].queries = r;
        });
        return Promise.resolve(evaluationFormat);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error('[EvaluationFormatService::SetAbilityQueriesIntoAbilityEvaluationFormat] ', errorMsg);
        return Promise.reject(err);
    });
}
function GenerateTemplate(instance, evaluationFormat) {
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        logger.error('[EvaluationFormatService::GenerateTemplate] Creating Template');
        var template = new template_model_1.Template('custom', evaluationFormat.templateId, evaluationFormat.description, null, null, moment().utc().toDate(), evaluationFormat.createdByUserId, moment().utc().toDate(), evaluationFormat.lastModifiedByUserId);
        var i = 0;
        var category = '';
        var _loop_1 = function (aef) {
            i++;
            if (category !== aef.idGroup) {
                i = 1;
                category = aef.idGroup;
            }
            var tempTheme = new template_model_1.TemplateTheme(template.name, utilities_1.guid(), i, aef.titleAbility, aef.descriptionGroup, aef.descriptionSubGroup, moment().utc().toDate(), template.createdByUserId, moment().utc().toDate(), template.lastModifiedByUserId);
            tempTheme.isNegative = null;
            tempTheme.analysisType = null;
            var tempThemeItem = null;
            if (aef.queries && aef.queries.length) {
                aef.queries.map(function (q) {
                    tempThemeItem = new template_model_1.TemplateThemeItem(template.name, tempTheme.id, q.queryId, q.queryId, moment().utc().toDate(), template.createdByUserId, moment().utc().toDate(), template.lastModifiedByUserId);
                    tempThemeItem.acceptancePercentage = q.acceptancePercentage;
                    template.templateThemeItems.push(tempThemeItem);
                });
            }
            else {
                tempThemeItem = new template_model_1.TemplateThemeItem(template.name, tempTheme.id, aef.idAbility, aef.idAbility, moment().utc().toDate(), template.createdByUserId, moment().utc().toDate(), template.lastModifiedByUserId);
                template.templateThemeItems.push(tempThemeItem);
            }
            template.templateThemes.push(tempTheme);
        };
        for (var _i = 0, _a = evaluationFormat.abilityEvaluationFormats; _i < _a.length; _i++) {
            var aef = _a[_i];
            _loop_1(aef);
        }
        return template_service_1.SaveTemplate(instance, template);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error('[EvaluationFormatService::GenerateTemplate] ', errorMsg);
        return false;
    });
}
