"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var utilities_1 = require("../shared/utilities");
var exception_mssql_1 = require("../db/mssql/exception.mssql");
var logger = log4js.getLogger('ServerScripts');
function ExceptionService(app) {
    app.get('/api/exception/get_exceptions/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'' + ' | ' + res.locals.instance);
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetExceptions(vcc, search_criteria, search_expression);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var groupMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: groupMsg, error: errorMsg });
        });
    });
    app.get('/api/exception/get_exception/:exception_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var exceptionId = req.params.exception_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!exceptionId)
                throw 'Missing exception input parameter';
            return mssql_service_1.MSSql.GetException(vcc, exceptionId);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/exception/create_exception', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'' + ' | ' + res.locals.instance);
        var vcc = res.locals.vcc;
        logger.info('[ExceptionService::create_exception] params: ' + vcc);
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newException = (req.body || {});
        newException.createdDate = nowUtc;
        newException.createdByUserId = userId;
        newException.lastModifiedDate = nowUtc;
        newException.lastModifiedByUserId = userId;
        newException.description = newException.description.trim();
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetException(vcc, null, newException.description);
        }).then(function (result) {
            if (result)
                throw 'Ya existe una excepcion con la descripción \'' + newException.description + '\'';
            newException.id = utilities_1.guid();
            return exception_mssql_1.SaveException(vcc, newException);
        }).then(function (result) {
            if (!result)
                throw 'Se produjo un error al guardar la excepcion';
            return Promise.resolve(newException.id);
        }).then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSave ? 'OK' : 'Exception save failed'), data: resultSave });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/exception/save_exception', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'' + ' | ' + res.locals.instance);
        var vcc = res.locals.vcc;
        var userId = req.app.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newException = (req.body || {});
        newException.createdDate = null;
        newException.createdByUserId = null;
        newException.lastModifiedDate = nowUtc;
        newException.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetException(vcc, null, newException.description);
        }).then(function (result) {
            var exceptionDesc = {};
            exceptionDesc = result || exceptionDesc;
            if (result && exceptionDesc.id !== newException.id)
                throw 'Ya existe una excepcion con la descripción \'' + newException.description + '\'';
            return exception_mssql_1.SaveException(vcc, newException);
        }).then(function (resultSave) {
            if (!resultSave)
                throw 'Se produjo un error al guardar la excepcion';
            return Promise.resolve(true);
        }).then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'OK' : 'Exception save failed'), data: resultSave });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/exception/delete_exception', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var exception = req.body.exception;
        var exceptionId = exception.id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!exception)
                throw 'Missing client input parameter';
            return mssql_service_1.MSSql.DeleteException(vcc, exceptionId);
        }).then(function (resultDeleteException) {
            if (!resultDeleteException)
                throw 'Se produjo un error al eliminar la excepcion';
            return Promise.resolve(true);
        }).then(function (resultDeleteException) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteException, description: (resultDeleteException ? 'OK' : 'Exception delete failed'), data: resultDeleteException });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/exception/delete_exceptions', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var exceptions = req.body.exceptions || [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!exceptions)
                throw 'Missing exceptions input parameter';
            var promiseArray = [];
            exceptions.forEach(function (exception) {
                promiseArray.push(mssql_service_1.MSSql.DeleteException(vcc, exception.id));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteExceptions) {
            var result;
            if (resultDeleteExceptions.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Excepciones eliminados correctamente');
            }
            return result;
        }).then(function (resultDeleteExceptions) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteExceptions, description: (resultDeleteExceptions ? 'OK' : 'Exceptions delete failed'), data: resultDeleteExceptions });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.ExceptionService = ExceptionService;
