"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function FeelingService(app) {
    app.get('/api/feeling/get_feelings/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetFeelings(vcc, search_criteria, search_expression);
        }).then(function (feelings) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: feelings });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.FeelingService = FeelingService;
function SaveFeelings(vcc, feeling) {
    var promiseArray = [];
    if (feeling) {
        feeling.forEach(function (feelings) {
            promiseArray.push(mssql_service_1.MSSql.SaveFeeling(vcc, feelings.name, feelings.description, feelings.createdDate, feelings.createdByUserId, feelings.lastModifiedDate, feelings.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveFeelings = SaveFeelings;
