"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var utilities_1 = require("../shared/utilities");
var logger = log4js.getLogger('ServerScripts');
function GroupService(app) {
    app.get('/api/group/get_groups/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'' + ' | ' + res.locals.instance);
        var instance = res.locals.instance;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var disabled = req.query.disabled === 'true' || false;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetGroups(instance, search_criteria, search_expression, disabled);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var groupMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: groupMsg, error: errorMsg });
        });
    });
    app.post('/api/group/create_group', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'' + ' | ' + res.locals.instance);
        var instance = res.locals.instance;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newGroup = (req.body || {});
        newGroup.createdDate = nowUtc;
        newGroup.createdByUserId = userId;
        newGroup.lastModifiedDate = nowUtc;
        newGroup.lastModifiedByUserId = userId;
        newGroup.description = newGroup.description.trim();
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetGroupDescription(instance, newGroup.description);
        }).then(function (result) {
            if (result)
                throw 'Ya existe un grupo con la descripción \'' + newGroup.description + '\'';
            newGroup.id = utilities_1.guid();
            return SaveGroup(instance, newGroup);
        }).then(function (result) {
            if (!result)
                throw 'Se produjo un error al guardar el grupo';
            return Promise.resolve(newGroup.id);
        }).then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSave ? 'OK' : 'Group save failed'), data: resultSave });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/group/get_group/:group_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var groupId = req.params.group_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!groupId)
                throw 'Missing group input parameter';
            return mssql_service_1.MSSql.GetGroup(instance, groupId);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/group/save_group', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'' + ' | ' + res.locals.instance);
        var instance = res.locals.instance;
        var userId = req.app.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newGroup = (req.body || {});
        newGroup.createdDate = null;
        newGroup.createdByUserId = null;
        newGroup.lastModifiedDate = nowUtc;
        newGroup.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetGroupDescription(instance, newGroup.description);
        }).then(function (result) {
            var grupDesc = {};
            grupDesc = result || grupDesc;
            if (result && grupDesc.id !== newGroup.id)
                throw 'Ya existe un grupo con la descripción \'' + newGroup.description + '\'';
            return SaveGroup(instance, newGroup);
        }).then(function (resultSave) {
            if (!resultSave)
                throw 'Se produjo un error al guardar el grupo';
            return Promise.resolve(true);
        }).then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'OK' : 'Category save failed'), data: resultSave });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/group/delete_group', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var group = (req.body || {});
        var groupId = group.id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!group)
                throw 'Missing group input parameter';
            return mssql_service_1.MSSql.DeleteGroup(instance, groupId);
        }).then(function (resultDelete) {
            if (!resultDelete)
                throw 'Se produjo un error al eliminar el grupo';
            return Promise.resolve(true);
        }).then(function (resultDelete) {
            if (res.headersSent)
                return;
            res.json({ status: resultDelete, description: (resultDelete ? 'OK' : 'Group delete failed'), data: resultDelete });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/group/delete_groups', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var groups = req.body.groups || [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!groups)
                throw 'Missing groups input parameter';
            var promiseArray = [];
            groups.forEach(function (group) {
                promiseArray.push(mssql_service_1.MSSql.DeleteGroup(instance, group.id));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDelete) {
            var result;
            if (resultDelete.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Groups eliminados correctamente');
            }
            return result;
        }).then(function (resultDelete) {
            if (res.headersSent)
                return;
            res.json({ status: resultDelete, description: (resultDelete ? 'OK' : 'Groups delete failed'), data: resultDelete });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.GroupService = GroupService;
function SaveGroup(instance, group) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveGroup(instance, group.id, group.description, group.createdDate, group.createdByUserId, group.lastModifiedDate, group.lastModifiedByUserId, group.enabled));
    return Promise.all(promiseArray)
        .then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
