"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var log4js = require("log4js");
var moment = require("moment");
var report_implementation_1 = require("../implementation/report.implementation");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function ReportService(app) {
    app.get('/api/report/get_reports/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[ReportService::get_reports] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var role = res.locals.userRole;
        var userId = res.locals.userId;
        if (userId === 'admin' && vcc === 'system') {
            role.id = 'provisioning';
        }
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (role.id == 'business_administrator' || role.id == 'provisioning') {
                return mssql_service_1.MSSql.GetReports(vcc, search_criteria, search_expression, role.id);
            }
            else {
                return mssql_service_1.MSSql.GetReports(vcc, search_criteria, search_expression, null);
            }
        })
            .then(function (resultReports) {
            if (!resultReports)
                throw 'No se pudo ejecutar consulta de reportes';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: resultReports });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/report/get_report/:report_id', function (req, res, next) {
        logger.info('[ReportService::get_report] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var role = res.locals.userRole;
        var reportId = req.params.report_id;
        var userId = res.locals.userId;
        if (userId === 'admin') {
            role.id = 'business_administrator';
        }
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!reportId)
                throw 'Missing report input parameter';
            if (role.id == 'business_administrator') {
                return mssql_service_1.MSSql.GetReport(vcc, reportId, role.id);
            }
            else {
                return mssql_service_1.MSSql.GetReport(vcc, reportId, null);
            }
        }).then(function (report) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: report });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/report/create_report', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newReport = (req.body || {});
        newReport.createdDate = nowUtc;
        newReport.createdByUserId = userId;
        newReport.lastModifiedDate = nowUtc;
        newReport.lastModifiedByUserId = userId;
        newReport.startDate = (newReport.startDate ? new Date(newReport.startDate) : null);
        newReport.endDate = (newReport.endDate ? new Date(newReport.endDate) : null);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetReport(vcc, newReport.id);
        }).then(function (resultReport) {
            if (resultReport)
                throw 'Ya existe un reporte con el identificador \'' + newReport.id + '\'';
            return mssql_service_1.MSSql.SaveReport(vcc, newReport.id, newReport.description, newReport.type, newReport.procedure, newReport.startDate, newReport.endDate, newReport.frecuency, newReport.hourSpace, newReport.exactTime, newReport.weekday, newReport.dayMonth, newReport.summary, newReport.filter, newReport.email, newReport.createdDate, newReport.createdByUserId, newReport.lastModifiedDate, newReport.lastModifiedByUserId);
        }).then(function (resultSaveReport) {
            if (!resultSaveReport)
                throw 'Error al guardar datos del reporte';
            return mssql_service_1.MSSql.SaveSinglePermission(vcc, 'REPORT', newReport.id);
        }).then(function (result) {
            if (res.headersSent)
                return;
            logger.info('Reporte \'' + newReport.id + '\' creado correctamente');
            res.json({ status: result, description: (result ? 'OK' : 'Report save failed'), data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/report/save_report', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var role = res.locals.userRole;
        var nowUtc = moment().utc().toDate();
        var newReport = (req.body || {});
        newReport.createdDate = null;
        newReport.createdByUserId = null;
        newReport.lastModifiedDate = nowUtc;
        newReport.lastModifiedByUserId = userId;
        newReport.startDate = (newReport.startDate ? new Date(newReport.startDate) : null);
        newReport.endDate = (newReport.endDate ? new Date(newReport.endDate) : null);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var allowedReports = (role && role.singlePermissions ?
                role && role.singlePermissions.filter(function (sp) { return sp.type === 'REPORT'; }).map(function (sp) { return sp.element; }) :
                []);
            if (!allowedReports.includes(newReport.id)) {
                res.sendStatus(403);
                throw 'El usuario no tiene permisos para acceder al elemento (REPORT / ' + newReport.id + ')';
            }
            return mssql_service_1.MSSql.SaveReport(vcc, newReport.id, newReport.description, newReport.type, newReport.procedure, newReport.startDate, newReport.endDate, newReport.frecuency, newReport.hourSpace, newReport.exactTime, newReport.weekday, newReport.dayMonth, newReport.summary, newReport.filter, newReport.email, newReport.createdDate, newReport.createdByUserId, newReport.lastModifiedDate, newReport.lastModifiedByUserId);
        }).then(function (resultSaveReport) {
            if (!resultSaveReport)
                throw 'Error al guardar datos del reporte';
            return mssql_service_1.MSSql.SaveSinglePermission(vcc, 'REPORT', newReport.id);
        }).then(function (result) {
            if (res.headersSent)
                return;
            logger.info('Reporte \'' + newReport.id + '\' actualizado correctamente');
            res.json({ status: result, description: (result ? 'OK' : 'Report save failed'), data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/report/delete_report', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var role = res.locals.userRole;
        var report = req.body.report || {};
        var reportId = report.id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!report)
                throw 'Missing report input parameter';
            var allowedReports = (role && role.singlePermissions ?
                role && role.singlePermissions.filter(function (sp) { return sp.type === 'REPORT'; }).map(function (sp) { return sp.element; }) :
                []);
            if (!allowedReports.includes(reportId)) {
                res.sendStatus(403);
                throw 'El usuario no tiene permisos para acceder al elemento (REPORT / ' + reportId + ')';
            }
            return mssql_service_1.MSSql.DeleteReport(vcc, reportId);
        }).then(function (resultDeleteReport) {
            if (!resultDeleteReport)
                throw 'Error al guardar datos del reporte';
            return mssql_service_1.MSSql.DeleteSinglePermission(vcc, 'REPORT', reportId);
        }).then(function (result) {
            if (res.headersSent)
                return;
            logger.info('Reporte \'' + reportId + '\' eliminado correctamente');
            res.json({ status: result, description: (result ? 'OK' : 'Report delete failed'), data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/report/delete_reports', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var role = res.locals.userRole;
        var reports = req.body.reports || [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!reports)
                throw 'Missing reports input parameter';
            var allowedReports = (role && role.singlePermissions ?
                role && role.singlePermissions.filter(function (sp) { return sp.type === 'REPORT'; }).map(function (sp) { return sp.element; }) :
                []);
            var forbiddenReportId;
            reports.forEach(function (report) {
                if (!forbiddenReportId && !allowedReports.includes(report.id)) {
                    forbiddenReportId = report.id;
                }
            });
            if (forbiddenReportId) {
                res.sendStatus(403);
                throw 'El usuario no tiene permisos para acceder al elemento (REPORT / ' + forbiddenReportId + ')';
            }
            var promiseArray = [];
            reports.forEach(function (report) {
                promiseArray.push(mssql_service_1.MSSql.DeleteReport(vcc, report.id));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteReports) {
            var result;
            if (resultDeleteReports.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Reportes eliminados correctamente');
            }
            var promiseArray = [];
            reports.forEach(function (report) {
                promiseArray.push(mssql_service_1.MSSql.DeleteSinglePermission(vcc, 'REPORT', report.id));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteSinglePermissions) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteSinglePermissions, description: (resultDeleteSinglePermissions ? 'OK' : 'Reports delete failed'), data: resultDeleteSinglePermissions });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/report/get_report_procedures', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetReportProcedures();
        }).then(function (response) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: response });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/report/get_report_procedure_parameters/:procedure', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var role = res.locals.userRole;
        var userId = res.locals.userId;
        var procedure = req.params.procedure;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (role.id === 'business_administrator' || userId === 'admin') {
                var report = new report_implementation_1.ReportImplementation();
                return report.GetReportProceduraParameters(procedure);
            }
            else {
                return mssql_service_1.MSSql.GetReportProcedureParameters(procedure);
            }
        }).then(function (response) {
            if (res.headersSent)
                return;
            var procedureParameter = [];
            if (response) {
                response.forEach(function (parameter) {
                    if (userId === 'admin') {
                        if (parameter.parameter !== 'PlatformHash' && parameter.parameter !== '@VCC' && parameter.parameter !== '@UserId' && parameter.parameter !== '@ClientOffset' && !parameter.isOutput) {
                            if (parameter.type === 'datetime') {
                                parameter.type = 'date';
                            }
                            else {
                                parameter.type = 'text';
                            }
                            parameter.parameter = parameter.parameter.replace('@', '');
                            procedureParameter.push(parameter);
                        }
                    }
                    else {
                        if (parameter.parameter !== 'SubAccountId' && parameter.parameter !== 'AccountId' && parameter.parameter !== 'InstallationId' && parameter.parameter !== 'PlatformHash' && parameter.parameter !== 'VCC' && parameter.parameter !== '@VCC' && parameter.parameter !== '@UserId' && parameter.parameter !== '@ClientOffset' && !parameter.isOutput) {
                            if (parameter.type === 'datetime') {
                                parameter.type = 'date';
                            }
                            else {
                                parameter.type = 'text';
                            }
                            parameter.parameter = parameter.parameter.replace('@', '');
                            procedureParameter.push(parameter);
                        }
                    }
                });
            }
            res.json({ status: true, description: 'OK', data: procedureParameter });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/report/execute_report', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var role = res.locals.userRole;
        var reportExecution = req.body.reportExecution || {};
        var procedure = reportExecution.procedure;
        var filters = reportExecution.filters;
        var clientOffset = reportExecution.clientOffset;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!filters && !procedure)
                throw 'Missing report input data';
            var allowedReports = (role && role.singlePermissions ?
                role && role.singlePermissions.filter(function (sp) { return sp.type === 'REPORT'; }).map(function (sp) { return sp.element; }) :
                []);
            if (role.id === 'business_administrator' || userId === 'admin') {
                var report = new report_implementation_1.ReportImplementation();
                return report.GetReportProceduraParameters(procedure);
            }
            else {
                return mssql_service_1.MSSql.GetReportProcedureParameters(procedure);
            }
        }).then(function (resultParameters) {
            var procedureParameters = [];
            var momentDate;
            resultParameters.forEach(function (parameter) {
                var column = parameter.parameter.replace('@', '');
                var colFilter = filters.find(function (filter) { return filter && filter.column === column; });
                var value = (colFilter && colFilter.value ? colFilter.value : null);
                if (value && (column === 'EndDate' || column === 'StartDate')) {
                    var momentDate_1 = moment.parseZone(value).isValid() && moment.parseZone(value) || null;
                    value = (momentDate_1) ? momentDate_1.utc().add(clientOffset, 'minutes').toDate() : null;
                }
                if (!value && column === 'VCC')
                    value = vcc;
                else if (!value && column === 'UserId')
                    value = userId;
                else if (!value && column === 'ClientOffset')
                    value = clientOffset;
                if (column !== 'RETURN_VALUE')
                    procedureParameters.push({
                        column: column,
                        value: value
                    });
            });
            if (role.id === 'business_administrator' || userId === 'admin') {
                var report = new report_implementation_1.ReportImplementation();
                return report.ReportExecution(procedureParameters, procedure);
            }
            else {
                return mssql_service_1.MSSql.ExecuteReportProcedure(procedure, procedureParameters);
            }
        }).then(function (responseReportData) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: responseReportData });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.ReportService = ReportService;
