"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var redis_service_1 = require("../db/redis.service");
var billing_batch_implementation_1 = require("../implementation/billing-batch.implementation");
var billing_implementation_1 = require("../implementation/billing.implementation");
var billing_model_1 = require("../models/billing.model");
var platform_service_1 = require("./platform.service");
var logger = log4js.getLogger('ServerScripts');
function StudyService(app) {
    app.post('/api/study/begin_study', function (req, res, next) {
        logger.info('[StudyService::begin_study] Received ' + req.method + ' on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var param = (req.body || {});
        var currentQueueId = '';
        var study = {};
        var channelSummaries = [];
        var totalSyncInteraction = 0;
        var totalAudios = 0;
        var totalAsyncInteraction = 0;
        var totalAsyncDetail = 0;
        var averageModelRequest = 0;
        var nowUtc = moment().utc().toDate();
        var batch = {};
        var billingVccRates = new billing_model_1.BillingVCCRates();
        var status = false;
        var saveResult = null;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetStudy(vcc, param.studyId);
        })
            .then(function (result) {
            if (!result)
                throw 'STUDY_NOT_FOUND';
            study = result;
            var billing = new billing_implementation_1.BillingImplementation();
            return billing.GetVCCRates(vcc, study.processId);
        })
            .then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_RATES_NOT_FOUND';
            billingVccRates.SetBillingVCCRates(result);
            var promiseArray = [];
            promiseArray.push(platform_service_1.GetPlatformVersion());
            promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeItems(vcc, study.templateId, null));
            promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeModels(vcc, study.templateId, null));
            promiseArray.push(mssql_service_1.MSSql.GetProcessInteractionsHistory(vcc, study.processId, billingVccRates.cognitiveAnalysisAverageSecondsPerRequest));
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (result[0] === null)
                throw 'INSTALLATIONID_MISSING';
            if (result[1] === null)
                throw 'QUERIES_INFO_MISSING';
            if (result[2] === null)
                throw 'MODELS_INFO_MISSING';
            if (result[3] === null)
                throw 'INTERACTIONS_INFO_MISSING';
            var modelIntents = result[2].map(function (mi) { return mi.id; });
            var models = modelIntents.map(function (i) { return i.split('|')[0]; });
            var nluModels = models.filter(function (item, i, ar) { return ar.indexOf(item) === i; });
            channelSummaries = result[3];
            channelSummaries.map(function (cs) {
                if (cs) {
                    if (cs.interactionType === 'CALL') {
                        totalSyncInteraction = cs.totalRecords;
                        totalAudios = cs.audios;
                        averageModelRequest += cs.totalCallPhrases || cs.averageModelRequest || 0;
                    }
                    else {
                        totalAsyncInteraction += cs.totalInteractions;
                        totalAsyncDetail += cs.totalInteractionDetails;
                        averageModelRequest += cs.totalTextPhrases || cs.averageModelRequest || 0;
                        ;
                    }
                }
            });
            batch.installationId = result[0].installationId;
            batch.vcc = vcc;
            batch.totalAsync = 0;
            batch.totalAsyncDetail = 0;
            batch.totalSync = 0;
            batch.totalAudio = 0;
            batch.durationTime = 0;
            batch.roundedTime = 0;
            batch.totalQueries = result[1].length;
            batch.totalModels = nluModels.length;
            batch.modelRequests = billingVccRates.isProcessPostChannel ? billingVccRates.GetCognitiveAnalysisTotalAveragePhrasesFromProcess() : averageModelRequest;
            batch.batchTypeName = 'STUDY';
            batch.batchTypeId = study.name;
            batch.templateId = study.templateId;
            batch.processId = study.processId;
            batch.userApi = userId;
            batch.startDate = nowUtc;
            var billing = new billing_batch_implementation_1.BillingBatchImplementation();
            return billing.SaveBatchSummary(batch);
        })
            .then(function (result) {
            if (typeof result !== 'boolean') {
                if (result) {
                    if (result.status === true) {
                        status = true;
                        study.status = 0.1;
                        var lastModifiedDate = moment().utc().toDate();
                        return mssql_service_1.MSSql.SaveStudyStatus(vcc, param.processId, study.templateId, param.studyId, study.status, null, null, lastModifiedDate, userId);
                    }
                    else {
                        status = false;
                        saveResult = result.error ? result.error : 'SERVER_ERROR_CAN_NOT_SAVE_BATCH';
                    }
                }
                else {
                    return Promise.resolve(false);
                }
            }
            else {
                return false;
            }
        })
            .then(function (result) {
            if (status === true) {
                currentQueueId = vcc;
                currentQueueId += ':';
                currentQueueId += study.processId;
                currentQueueId += ':study:';
                currentQueueId += study.name;
                return redis_service_1.Redis.CreateQueueL('Process', currentQueueId);
            }
            return false;
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (status ? 'OK' : saveResult ? saveResult : 'STUDY_START_FAILED'), data: status });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[StudyService::begin_study with param:', param, 'an error has occured in:', req.path, 'with error meesage:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/study/get_studies', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var study = (req.body || {});
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetStudies(vcc, study.searchCriteria, study.searchExpression, study.startDate, study.endDate, study.type);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/study/get_study_result_summary/:study_id/?', function (req, res, next) {
        logger.info('[StudyService::get_study_result_summary] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var studyId = req.params.study_id || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetStudyResultSummary(vcc, studyId, null);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/study/get_study/:study_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var studyId = req.params.study_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!studyId)
                throw 'Missing study input parameter';
            return mssql_service_1.MSSql.GetStudy(vcc, studyId);
        }).then(function (resultStudy) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: resultStudy });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/study/create_study', function (req, res, next) {
        logger.info('[StudyService::create_study] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newStudy = (req.body || {});
        newStudy.name = (newStudy.name || '').toLowerCase();
        newStudy.createdDate = nowUtc;
        newStudy.createdByUserId = userId;
        newStudy.lastModifiedDate = nowUtc;
        newStudy.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetStudyByName(vcc, newStudy.name);
        })
            .then(function (resultStudy) {
            if (resultStudy)
                throw 'Ya existe un estudio con el identificador \'' + newStudy.name + '\'';
            return platform_service_1.GetPlatformVersion();
        })
            .then(function (result) {
            if (result === null)
                throw 'INSTALLATIONID_MISSING';
            var batch = {};
            batch.installationId = result.installationId;
            batch.vcc = vcc;
            batch.batchTypeName = 'STUDY';
            batch.batchTypeId = newStudy.name;
            var billing = new billing_batch_implementation_1.BillingBatchImplementation();
            return billing.FindBatchId(batch);
        })
            .then(function (result) {
            if (result && result.result && result.result.length > 0)
                throw 'ERROR_STUDY_ID_INVALID';
            return SaveStudy(vcc, newStudy);
        })
            .then(function (resultSaveStudy) {
            if (!resultSaveStudy)
                throw 'Se produjo un error al guardar el estudio.';
            return resultSaveStudy;
        }).then(function (resultSaveStudy) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSaveStudy ? 'OK' : 'Study save failed'), data: resultSaveStudy });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[StudyService::create_study] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/study/save_study', function (req, res, next) {
        logger.info('[StudyService::save_study] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newStudy = (req.body || {});
        newStudy.name = (newStudy.name || '').toLowerCase();
        newStudy.createdDate = null;
        newStudy.createdByUserId = null;
        newStudy.lastModifiedDate = nowUtc;
        newStudy.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return SaveStudy(vcc, newStudy);
        }).then(function (resultSaveStudy) {
            if (!resultSaveStudy)
                throw 'Se produjo un error al guardar el estudio.';
            return Promise.resolve(true);
        }).then(function (resultSaveStudy) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveStudy, description: (resultSaveStudy ? 'OK' : 'Study save failed'), data: resultSaveStudy });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/study/delete_study', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var study = req.body.study;
        var studyId = study.name;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!study)
                throw 'Missing study input parameter';
            return mssql_service_1.MSSql.DeleteStudy(vcc, studyId);
        }).then(function (resultDeleteStudy) {
            if (!resultDeleteStudy)
                throw 'Se produjo un error al eliminar el estudio';
            return Promise.resolve(true);
        }).then(function (resultDeleteStudy) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteStudy, description: (resultDeleteStudy ? 'OK' : 'Study delete failed'), data: resultDeleteStudy });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/study/delete_studies', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var studies = req.body.studies || [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!studies)
                throw 'Missing studies input parameter';
            var promiseArray = [];
            studies.forEach(function (study) {
                promiseArray.push(mssql_service_1.MSSql.DeleteStudy(vcc, study.name));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteStudies) {
            var result;
            if (resultDeleteStudies.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Estudios eliminados correctamente');
            }
            return result;
        }).then(function (resultDeleteStudies) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteStudies, description: (resultDeleteStudies ? 'OK' : 'Studies delete failed'), data: resultDeleteStudies });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/public/study/get_study_report/:start_date/:end_date/:processes', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var startDate = req.params.start_date || '';
        var endDate = req.params.end_date || '';
        var processes = req.params.processes || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetStudyReport(startDate, endDate, processes);
        })
            .then(function (result) {
            return Promise.resolve(JSONToCSV(result));
        }).then(function (result) {
            if (res.headersSent)
                return;
            var fileName = [startDate.replace('-', ''), endDate.replace('-', ''), processes.replace(',', '_')].join('_');
            res.setHeader('Content-disposition', 'attachment; filename=' + fileName + '.csv');
            res.set('Content-Type', 'text/csv');
            res.status(200).send(result);
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
}
exports.StudyService = StudyService;
function GetStudy(vcc, studyId) {
    studyId = (studyId || '').toLowerCase();
    var cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(function () {
        return mssql_service_1.MSSql.GetStudy(vcc, studyId);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('Error al ejecutar GetStudy: ' + errMsg);
    });
}
exports.GetStudy = GetStudy;
function SaveStudy(vcc, study) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveStudy(vcc, study.processId, study.templateId, study.name, study.description, study.status, study.type, study.createdDate, study.createdByUserId, study.lastModifiedDate, study.lastModifiedByUserId));
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
function JSONToCSV(data) {
    var items = data, replacer = function (key, value) { return value === null ? '' : value; }, header = Object.keys(items[0]), csv = items.map(function (row) { return header.map(function (fieldName) { return JSON.stringify(row[fieldName], replacer); }).join(','); });
    csv.unshift(header.join(','));
    csv = csv.join('\r\n');
    return csv;
}
exports.JSONToCSV = JSONToCSV;
