"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
var nlu_service_1 = require("./nlu.service");
var logger = log4js.getLogger('ServerScripts');
function UtteranceService(app) {
    var cognitiveService;
    app.get('/api/utterance/get_utterances/:app_id/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[UtteranceService::get_utterances] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var appId = req.params.app_id || '';
        var implUtterances = [];
        var dbUtterances = [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!appId)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, appId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_NOT_FOUND';
            if (result.isSystemApp) {
                return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
            }
            else {
                return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
            }
        })
            .then(function (result) {
            cognitiveService = result;
            return cognitiveService.GetUtterancesList(azure.subscriptionRegion, azure.subscriptionKey, appId);
        })
            .then(function (utterances) {
            implUtterances = utterances;
            return GetUtterances(vcc, appId, null, false);
        })
            .then(function (utterances) {
            dbUtterances = utterances;
            var promiseArray = [];
            implUtterances.map(function (iu) {
                if (!dbUtterances.find(function (du) { return du.id === iu.id; }) && iu.intentLabel !== 'None') {
                    promiseArray.push(mssql_service_1.MSSql.SaveUtterance(vcc, appId, iu.intentLabel, iu.id, iu.text, 'system'));
                }
            });
            dbUtterances.map(function (du) {
                if (!implUtterances.find(function (iu) { return iu.id === du.id; })) {
                    promiseArray.push(mssql_service_1.MSSql.DeleteUtterance(vcc, appId, du.intentId, du.Id));
                }
            });
            return Promise.resolve(promiseArray);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: implUtterances });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/utterance/get_utterance/:utterance_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var utteranceId = req.params.utterance_id;
        var utterance;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!utteranceId)
                throw 'Missing utterance input parameter';
            return Promise.resolve(true);
        }).then(function (resultUtterance) {
            utterance = resultUtterance;
            return Promise.resolve(true);
        }).then(function (result) {
            return utterance;
        }).then(function (utterance) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: utterance });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/utterance/get_utterances_by_interaction/:audio_id/:section/:part', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var audioId = req.params.audio_id;
        var section = req.params.section;
        var part = req.params.part;
        logger.info('Request parameters: ', { audioId: audioId, section: section, part: part });
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!audioId)
                throw 'Missing audioId input parameter';
            if (!section)
                throw 'Missing section input parameter';
            if (!part)
                throw 'Missing part input parameter';
            return mssql_service_1.MSSql.GetUtterancesByInteraction(vcc, audioId, section, part);
        }).then(function (resultUtterances) {
            logger.info('resultUtterances: ', resultUtterances);
            return Promise.resolve(resultUtterances);
        }).then(function (resultUtterances) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: resultUtterances });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/create_utterance', function (req, res, next) {
        logger.info('[UtteranceService::create_utterance] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var nlu = null;
        var newUtterance = (req.body || {});
        newUtterance.name = newUtterance.name.trim();
        var createUtteranceResult = null;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            if (!newUtterance.appId)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, newUtterance.appId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_NOT_FOUND';
            nlu = result;
            if (nlu.isSystemApp) {
                return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
            }
            else {
                return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
            }
        })
            .then(function (result) {
            cognitiveService = result;
            if (nlu.isSystemApp) {
                return CreateSystemAppIfNotExists(cognitiveService, vcc, nlu);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(function (_) {
            return cognitiveService.AddUtterance(azure.subscriptionRegion, azure.subscriptionKey, newUtterance.appId, newUtterance);
        })
            .then(function (resultSaveUtterance) {
            if (!resultSaveUtterance)
                throw 'Se produjo un error al guardar la expresión';
            createUtteranceResult = resultSaveUtterance;
            var promiseArray = [];
            promiseArray.push(mssql_service_1.MSSql.SaveUtterance(vcc, newUtterance.appId, newUtterance.intent, resultSaveUtterance.ExampleId, resultSaveUtterance.UtteranceText, userId));
            nlu.needTrain = true;
            nlu.lastModifiedDate = nowUtc;
            nlu.lastModifiedByUserId = userId;
            promiseArray.push(nlu_service_1.SaveNLU(vcc, nlu));
            return Promise.all(promiseArray);
        })
            .then(function (results) {
            if (newUtterance.entityLabels && newUtterance.entityLabels.length > 0) {
                var newUtteranceId = results[0];
                if (newUtterance) {
                    var promises = [];
                    for (var _i = 0, _a = newUtterance.entityLabels; _i < _a.length; _i++) {
                        var e = _a[_i];
                        promises.push(mssql_service_1.MSSql.SaveUtteranceEntity(vcc, newUtteranceId, e.entityName, e.start, e.end, e.startTokenIndex, e.endTokenIndex));
                    }
                    return Promise.all(promises);
                }
            }
            return results;
        })
            .then(function (_) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (createUtteranceResult ? 'OK' : 'Utterance save failed'), data: createUtteranceResult });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[UtteranceService::create_utterance] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/create_utterance_underlined', function (req, res, next) {
        logger.info('[UtteranceService::create_utterance_underlined] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newUtteranceData = (req.body || {});
        newUtteranceData.createdDate = nowUtc;
        newUtteranceData.createdByUserId = userId;
        newUtteranceData.lastModifiedDate = nowUtc;
        newUtteranceData.lastModifiedByUserId = userId;
        logger.info('newUtteranceDTO: ', newUtteranceData);
        promises_shared_1.DummyPromise()
            .then(function (resultSaveUnderlined) {
            if (!resultSaveUnderlined)
                throw 'Se produjo un error al guardar la expresión';
            logger.info('[UtteranceService::create_utterance_underlined] resultSaveUnderlined: ', resultSaveUnderlined);
            return mssql_service_1.MSSql.SaveUnderlined(vcc, newUtteranceData.appId, newUtteranceData.intentId, newUtteranceData.audioId, newUtteranceData.section, newUtteranceData.part, newUtteranceData.speaker, newUtteranceData.anchorOffset, newUtteranceData.focusOffset, newUtteranceData.currentTime, newUtteranceData.endCurrentTime, newUtteranceData.utteranceId, newUtteranceData.textContent, newUtteranceData.createdDate, newUtteranceData.createdByUserId, newUtteranceData.lastModifiedDate, newUtteranceData.lastModifiedByUserId);
        }).then(function (resultSaveUtteranceUnderlined) {
            logger.info('[UtteranceService::create_utterance_underlined] resultSaveUtteranceUnderlined: ', resultSaveUtteranceUnderlined);
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSaveUtteranceUnderlined ? 'OK' : 'Utterance save failed'), data: resultSaveUtteranceUnderlined });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[UtteranceService::create_utterance_underlined] Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/save_utterance', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newUtterance = (req.body || {});
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return Promise.resolve(true);
        }).then(function (resultSaveUtterance) {
            if (!resultSaveUtterance)
                throw 'Se produjo un error al guardar la expresión';
            return Promise.resolve(true);
        }).then(function (resultSaveUtterance) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveUtterance, description: (resultSaveUtterance ? 'OK' : 'Utterance save failed'), data: resultSaveUtterance });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/utterance/delete_utterance', function (req, res, next) {
        logger.info('[UtteranceService::delete_utterance] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var utterance = req.body.utterance;
        var utteranceId = utterance.id;
        var intentId = utterance.intentLabel;
        var appId = utterance.appId;
        var nlu = null;
        var deleteUtteranceResult = null;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!appId)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, appId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_NOT_FOUND';
            nlu = result;
            if (nlu.isSystemApp) {
                return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
            }
            else {
                return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
            }
        })
            .then(function (result) {
            cognitiveService = result;
            if (cognitiveService.GetCognitiveEngineName() === 'RASA')
                utteranceId = utterance.text;
            if (!utterance)
                throw 'Missing utterance input parameter';
            return cognitiveService.DeleteAppUtterance(azure.subscriptionRegion, azure.subscriptionKey, appId, intentId, utteranceId);
        }).then(function (resultDeleteUtterance) {
            if (!resultDeleteUtterance)
                throw 'Se produjo un error al eliminar la expresión';
            var promises = [mssql_service_1.MSSql.DeleteUtterance(vcc, appId, intentId, utteranceId)];
            deleteUtteranceResult = resultDeleteUtterance;
            nlu.needTrain = true;
            nlu.lastModifiedDate = nowUtc;
            nlu.lastModifiedByUserId = userId;
            promises.push(nlu_service_1.SaveNLU(vcc, nlu));
            return Promise.all(promises);
        }).then(function (resultDeleteUtterance) {
            if (res.headersSent)
                return;
            var status = !resultDeleteUtterance.includes(false);
            res.json({ status: status, description: (status ? 'OK' : 'Utterance delete failed'), data: resultDeleteUtterance });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[UtteranceService::delete_utterance] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.UtteranceService = UtteranceService;
function CreateSystemAppIfNotExists(cognitiveService, vcc, nlu) {
    var arrAppId = nlu.appId.split('::');
    var systemAppType = arrAppId[1].split('-')[0];
    var isCreated = false;
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        return cognitiveService.GetAppsList(null, null);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_GET_INTENTS_FAILED';
        if (!result.find(function (n) { return n.id === nlu.appId; })) {
            return cognitiveService.AddApp(null, null, nlu);
        }
        else {
            isCreated = true;
            return Promise.resolve(true);
        }
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_CREATE_SYSTEM_MODEL_FAILED';
        if (!isCreated) {
            return cognitiveService.AddIntent(null, null, nlu.appId, 'Agent');
        }
        else {
            return Promise.resolve(true);
        }
    })
        .then(function (result) {
        if (result)
            throw 'SERVER_ERROR_CREATE_SYSTEM_MODEL_INTENT_FAILED';
        return Promise.resolve(true);
    })
        .catch(function (err) {
        var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('[UtteranceService::CreateDiarizationAppIfNotExists] Error: ' + errorMsg);
        return Promise.resolve(false);
    });
}
function GetIntent(vcc, name) {
    var cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(function () {
        return mssql_service_1.MSSql.GetIntent(vcc, name);
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('Error al ejecutar GetIntent: ' + errMsg);
    });
}
exports.GetIntent = GetIntent;
function GetUtterances(vcc, appId, intentId, findUtteranceEntities) {
    var utterances = [];
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        return mssql_service_1.MSSql.GetUtterances(vcc, appId, intentId);
    })
        .then(function (result) {
        utterances = result;
        var promiseArray = [];
        if (findUtteranceEntities) {
            result.map(function (r) {
                promiseArray.push(mssql_service_1.MSSql.GetUtteranceEntities(vcc, r.id));
            });
        }
        else {
            promiseArray.push(Promise.resolve([]));
        }
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        utterances.filter(function (u) { return u && u.text; }).map(function (u, i) {
            var tokenizedText = u.text.split(/(.+?)([\s,.!?;:)([\]]+)/);
            tokenizedText = tokenizedText.filter(function (tt) { return tt.trim().length > 0; });
            u.entityLabels = result[i] ? result[i] : [];
            u.entityPredictions = [];
            u.intentLabel = u.intentName;
            u.intentPredictions = [
                {
                    name: u.intentName.trim(),
                    score: null
                }
            ],
                u.tokenizedText = tokenizedText;
        });
        return Promise.resolve(utterances.filter(function (u) { return u && u.text; }));
    })
        .catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('Error on:', errMsg);
        return Promise.resolve(null);
    });
}
exports.GetUtterances = GetUtterances;
function GetSaveUtterances(nlu) {
    var cognitiveService;
    var implUtterances = [];
    var dbUtterances = [];
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        if (nlu.cognitiveEngine) {
            return cognitive_analysis_implementation_1.GetCognitiveService(null, nlu.cognitiveEngine);
        }
        else {
            return cognitive_analysis_implementation_1.GetCognitiveService(nlu.vcc, null);
        }
    })
        .then(function (result) {
        cognitiveService = result;
        if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
            return mssql_service_1.MSSql.GetAzureSubscriptionKeys(nlu.vcc);
        }
        else {
            return Promise.resolve(null);
        }
    })
        .then(function (result) {
        if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
            if (result && result.find(function (r) { return r.isDemo; })) {
                return cognitiveService.GetUtterancesList(result.find(function (r) { return r.isDemo; }) ? result.find(function (r) { return r.isDemo; }).subscriptionRegion : null, result.find(function (r) { return r.isDemo; }) ? result.find(function (r) { return r.isDemo; }).subscriptionKey : null, nlu.appId);
            }
            else {
                throw 'SERVER_ERROR_NOT_FOUND_AZURE_CREDENTIALS';
            }
        }
        else {
            return cognitiveService.GetUtterancesFromMD(nlu.appId);
        }
    })
        .then(function (utterances) {
        implUtterances = utterances;
        return GetUtterances(nlu.vcc, nlu.appId, null, true);
    })
        .then(function (utterances) {
        dbUtterances = utterances;
        var promiseArray = [];
        implUtterances.map(function (iu) {
            if (!dbUtterances.find(function (du) { return du.id === iu.id; }) && iu.intentLabel !== 'None') {
                logger.debug('[UtteranceService::GetSaveUtterances] Saving:', iu.text);
                promiseArray.push(mssql_service_1.MSSql.SaveUtterance(nlu.vcc, nlu.appId, iu.intentLabel, iu.id, iu.text, 'system'));
                if (iu.entityLabels) {
                    iu.entityLabels.map(function (el) {
                        if (!el.start) {
                            var utteranceText_1 = '';
                            iu.tokenizedText.map(function (tt, i) {
                                if (i > 0
                                    &&
                                        iu.text.startsWith(utteranceText_1 + ' '))
                                    utteranceText_1 += ' ';
                                if (i === el.startTokenIndex)
                                    el.start = utteranceText_1.length;
                                utteranceText_1 += tt;
                                if (i === el.endTokenIndex)
                                    el.end = utteranceText_1.length - 1;
                            });
                        }
                        promiseArray.push(mssql_service_1.MSSql.SaveUtteranceEntity(nlu.vcc, iu.id, el.entityName, el.start ? el.start : null, el.end ? el.end : null, el.startTokenIndex, el.endTokenIndex));
                    });
                }
            }
        });
        dbUtterances.map(function (du) {
            if (!implUtterances.find(function (iu) { return iu.id === du.id; })) {
                promiseArray.push(mssql_service_1.MSSql.DeleteUtterance(nlu.vcc, nlu.appId, du.intentId, du.Id));
            }
        });
        return Promise.resolve(promiseArray);
    })
        .then(function (result) {
    })
        .then(function (result) {
        nlu.lastModifiedByUserId = 'system';
        nlu.lastModifiedDate = moment().utc().toDate();
        return nlu_service_1.SaveNLU(nlu.vcc, nlu);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (error) {
        logger.error('[UtteranceService::GetSaveUtterances] Error on:', error);
        if (typeof error === 'string' && error === 'SERVER_ERROR_NOT_FOUND_AZURE_CREDENTIALS') {
            nlu.lastModifiedByUserId = 'system';
            nlu.lastModifiedDate = moment().utc().toDate();
            return nlu_service_1.SaveNLU(nlu.vcc, nlu);
        }
        else {
            if (error.message) {
                if (error.message.includes("Cannot find an application with the ID " + nlu.appId + ".")) {
                    return mssql_service_1.MSSql.DeleteNLU(nlu.vcc, nlu.name);
                }
                else if (error.message.includes("The user is not the owner")) {
                    return mssql_service_1.MSSql.DeleteNLU(nlu.vcc, nlu.name);
                }
                else if (error.message.includes("wrong API")) {
                    return mssql_service_1.MSSql.DeleteNLU(nlu.vcc, nlu.name);
                }
                else {
                    return Promise.resolve(false);
                }
            }
            else {
                return Promise.resolve(false);
            }
        }
    });
}
exports.GetSaveUtterances = GetSaveUtterances;
