"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var rp = require("request-promise-native");
var log4js = require("log4js");
var integration_config_1 = require("../config/integration.config");
var loggerAllegro = log4js.getLogger('ServerScripts');
function Login(user, password) {
    loggerAllegro.info('Login por API i6', JSON.stringify({ user: user, password: password }));
    var data = { user: user, password: password };
    var postOptions = {
        uri: 'http' + '://' + integration_config_1.DNS + '/inconcert/api/login/',
        method: 'POST',
        body: data,
        headers: {
            'Content-Type': 'application/json'
        },
        json: true,
        strictSSL: false
    };
    return rp(postOptions)
        .then(function (response) {
        var status = (response && response.status) ? true : false;
        var message = (response && response) ? response : '';
        return Promise.resolve({ status: status, accessToken: message.token });
    })
        .catch(function (err) {
        loggerAllegro.error('Login: Error ' + err.message);
        return Promise.resolve({ status: false, body: err.message });
    });
}
exports.Login = Login;
function SearchUsers(accessToken, request) {
    loggerAllegro.info('Request (inConcert): Listado de usuarios por API i6');
    var postOptions = {
        uri: 'http' + '://' + integration_config_1.DNS + '/inconcert/api/users/search/',
        method: 'POST',
        body: request,
        headers: {
            'Content-Type': 'application/json',
            'Authorization': accessToken
        },
        json: true,
        strictSSL: false,
        resolveWithFullResponse: false
    };
    return rp(postOptions)
        .then(function (response) {
        return Promise.resolve(response);
    })
        .catch(function (err) {
        loggerAllegro.error('SearchUsers: Error ' + err.message);
        return Promise.resolve({ status: false, body: err.message });
    });
}
exports.SearchUsers = SearchUsers;
function SaveUserByAPIi6(accessToken, user) {
    loggerAllegro.info('Request (inConcert): Creación de usuario por API i6');
    var postOptions = {
        uri: 'http' + '://' + integration_config_1.DNS + '/inconcert/api/users/save/',
        method: 'POST',
        body: user,
        headers: {
            'Content-Type': 'application/json',
            'Authorization': accessToken
        },
        json: true,
        strictSSL: false,
        resolveWithFullResponse: false
    };
    return rp(postOptions)
        .then(function (response) {
        return Promise.resolve(response);
    })
        .catch(function (err) {
        loggerAllegro.error('SaveUserByAPIi6: Error ' + err.message);
        return Promise.resolve({ status: false, body: err.message });
    });
}
exports.SaveUserByAPIi6 = SaveUserByAPIi6;
function ApiI6SearchUsers(vcc, campaignId) {
    loggerAllegro.info('[httpRequestOCC::ApiI6SearchUsers] Getting users');
    return this.Login(integration_config_1.USER_API_I6 + '@' + vcc, integration_config_1.PASS_API_I6)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var postOptions = {
            uri: 'http' + '://' + integration_config_1.DNS + '/inconcert/api/users/search/',
            method: 'POST',
            body: {
                query: { Application: "interaction" }
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            json: true,
            strictSSL: false,
            resolveWithFullResponse: false
        };
        return rp(postOptions);
    })
        .then(function (response) {
        if (response && response.status === false)
            throw 'SERVER_ERROR_GET_API_I6_AGENTS_FAILED';
        var agents = [];
        response['rows'].map(function (r) {
            agents.push({
                vcc: vcc,
                campaignId: null,
                userId: r.Id,
                userName: (r.Name ? r.Name.trim() + ' ' : '') + (r.FirstSurname ? r.FirstSurname.trim() : '')
            });
        });
        return Promise.resolve({ status: true, body: agents });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[httpRequestOCC::ApiI6SearchUsers] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6SearchUsers = ApiI6SearchUsers;
function ApiI6SearchCampaigns(vcc) {
    loggerAllegro.info('[httpRequestOCC::ApiI6SearchCampaigns] Getting campaigns');
    return this.Login(integration_config_1.USER_API_I6 + '@' + vcc, integration_config_1.PASS_API_I6)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var postOptions = {
            uri: 'http' + '://' + integration_config_1.DNS + '/inconcert/api/campaigns/search/',
            method: 'POST',
            body: {
                columns: ["Id", "Name", "Description", "dispositionCodes"]
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            json: true,
            strictSSL: false,
            resolveWithFullResponse: false
        };
        return rp(postOptions);
    })
        .then(function (response) {
        if (response && response.status === false)
            throw 'SERVER_ERROR_GET_API_I6_CAMPAIGNS_FAILED';
        var result;
        result = response['rows'].map(function (r) {
            var campaign = r.Id.split('@');
            var dispositionCodes = [];
            if (r.dispositionCodes) {
                r.dispositionCodes.map(function (dc) {
                    if (dc.childs) {
                        dc.childs.map(function (c1) {
                            if (c1.childs) {
                                c1.childs.map(function (c2) {
                                    if (c2.childs) {
                                        c2.childs.map(function (c3) {
                                            dispositionCodes.push({ parent: dc.name, child: c1.name, child2: c2.name, child3: c3.name });
                                        });
                                    }
                                    else {
                                        dispositionCodes.push({ parent: dc.name, child: c1.name, child2: c2.name, child3: null });
                                    }
                                });
                            }
                            else {
                                dispositionCodes.push({ parent: dc.name, child: c1.name, child2: null, child3: null });
                            }
                        });
                    }
                    else {
                        dispositionCodes.push({ parent: dc.name, child: null, child2: null, child3: null });
                    }
                });
            }
            return { id: campaign[0], description: r.Description, dispositionCodes: dispositionCodes };
        });
        return Promise.resolve({ status: true, body: result });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[httpRequestOCC::ApiI6SearchCampaigns] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6SearchCampaigns = ApiI6SearchCampaigns;
function ApiI6GetCampaignDispositions(vcc) {
    loggerAllegro.info('[httpRequestOCC::ApiI6GetCampaignDispositions] Getting campaigns');
    return this.Login(integration_config_1.USER_API_I6 + '@' + vcc, integration_config_1.PASS_API_I6)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var postOptions = {
            uri: 'http' + '://' + integration_config_1.DNS + '/inconcert/api/campaigns/get_dispositions/' + null,
            method: 'GET',
            body: {
                columns: ["Id", "Name", "Description", "dispositionCodes"]
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            json: true,
            strictSSL: false,
            resolveWithFullResponse: false
        };
        return rp(postOptions);
    })
        .then(function (response) {
        return Promise.resolve({ status: true, body: response });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[httpRequestOCC::ApiI6GetCampaignDispositions] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6GetCampaignDispositions = ApiI6GetCampaignDispositions;
function ApiI6ConfiguredChannels(vcc, campaign) {
    loggerAllegro.info('[httpRequestOCC::ApiI6ConfiguredChannels] Getting campaigns');
    return this.Login(integration_config_1.USER_API_I6 + '@' + vcc, integration_config_1.PASS_API_I6)
        .then(function (result) {
        if (!result.accessToken)
            throw 'API_I6_LOGIN_FAILED';
        var postOptions = {
            uri: 'http' + '://' + integration_config_1.DNS + '/inconcert/api/campaigns/get_configured_channels/',
            method: 'POST',
            body: {
                campaign: campaign
            },
            headers: {
                'Content-Type': 'application/json',
                'Authorization': result.accessToken
            },
            json: true,
            strictSSL: false,
            resolveWithFullResponse: false
        };
        return rp(postOptions);
    })
        .then(function (response) {
        if ((!response) || (response && response.status === false))
            throw 'SERVER_ERROR_GET_API_I6_INTERACTION_TYPES_FAILED';
        var keys = Object.keys(response.channels);
        var result;
        result = keys.map(function (k) {
            return { intance: vcc, id: k, description: k };
        });
        return Promise.resolve({ status: true, body: result });
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : err.message;
        loggerAllegro.error('[httpRequestOCC::ApiI6ConfiguredChannels] Error ' + errorMessage);
        return Promise.resolve({ status: false, body: errorMessage });
    });
}
exports.ApiI6ConfiguredChannels = ApiI6ConfiguredChannels;
