"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var cors = require("cors");
function InitHeaders(app) {
    app.use(FrameGuard);
    app.use(HidePoweredBy);
    app.use(XssFilter);
    app.use(IeNoOpen);
    var corsOptions = {
        origin: true,
        credentials: true,
        methods: ['GET', 'POST']
    };
    app.all('/api|auth|public|popup|builder|payments|distributed|static|template|assets|favicon/*', cors(corsOptions));
}
exports.InitHeaders = InitHeaders;
function FrameGuard(req, res, next) {
    if (req.url.match('^\/(builder|payments|popup)\/') || req.url.startsWith('/public/tracking/frame.html') || req.url.startsWith('/public/preview_template_edit/'))
        return next();
    res.setHeader('X-Frame-Options', 'SAMEORIGIN');
    return next();
}
exports.FrameGuard = FrameGuard;
function HidePoweredBy(req, res, next) {
    res.removeHeader('X-Powered-By');
    return next();
}
exports.HidePoweredBy = HidePoweredBy;
function XssFilter(req, res, next) {
    var matches = /msie\s*(\d+)/i.exec(req.headers['user-agent']);
    var value;
    if (!matches || (parseFloat(matches[1]) >= 9))
        value = '1; mode=block';
    else
        value = '0';
    res.setHeader('X-XSS-Protection', value);
    return next();
}
exports.XssFilter = XssFilter;
function IeNoOpen(req, res, next) {
    res.setHeader('X-Download-Options', 'noopen');
    return next();
}
exports.IeNoOpen = IeNoOpen;
function HstsGuard(req, res, next) {
    var maxAge = '0';
    if (res.locals && res.locals.domain && res.locals.domain.hasHttps)
        maxAge = '31536000';
    res.setHeader('Strict-Transport-Security', 'max-age=' + maxAge);
    return next();
}
exports.HstsGuard = HstsGuard;
