"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var jwt = require("jsonwebtoken");
var log4js = require("log4js");
var jwt_config_1 = require("../config/jwt.config");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var user_service_1 = require("../services/user.service");
var role_service_1 = require("../services/role.service");
var logger = log4js.getLogger('ServerScripts');
function ValidateUserSession(req, res, next) {
    if (!req.url.match('^\/(api|auth|builder)\/'))
        return next();
    var vcc = res.locals.vcc;
    var requestSession;
    var decodedRequestSession;
    var decodedStoredSession;
    var decodedUserId;
    var isLoginRequest = !!req.url.match('^\/auth\/login');
    var isLogoutRequest = !!req.url.match('^\/auth\/logout');
    var isPublicResource = !!req.url.match('^\/api\/public');
    if (isLoginRequest || isPublicResource)
        return next();
    var userId;
    var user;
    var roleId;
    var role;
    promises_shared_1.DummyPromise()
        .then(function (result) {
        if (req.url.match('^\/builder\/')) {
            requestSession = req.cookies['Session'];
        }
        else {
            requestSession = (req.headers['authorization'] || '').replace('bearer ', '');
        }
        if (!requestSession)
            throw 'No se recibió token de autenticación.';
        res.locals.session = requestSession;
        if (res.locals.session && !req.cookies['Session'] && !isLogoutRequest) {
            res.cookie('Session', res.locals.session, { httpOnly: true });
        }
        userId = GetUserIdFromSession(res.locals.session);
        if (!userId)
            throw 'No se pudo obtener usuario asociado a la sesión';
        res.locals.userId = userId;
        return user_service_1.GetUser(vcc, userId);
    }).then(function (resultUser) {
        if (!resultUser)
            throw 'No se pudo obtener la información del usuario \'' + userId + '\'.';
        user = resultUser;
        roleId = user.role;
        res.locals.user = user;
        return role_service_1.GetRole(vcc, roleId);
    }).then(function (resultRole) {
        if (!resultRole)
            throw 'No se pudo obtener la información del rol \'' + roleId + '\' del usuario \'' + userId + '\'.';
        role = resultRole;
        res.locals.userRole = resultRole;
        try {
            decodedRequestSession = jwt.verify(requestSession, jwt_config_1.JWT_SECRET);
        }
        catch (e) {
            decodedRequestSession = null;
        }
        if (!decodedRequestSession)
            throw 'No se pudo decodificar el token recibido';
        decodedUserId = (decodedRequestSession.userId || '').toLowerCase();
        return redis_service_1.Redis.GetSession(vcc, decodedUserId);
    }).then(function (storedSession) {
        if (storedSession === requestSession)
            return true;
        if (!storedSession && decodedRequestSession)
            return redis_service_1.Redis.SaveSession(vcc, decodedUserId, requestSession);
        try {
            decodedStoredSession = jwt.verify(storedSession, jwt_config_1.JWT_SECRET);
        }
        catch (e) {
            decodedStoredSession = null;
        }
        if (decodedRequestSession && decodedStoredSession && decodedRequestSession.iat >= decodedStoredSession.iat)
            return redis_service_1.Redis.SaveSession(vcc, decodedUserId, requestSession);
        logger.warn('Sesión inválida: El token recibido no es válido.');
        if (res.headersSent)
            return;
        if (req.url.match('^\/builder\/')) {
            res.status(401).type('txt').send('Invalid session');
        }
        else {
            res.status(401).json({ description: 'Invalid session token', error: 'Invalid session' });
        }
        return false;
    }).then(function (result) {
        if (result)
            return next();
    }).catch(function (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('Error al procesar \'' + req.path + '\': ' + errMsg);
        if (res.headersSent)
            return;
        if (req.url.match('^\/builder\/')) {
            res.status(401).type('txt').send('Invalid session');
        }
        else {
            res.status(401).json({ description: 'Invalid session token', error: 'Invalid session' });
        }
    });
}
exports.ValidateUserSession = ValidateUserSession;
function GetUserIdFromSession(session) {
    try {
        if (session) {
            var decoded = jwt.verify(session, jwt_config_1.JWT_SECRET);
            if (decoded.userId)
                return decoded.userId.toLowerCase();
        }
    }
    catch (err) {
        var errMsg = (typeof err === 'string' ? err : err.message || err.description || '');
        logger.error('Error el decodificar datos de la sesion: ' + errMsg);
    }
    return null;
}
exports.GetUserIdFromSession = GetUserIdFromSession;
